/*
 * EintragEntfernenAktion.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JTable;

import eu.gronos.kostenrechner.interfaces.RowHandler;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.forderungen.ForderungsStaffelungPanel;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;

/**
 * Die Action soll für die Beteiligtentabellen und die Gebührentabellen
 * einsetzbar sein und kann JPopupMenus hinzugefügt werden.
 * {@link KeyEvent#VK_E}
 *
 * @author Peter Schuster (setrok)
 * @date 5 May 2019
 *
 */
public class EintragEntfernenAktion extends BeschriebeneAktion {
	private static final long serialVersionUID = 8001075205760187469L;

	/**
	 * Konstruktor: TODO (kommentieren)
	 * 
	 * @param name
	 * @param shortDescription
	 * @param mnemonic
	 * @param actionCommandKey
	 */
	public EintragEntfernenAktion(String name, String shortDescription, int mnemonic, String actionCommandKey) {
		super(name, shortDescription, mnemonic, actionCommandKey);
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		Object o = ae.getSource();
		if (!(o instanceof JMenuItem)) {
			return;
		}
		// Herausfinden, welche Tabelle ruft
		JPopupMenu popup = (JPopupMenu) ((JMenuItem) o).getParent();
		if (popup.getLabel() == TeilklageruecknahmePanel.VORHER_POPUPLABEL)
			entferneTabellenEintrag(TeilklageruecknahmePanel.getInstance().tabVorher,
					TeilklageruecknahmePanel.getInstance().getGttmVorher());
		if (popup.getLabel() == TeilklageruecknahmePanel.NACHHER_POPUPLABEL)
			entferneTabellenEintrag(TeilklageruecknahmePanel.getInstance().tabNachher,
					TeilklageruecknahmePanel.getInstance().getGttmNachher());
		if (popup.getLabel() == StreitgenossenPanel.BEKLAGTE_POPUPLABEL)
			StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(
					StreitgenossenPanel.getInstance().tabBeklagte, StreitgenossenPanel.getInstance().btmBeklagte);
		if (popup.getLabel() == StreitgenossenPanel.KLAEGER_POPUPLABEL)
			StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(
					StreitgenossenPanel.getInstance().tabKlaeger, StreitgenossenPanel.getInstance().btmKlaeger);
		if (popup.getLabel() == StreitgenossenPanel.DRITTWIDERBEKLAGTE_POPUPLABEL)
			StreitgenossenPanel.getInstance().entferneBeteiligtenEintrag(
					StreitgenossenPanel.getInstance().tabDrittwiderbeklagte,
					StreitgenossenPanel.getInstance().btmDrittwiderbeklagte);
		if (popup.getLabel() == ForderungsStaffelungPanel.FORDERUNGEN_POPUPLABEL)
			entferneTabellenEintrag(ForderungsStaffelungPanel.getInstance().tbForderungen,
					ForderungsStaffelungPanel.getInstance().ftm);
		if (popup.getLabel() == GebuehrenBerechnungPanel.RECHNER_POPUPLABEL)
			entferneTabellenEintrag(GebuehrenBerechnungPanel.getInstance().tabGebuehrenTatbestaende,
					GebuehrenBerechnungPanel.getInstance().gttmRechner);
	}

	/**
	 * Entfernt eine Zeile aus einer Gebühren- oder Forderungs-Tabelle.
	 * 
	 * @param table die JTable, aus der die Zeile gelöscht werden soll
	 * @param model das RowHandler-implementierende TableModel, aus der die Zeile
	 *              gelöscht werden soll
	 */
	public void entferneTabellenEintrag(JTable table, RowHandler<?> model) {
		if (table == null || model == null || table.getSelectedRow() < 0)
			return;
		int selectedRow = table.getSelectedRow();
		model.removeRow(selectedRow);
	}
}