package eu.gronos.kostenrechner.controller.baumbach;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.model.baumbach.BaumbachGesamtschuldnerschaft;
import eu.gronos.kostenrechner.view.baumbach.GesamtschuldnerDialog;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Die {@link Action} fügt eine gesamtschuldnerische Verurteilung auf
 * Kläger-/Drittwiderbeklagtenseite hinzu.
 *
 * @author Peter Schuster (setrok)
 * @date 10.11.2018
 *
 */
public class StreitgenossenGesamtschuldnerDwbkAction extends BeschriebeneAktion {
	private final StreitgenossenPanel panel;
	private static final long serialVersionUID = -6637467913455344657L;

	/**
	 * Konstruktor.
	 * 
	 * @param panel            {@link StreitgenossenPanel}
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 */
	public StreitgenossenGesamtschuldnerDwbkAction(StreitgenossenPanel panel, String name, String shortDescription,
			int mnemonic, String actionCommandKey) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.panel = panel;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		try {
			GesamtschuldnerDialog gesamtschuldnerDialog = new GesamtschuldnerDialog(panel.getKostenrechner(),
					panel.btmKlaeger.getAllValues(), panel.btmDrittwiderbeklagte.getAllValues());
			BaumbachGesamtschuldnerschaft bt = gesamtschuldnerDialog.showDialog();
			if (bt != null)
				panel.btmDrittwiderbeklagte.addRow(bt);
		} catch (IllegalArgumentException iae) {
			// Meldung, wenn noch keine Beteiligten in Tabelle
			new FehlerDialog(StreitgenossenPanel.FEHLER_GESAMTSCHULDNERSCHAFT, iae.getLocalizedMessage(), iae)
					.showDialog();
		}
	}
}