/**
 * SpeichereTenorAction.java
 * eu.gronos.kostenrechner.view.result (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.files;

import java.awt.event.ActionEvent;
import java.io.File;

import javax.swing.Action;
import javax.swing.KeyStroke;
import javax.xml.bind.JAXBException;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.system.BooleanConfigOptionHorcher;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Die {@link Action} speichert den Tenortext im Rich Text Format (RTF) über
 * {@link RtfDocHelper#speichereRTF()}.
 * 
 * Wenn {@link Kostenrechner#debug} gesetzt ist (TODO
 * {@link BooleanConfigOptionHorcher} bauen TODO), wird über
 * {@link TenorXmlDatei} auch der {@link TenorDatenContainer} gespeichert.
 *
 * @author Peter Schuster (setrok)
 * @date 22 Nov 2018
 *
 */
public class SpeichereRtfAction extends BeschriebeneAktion/* AbstractAction */ {
	private final TenorDialog dialog;
	private TenorDatenContainer tenorDatenContainer;
	private static final long serialVersionUID = 1162624762983290295L;

	/**
	 * Konstruktor.
	 * 
	 * @param dialog              Verweis auf {@link TenorDialog}
	 * @param tenorDatenContainer ein {@link TenorDatenContainer}
	 * @param name                {@link Action#NAME}
	 * @param shortDescription    {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic            {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey    {@link Action#ACTION_COMMAND_KEY}
	 * @param acceleratorKey      {@link Action#ACCELERATOR_KEY}
	 */
	public SpeichereRtfAction(TenorDialog dialog, TenorDatenContainer tenorDatenContainer, String name,
			String shortDescription, int mnemonic, String actionCommandKey, KeyStroke acceleratorKey) {
		super(name, shortDescription, mnemonic, actionCommandKey, acceleratorKey);
		this.dialog = dialog;
		this.tenorDatenContainer = tenorDatenContainer;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		File datei = this.dialog.getDocHelper().speichereRTF();
		if (datei != null && dialog.isXmlSelected()) {
			datei = new File(datei.getAbsoluteFile() + TenorXmlDatei.DATEIENDUNG_GANZ);
			try {
				new TenorXmlDatei().speicherXml(tenorDatenContainer, datei);
			} catch (JAXBException je) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, "Fehler beim Umwandeln in XML", je);
			}
		}
	}
}