/**
 * ForderungsStaffelungTenorAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.forderungen;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainer2GUI;
import eu.gronos.kostenrechner.logic.forderungen.AngriffsVerteidigungsmittelStaffelung;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.AktionenJMenu;
import eu.gronos.kostenrechner.view.forderungen.ForderungsStaffelungPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;
import eu.gronos.kostenrechner.view.result.TenorDialog;

/**
 * Die AbstractAction ist eine {@link Action} zum Ermitteln des Tenors auf
 * Registerkarte Hilfsaufrechnung (3) über
 * {@link AngriffsVerteidigungsmittelStaffelung}
 * 
 * @author Peter Schuster (setrok)
 * @date 04.11.2018
 *
 */
public class ForderungsStaffelungTenorAction extends BeschriebeneAktion {
	private final ForderungsStaffelungPanel panel;
	private static final long serialVersionUID = 2748367225462023438L;

	/**
	 * Konstruktor.
	 * 
	 * @param panel            Verweis auf {@link ForderungsStaffelungPanel}
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 */
	public ForderungsStaffelungTenorAction(ForderungsStaffelungPanel panel, String name, String shortDescription,
			int mnemonic, String actionCommandKey) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.panel = panel;
	}

	@Override
	public void actionPerformed(ActionEvent ae) {
		if (this.panel.cbAngriffsKlaeger == null || this.panel.cbVerteidigungsBeklagter == null
				|| this.panel.ftm == null || AktionenJMenu.getInstance().mi92 == null)
			return;
		try {
			final VerfahrensDatenContainer container = new VerfahrensDatenContainer2GUI().leseAlleWertefuerContainer();
			new TenorDialog(this.panel.getKostenrechner(),
					new AngriffsVerteidigungsmittelStaffelung(container.allgemein, container.staffelung)
							.erzeugeContainer(),
					container).showDialog();
		} catch (IllegalArgumentException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
	}
}