/**
 * TeilklageGebuehrVorherAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;
import javax.swing.JComponent;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.interfaces.PanelTableModelHandler;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrHinzufuegenDialog;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;

/**
 * Der AbstractAction mit dem Namen neueGebuehrVorher fügt einen neuen
 * Gebührentatbestand in die Tabelle der vor der Streitwertreduktion
 * entstandenen, in die Tabelle für die nach Gebühren ein, die nach der
 * Streitwertreduktion entstanden sind oder in die Tabelle für die Gebühren zur
 * freien Berechnung ein.
 * 
 * {@link KeyEvent#VK_N} und {@link KeyEvent#VK_R}
 *
 */
public class GebuehrNeuAction extends BeschriebeneAktion {
	private final PanelTableModelHandler<GebuehrenTatbestandTableModel> panel;
	private static final long serialVersionUID = 7170038539836176410L;
	private final int index;
	private final GebuehrenTatbestand[] alleTatbestaende;

	/**
	 * Der Standardkonstruktor fordert auch ein Array aller
	 * {@link GebuehrenTatbestand}e
	 * 
	 * @param panel            ein {@link PanelTableModelHandler}
	 * @param name             the name ({@link javax.swing.Action#NAME}) for the
	 *                         action; a value of null is ignored
	 * @param shortDescription ({@link javax.swing.Action#SHORT_DESCRIPTION}) The
	 *                         key used for storing a short String description for
	 *                         the action, used for tooltip text.
	 * @param mnemonic         ({@link javax.swing.Action#MNEMONIC_KEY}) The key
	 *                         used for storing an Integer that corresponds to one
	 *                         of the KeyEvent key codes. The value is commonly used
	 *                         to specify a mnemonic.
	 *                         KeyEvent.getExtendedKeyCodeForChar('\u0444')) sets
	 *                         the mnemonic of myAction to Cyrillic letter "Ef".
	 * @param actionCommandKey ({@link javax.swing.Action#ACTION_COMMAND_KEY}) The
	 *                         key used to determine the command String for the
	 *                         ActionEvent that will be created when an Action is
	 *                         going to be notified as the result of residing in a
	 *                         Keymap associated with a {@link JComponent}.
	 * @param index            {@link TeilklageruecknahmePanel#TABLE_FTF_VORHER} oder
	 *                         {@link TeilklageruecknahmePanel#TABLE_FTF_NACHHER} oder
	 *                         {@link GebuehrenBerechnungPanel#TABLE_FTF_GEBUEHREN}
	 * @param alleTatbestaende ein Array aller {@link GebuehrenTatbestand}e
	 */
	public GebuehrNeuAction(PanelTableModelHandler<GebuehrenTatbestandTableModel> panel, String name, String shortDescription,
			int mnemonic, String actionCommandKey, int index, GebuehrenTatbestand[] alleTatbestaende) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.panel = panel;
		this.index = index;
		this.alleTatbestaende = alleTatbestaende;
	}

	/**
	 * Der weitere Konstruktor nimmt einfach alle Gebührentatbestände
	 * 
	 * @param panel            ein {@link PanelTableModelHandler}
	 * @param name             the name ({@link javax.swing.Action#NAME}) for the
	 *                         action; a value of null is ignored
	 * @param shortDescription ({@link javax.swing.Action#SHORT_DESCRIPTION}) The
	 *                         key used for storing a short String description for
	 *                         the action, used for tooltip text.
	 * @param mnemonic         ({@link javax.swing.Action#MNEMONIC_KEY}) The key
	 *                         used for storing an Integer that corresponds to one
	 *                         of the KeyEvent key codes. The value is commonly used
	 *                         to specify a mnemonic.
	 *                         KeyEvent.getExtendedKeyCodeForChar('\u0444')) sets
	 *                         the mnemonic of myAction to Cyrillic letter "Ef".
	 * @param actionCommandKey ({@link javax.swing.Action#ACTION_COMMAND_KEY}) The
	 *                         key used to determine the command String for the
	 *                         ActionEvent that will be created when an Action is
	 *                         going to be notified as the result of residing in a
	 *                         Keymap associated with a {@link JComponent}.
	 * @param index            {@link TeilklageruecknahmePanel#TABLE_FTF_VORHER} oder
	 *                         {@link TeilklageruecknahmePanel#TABLE_FTF_NACHHER} oder
	 *                         {@link GebuehrenBerechnungPanel#TABLE_FTF_GEBUEHREN}
	 */
	public GebuehrNeuAction(PanelTableModelHandler<GebuehrenTatbestandTableModel> panel, String name, String shortDescription,
			int mnemonic, String actionCommandKey, int index) {
		this(panel, name, shortDescription, mnemonic, actionCommandKey, index, createAlleGebuehrenTatbestaende());
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final String name = (String) getValue(Action.NAME);
		GebuehrenTatbestand gt = new GebuehrHinzufuegenDialog(Kostenrechner.getInstance(), // panel.getKostenrechner(),
				name, alleTatbestaende).showDialog();
		if (gt != null)
			panel.getTableModel(index).addRow(gt);
	}

	/**
	 * Die Methode dient dazu, alle {@link GebuehrenTatbestand}e für
	 * {@link GebuehrHinzufuegenDialog} zusammenzustellen, falls keine dem
	 * Konstruktor übergeben wurden.
	 * 
	 * @return die {@link GebuehrenTatbestand}e der {@link GerichtsGebuehrenTabelle}
	 *         und der {@link AnwaltsGebuehrenTabelle} als Array
	 */
	private static GebuehrenTatbestand[] createAlleGebuehrenTatbestaende() {
		return GebuehrenTabelle.mergeAllGebuehrenTatbestaende(GerichtsGebuehrenTabelle.getAllGebuehrenTatbestaende(),
				AnwaltsGebuehrenTabelle.getAllGebuehrenTatbestaende());
	}
}