/**
 * GebuehrenNormalAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.interfaces.GebuehrenTableTextfieldHandler;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;

/**
 * Die {@link Action} leert die Tabelle mit den Gebührentatbeständen des
 * Rechners und füllt sie mit der Konstellation und füllt sie mit der
 * Konstellation eines normalen erstinstanzlichen Verfahrens,
 * {@link KeyEvent#VK_UNDEFINED}, oder füllt sie mit der Konstellation Gebühren
 * bei Anerkenntnis, {@link KeyEvent#VK_G}, oder füllt sie mit der Konstellation
 * Gebühren bei Säumnis, {@link KeyEvent#VK_S}, oder füllt sie mit der
 * Konstellation und füllt sie mit der Konstellation eines normalen
 * erstinstanzlichen Verfahrens mit anschließender Berufung
 * {@link KeyEvent#VK_UNDEFINED}
 * 
 * @author Peter Schuster (setrok)
 * @date 04.11.2018
 */
public class GebuehrenSetzeRechnerTabelleAction extends BeschriebeneAktion {
	private final GebuehrenTableTextfieldHandler handler;
	private static final long serialVersionUID = 5567272748853539208L;
	private int key;
	public static final int GEBUEHREN_2INSTANZ = 104;
	public static final int GEBUEHREN_STANDARD = 103;
	public static final int GEBUEHREN_SAEUMNIS = 102;
	public static final int GEBUEHREN_ANERKENNTNIS = 101;

	/**
	 * Konstruktor.
	 * 
	 * @param handler          Verweis auf {@link GebuehrenBerechnungPanel}
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 * @param key              ein int,
	 *                         {@link GebuehrenSetzeRechnerTabelleAction#GEBUEHREN_ANERKENNTNIS},
	 *                         {@link GebuehrenSetzeRechnerTabelleAction#GEBUEHREN_SAEUMNIS},
	 *                         {@link GebuehrenSetzeRechnerTabelleAction#GEBUEHREN_STANDARD}
	 *                         oder
	 *                         {@link GebuehrenSetzeRechnerTabelleAction#GEBUEHREN_2INSTANZ}
	 */
	public GebuehrenSetzeRechnerTabelleAction(GebuehrenTableTextfieldHandler handler, String name, String shortDescription,
			int mnemonic, String actionCommandKey, int key) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.handler = handler;
		this.key = key;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		setzeGebuehrenRechnerTabelle(key);
		handler.requestFocusInWindow();
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle oder füllt sie mit
	 * üblicherweise anfallenden Gebühren, lässt aber die Listener in Ruhe.
	 * 
	 * @param key 0 = Tabelle leeren, {@link #GEBUEHREN_ANERKENNTNIS} = Tabelle mit
	 *            den bei einem Anerkenntnis üblicherweise anfallenden Gebühren
	 *            füllen, {@link #GEBUEHREN_SAEUMNIS} Tabelle mit den bei einer
	 *            Säumnis üblicherweise anfallenden Gebühren füllen
	 * 
	 */
	private void setzeGebuehrenRechnerTabelle(int key) {
		List<GebuehrenTatbestand> values;
		switch (key) {
		case GEBUEHREN_STANDARD:
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_ANERKENNTNIS:
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1211,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_SAEUMNIS:
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3105, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		case GEBUEHREN_2INSTANZ:
			values = new ArrayList<GebuehrenTatbestand>();
			values.addAll(Arrays.asList(new GebuehrenTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
					GerichtsGebuehrenTabelle.KV1220, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV3200, AnwaltsGebuehrenTabelle.VV3104,
					AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV7008_R, AnwaltsGebuehrenTabelle.VV3100,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002, AnwaltsGebuehrenTabelle.VV3200,
					AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV7002,
					AnwaltsGebuehrenTabelle.VV7008_R }));
			break;
		default:
			values = new ArrayList<GebuehrenTatbestand>();
			break;
		}
		GebuehrenTatbestandTableModel gttm = handler.getTableModel(GebuehrenBerechnungPanel.TABLE_GEBUEHREN);
		gttm.clear();
		gttm.addAll(values);
	}

//		panel.
//		panel.getKostenrechner().getTabbedPane().setSelectedIndex(3);

	// GebuehrenTatbestand[] values;
	// values = new GebuehrenTatbestand[]
	// values = new GebuehrenTatbestand[]
	// values = new GebuehrenTatbestand[]
	// values = new GebuehrenTatbestand[]
//		if (gttmRechner == null) {
//			gttmRechner = new GebuehrenTatbestandTableModel(values);
//			gttmRechner.addTableModelListener(aktualisierungsHorcher);
//			tabGebuehrenTatbestaende.setModel(gttmRechner);
//		} else {
//		}
	// gttmRechner.
	// gttmRechner.
}