/**
 * TeilklageMethodeAction.java
 * eu.gronos.kostenrechner.view.gebuehren (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;

import javax.swing.AbstractAction;
import javax.swing.AbstractButton;
import javax.swing.JComponent;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.logic.gebuehren.MehrkostenMethode;
import eu.gronos.kostenrechner.logic.gebuehren.QuotenMethode;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;

/**
 * Mit der {@link AbstractAction} lässt sich auswählen, ob der Tenor im Panel
 * {@link TeilklageruecknahmePanel} nach {@link QuotenMethode} oder
 * {@link MehrkostenMethode} berechnet wird.
 *
 * @author Peter Schuster (setrok)
 * @date 18 Feb 2019
 *
 */
public class TeilklageMethodeAction extends BeschriebeneAktion {

	private static final long serialVersionUID = 6046512398030660248L;
	private String nameSelected;
	private String nameUnselected;

	/**
	 * 
	 * @param nameSelected     Beschriftung für den {@link AbstractButton} im
	 *                         ausgewählten Zustand. The name
	 *                         ({@link javax.swing.Action#NAME}) for the action; a
	 *                         value of null is ignored
	 * @param nameUnselected   Beschriftung für den {@link AbstractButton} im
	 *                         unausgewählten Zustand
	 * @param shortDescription ({@link javax.swing.Action#SHORT_DESCRIPTION}) The
	 *                         key used for storing a short String description for
	 *                         the action, used for tooltip text.
	 * @param mnemonic         ({@link javax.swing.Action#MNEMONIC_KEY}) The key
	 *                         used for storing an Integer that corresponds to one
	 *                         of the KeyEvent key codes. The value is commonly used
	 *                         to specify a mnemonic.
	 *                         KeyEvent.getExtendedKeyCodeForChar('\u0444')) sets
	 *                         the mnemonic of myAction to Cyrillic letter "Ef".
	 * @param actionCommandKey ({@link javax.swing.Action#ACTION_COMMAND_KEY}) The
	 *                         key used to determine the command String for the
	 *                         ActionEvent that will be created when an Action is
	 *                         going to be notified as the result of residing in a
	 *                         Keymap associated with a {@link JComponent}.
	 */
	public TeilklageMethodeAction(String nameSelected, String nameUnselected, String shortDescription, int mnemonic,
			String actionCommandKey) {
		super(nameSelected, shortDescription, mnemonic, actionCommandKey);
		this.nameSelected = nameSelected;
		this.nameUnselected = nameUnselected;
	}

	/**
	 * Beim Klicken wird der Text des {@link AbstractButton} der Auswahl angepasst,
	 * ob {@link #nameSelected} oder {@link #nameUnselected} gesetzt werden soll.
	 * 
	 * @param e
	 * 
	 * @see java.awt.event.ActionListener#actionPerformed(java.awt.event.ActionEvent)
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();
		AbstractButton button;
		if (source instanceof AbstractButton) {
			button = (AbstractButton) source;
		} else {
			return;
		}
		setButtonText(button);
	}

	private void setButtonText(AbstractButton button) {
		if (button.isSelected()) {
			button.setText(nameSelected);
		} else {
			button.setText(nameUnselected);
		}
	}

}
