/**
 * TeiklageRuecknahmeTerminAction.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.gebuehren;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.interfaces.GebuehrenTableTextfieldHandler;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;

/**
 * Die {@link Action} leert die Tabelle mit den Gebührentatbeständen und füllt
 * sie mit der Konstellation Teilklagerücknahme vor Anfall der Terminsgebühr.
 * {@link KeyEvent#VK_F}
 */
public class TeilklageTerminAction extends BeschriebeneAktion {
	private final GebuehrenTableTextfieldHandler handler;
	private static final long serialVersionUID = -7826777167427931422L;

	/**
	 * Konstruktor.
	 * 
	 * @param handler          Verweis auf {@link TeilklageruecknahmePanel}
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 */
	public TeilklageTerminAction(GebuehrenTableTextfieldHandler handler, String name, String shortDescription,
			int mnemonic, String actionCommandKey) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.handler = handler;
	}

	/**
	 * Die Methode füllt die Tabellen mit den Tatbeständen zum Standardfall, wenn
	 * mit den Tatbeständen zum Standardfall gefüllt werden sollen.
	 */
	@Override
	public void actionPerformed(ActionEvent e) {
		GebuehrenTatbestandTableModel gttm = handler.getTableModel(TeilklageruecknahmePanel.TABLE_FTF_VORHER);

		gttm.clear();
		gttm.addAll(new ArrayList<GebuehrenTatbestand>(Arrays.asList(new GebuehrenSatzTatbestand[] {
				GerichtsGebuehrenTabelle.KV1210, AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100 })));

		gttm = handler.getTableModel(TeilklageruecknahmePanel.TABLE_FTF_NACHHER);
		gttm.clear();
		gttm.addAll(new ArrayList<GebuehrenTatbestand>(Arrays.asList(
				new GebuehrenSatzTatbestand[] { AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104 })));
		handler.requestFocusInWindow();
	}

	// boolean mitInhalt
//			if (mitInhalt) {
//				setGttmVorher(new GebuehrenTatbestandTableModel(new ArrayList<GebuehrenTatbestand>(
//						Arrays.asList(new GebuehrenSatzTatbestand[] { GerichtsGebuehrenTabelle.KV1210,
//								AnwaltsGebuehrenTabelle.VV3100, AnwaltsGebuehrenTabelle.VV3100 }))));
//				setGttmNachher(new GebuehrenTatbestandTableModel(
//						new ArrayList<GebuehrenTatbestand>(Arrays.asList(new GebuehrenSatzTatbestand[] {
//								AnwaltsGebuehrenTabelle.VV3104, AnwaltsGebuehrenTabelle.VV3104 }))));
//			} else {
//			}
//		Kostenrechner.getInstance().getTabbedPane().setSelectedIndex(0);
}