/*
 * FehlerHelper.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

/**
 * Klasse dient dazu die Message, LocalizedMessage und den StackTrace eines
 * Fehlers in einen StringBuffer zu schreiben.
 *
 * @author Peter Schuster (setrok)
 * @date 24 Apr 2019
 *
 */
public class FehlerHelper {

	private Throwable throwable;

	/**
	 * @param throwable ein {@link Throwable}, um die {@link StackTraceElement}s
	 *                  auszulesen
	 */
	public FehlerHelper(Throwable throwable) {
		this.throwable = throwable;
	}

	/**
	 * Die Methode dient dazu, die Message, {@link Throwable#getLocalizedMessage()}
	 * und den {@link Throwable#getStackTrace() eines Fehlers in einen
	 * {@link StringBuffer} zu schreiben.
	 * 
	 * @return {@link Throwable#getLocalizedMessage()} und
	 *         {@link StackTraceElement}s als {@link StringBuffer}
	 */
	public StringBuffer getFehlerText() {
		StringBuffer sb = new StringBuffer();
		if (throwable != null) {
			sb.append(throwable.getMessage() + "\n");
			sb.append(throwable.getLocalizedMessage() + "\n");
			for (StackTraceElement ste : throwable.getStackTrace()) {
				sb.append("\t" + ste.toString() + "\n");
			}
		}
		return sb;
	}

}