/**
 * GplAnzeigenToggleAction.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.files.HtmlHelper;
import eu.gronos.kostenrechner.view.helpanderror.AkzeptanzDialog;
import eu.gronos.kostenrechner.view.helpanderror.LizenzAkzeptanzDialog;

/**
 * Je nach Betätigung dieser Action zeigt das Fenster die gesamte Lizenz (GPL
 * 3.0) oder die für das Programm spezifische Kurzfassung der Lizenz an."
 *
 * @author Peter Schuster (setrok)
 * @date 21 Feb 2019
 *
 */
public class GplAnzeigenToggleAction extends BeschriebeneAktion {
	private final AkzeptanzDialog dialog;
	private static final long serialVersionUID = -5478738058066499706L;
	private boolean ausfuehrlich = false;
	public static final String GPL_GESAMT_TIPP = "Zeigt die gesamte Lizenz (GPL 3.0) an";
	public static final String GPL_GESAMT_TEXT = "Gesamte GPL anzeigen";
	static final String KURZ_LIZENZ_TIPP = "Zeigt die für das Programm spezifische Kurzfassung der Lizenz an.";
	static final String KURZ_LIZENZ_TEXT = "Lizenz anzeigen";

	/**
	 * 
	 * @param dialog   die Instanz des {@link LizenzAkzeptanzDialog}-Dialogs
	 * @param mnemonic ({@link javax.swing.Action#MNEMONIC_KEY}) The key used for
	 *                 storing an Integer that corresponds to one of the KeyEvent
	 *                 key codes. The value is commonly used to specify a mnemonic.
	 *                 KeyEvent.getExtendedKeyCodeForChar('\u0444')) sets the
	 *                 mnemonic of myAction to Cyrillic letter "Ef".
	 */
	public GplAnzeigenToggleAction(AkzeptanzDialog dialog, int mnemonic) {
		super(GPL_GESAMT_TEXT, GPL_GESAMT_TIPP, mnemonic);
		this.dialog = dialog;
	}

	@Override
	public void actionPerformed(ActionEvent e) {

		HtmlHelper helper = new HtmlHelper();
		if (!ausfuehrlich) {
			helper.leseHtml("resources/gpl3.html", dialog.getDoc(), dialog.getKit());
			putValue(Action.NAME, KURZ_LIZENZ_TEXT);
			putValue(Action.SHORT_DESCRIPTION, KURZ_LIZENZ_TIPP);
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_L);
			ausfuehrlich = true;
		} else {
			helper.leseHtml("resources/lizenz.html", dialog.getDoc(), dialog.getKit());
			putValue(Action.NAME, GPL_GESAMT_TEXT);
			putValue(Action.SHORT_DESCRIPTION, GPL_GESAMT_TIPP);
			putValue(Action.MNEMONIC_KEY, KeyEvent.VK_G);
			ausfuehrlich = false;
		}
		helper.leseCss("resources/kostentenor.css", dialog.getKit());
	}
}