/*
 * SchreibeEmailFehler.java
 * eu.gronos.kostenrechner.view.helpanderror (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;

/**
 * Die Klasse lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client über
 * {@link TenorUtils.emailSchreiben(String, String)} schreiben. Dabei wird der
 * Text des Tenors und noch XML-Daten mitgeschickt.
 *
 * @author Peter Schuster (setrok)
 * @date 24 Apr 2019
 *
 */
public class SchreibeEmailFehler extends SchreibeEmail {
	private final Throwable throwable;
	private static final long serialVersionUID = -4095891796997774737L;

	/**
	 * 
	 * @param name             {@link Action#NAME}
	 * @param shortDescription {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic         {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey {@link Action#ACTION_COMMAND_KEY}
	 * @param throwable        ein {@link Throwable}, um die
	 *                         {@link StackTraceElement}s auszulesen
	 */
	public SchreibeEmailFehler(String name, String shortDescription, int mnemonic, String actionCommandKey,
			Throwable throwable) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.throwable = throwable;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final String subject = "Fehlermeldung aus kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
		// Im E-Mail-Text wird der Stacktrace mitgeschickt.
		if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_BASIS)
				.noergle()) {
			// Wenn dem Anhang nicht zugestimmt wird, einfache E-Mail schreiben.
			final StringBuilder body = new StringBuilder(
					"Bitte beschreiben Sie, wie es zu dem Fehler gekommen ist: " + SchreibeEmail.NEUER_ABSATZ);
			if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_ANHANG)
					.noergle()) {
				// Bei Zustimmung auch Fehlertext und StackTrace mitschicken
				final String fehler = new FehlerHelper(throwable).getFehlerText().toString();
				body.append("Debug-Informationen für den Programmierer" + SchreibeEmail.NEUER_ABSATZ + fehler);
			}
			super.schreibe(subject, body.toString());
		}
	}
}