/**
 * EmailTenorFalschAction.java
 * eu.gronos.kostenrechner.view.result (Kostenrechner)
 */
package eu.gronos.kostenrechner.controller.system;

import java.awt.event.ActionEvent;

import javax.swing.Action;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.files.BerechnungXmlDatei;
import eu.gronos.kostenrechner.controller.files.TenorXmlDatei;
import eu.gronos.kostenrechner.model.tenordaten.TenorDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;

/**
 * Die {@link Action} lässt eine Beschwerde-E-Mail im Standard-E-Mail-Client
 * über {@link TenorUtils.emailSchreiben(String, String)} schreiben. Dabei wird
 * der Text des Tenors und noch XML-Daten mitgeschickt.
 * 
 *
 *
 * @author Peter Schuster (setrok)
 * @date 22 Nov 2018
 *
 */
public class SchreibeEmailTenorFalsch extends SchreibeEmail {
	private static final long serialVersionUID = -978875818285786710L;
	private final TenorDatenContainer tenorDatenContainer;
	private final VerfahrensDatenContainer container;

	/**
	 * Konstruktor.
	 * 
	 * @param name                {@link Action#NAME}
	 * @param shortDescription    {@link Action#SHORT_DESCRIPTION}
	 * @param mnemonic            {@link Action#MNEMONIC_KEY}
	 * @param actionCommandKey    {@link Action#ACTION_COMMAND_KEY}
	 * @param tenorDatenContainer ein {@link TenorDatenContainer}
	 * @param container           ein {@link VerfahrensDatenContainer}
	 */
	public SchreibeEmailTenorFalsch(String name, String shortDescription, int mnemonic, String actionCommandKey,
			TenorDatenContainer tenorDatenContainer, VerfahrensDatenContainer container) {
		super(name, shortDescription, mnemonic, actionCommandKey);
		this.tenorDatenContainer = tenorDatenContainer;
		this.container = container;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		final String subject = "Falscher Tenor in kostentenor.de " + Kostenrechner.KOSTENRECHNER_VERSION;
		// Im E-Mail-Text werden Tenor und noch XML-Daten mitgeschickt
		final String xmlString = new BerechnungXmlDatei().toXmlString(container);
		final String tenor = new TenorXmlDatei().toXmlString(tenorDatenContainer);
		final StringBuilder body = new StringBuilder("Aus den folgenden Gründen halte ich den Tenor für falsch\n:");
		if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_BASIS)
				.noergle()) {
			// Wenn dem Anhang nicht zugestimmt wird, einfache E-Mail schreiben.
			if (new DatenschutzNoergler(Kostenrechner.getInstance(), DatenschutzNoergler.OPTION_DSGVO_EMAIL_ANHANG)
					.noergle()) {
				body.append(SchreibeEmail.NEUER_ABSATZ + tenor + SchreibeEmail.NEUER_ABSATZ + xmlString);
			}
			super.schreibe(subject, body.toString());
		}
	}
	// (ich bin mir bewusst und damit einverstanden, dass in der E-Mail mein Name
	// und meine E-Mail-Adresse übermittelt werden sowie die anliegenden
	// Programmdaten)
}