/**
 * KostenTenorContainer.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.tenordaten;

import java.util.ArrayList;
import java.util.List;

import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlElements;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.datatype.XMLGregorianCalendar;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.interfaces.UnterContainerKlasse;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.baumbach.BaumbachGesamtschuldnerschaft;
import eu.gronos.kostenrechner.model.forderungen.AufrechnungsForderung;
import eu.gronos.kostenrechner.model.forderungen.Forderung;
import eu.gronos.kostenrechner.model.forderungen.KlageForderung;
import eu.gronos.kostenrechner.model.gebuehren.AuslagenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenErhoehungsTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenSatzTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.MehrwertsteuerTatbestand;

/**
 * Containerklasse, um alle Zustände im XML-Format speichern zu können.
 *
 * @author Peter Schuster (setrok)
 * @date 26.12.2014
 *
 */
@XmlRootElement(name = "SchustersKleinerKostenTenorXML", namespace = "http://www.kostentenor.de/")
public class VerfahrensDatenContainer {

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für Programminformationen im
	 * Dateikopf zu Debug-Zwecken
	 *
	 * @author Peter Schuster (setrok)
	 * @date 28.12.2014
	 */
	public static class Dateikopf {
		@XmlAttribute(name = "version")
		public String version = Kostenrechner.KOSTENRECHNER_VERSION.trim();

		@XmlAttribute(name = "osName")
		public String osName = System.getProperty("os.name");

		@XmlAttribute(name = "osArch")
		public String osArch = System.getProperty("os.arch");

		@XmlAttribute(name = "osVersion")
		public String osVersion = System.getProperty("os.version");

		@XmlAttribute(name = "javaVendor")
		public String javaVendor = System.getProperty("java.vm.vendor");

		@XmlAttribute(name = "javaName")
		public String javaName = System.getProperty("java.vm.name");

		@XmlAttribute(name = "javaVersion")
		public String javaVersion = System.getProperty("java.version");

	}

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für allgemeine,
	 * Registerkarten-übergreifende Einstellungne
	 *
	 * @author Peter Schuster (setrok)
	 * @date 28.12.2014
	 */
	public static class Allgemein implements UnterContainerKlasse {
		@XmlAttribute(name = "speicherdatum")
		public XMLGregorianCalendar speicherdatum;

		@XmlAttribute(name = "p92iiZPO")
		public boolean b92ii;

		@XmlAttribute(name = "aktenzeichen")
		public String aktenzeichen;// = "k.A.";

		@XmlAttribute(name = "streitwert")
		public double streitwert;
	}

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für die Registerkarte
	 * Teilklagerücknahme (Quotenmethode, Mehrkostenmethode)
	 *
	 * @author Peter Schuster (setrok)
	 * @date 27.12.2014
	 *
	 */
	public static class Teilklageruecknahme implements UnterContainerKlasse {
		@XmlElement(name = "partei")
		@XmlElementWrapper(name = "parteien")
		public List<Beteiligter> parteien = new ArrayList<Beteiligter>();

		@XmlElement(name = "wert")
		@XmlElementWrapper(name = "streitwerteUndObsiegen")
		public List<Long> streitwerteUndObsiegen = new ArrayList<Long>();

		@XmlElements({ @XmlElement(name = "gebuehrenSatz", type = GebuehrenSatzTatbestand.class),
				@XmlElement(name = "auslagen", type = AuslagenTatbestand.class),
				@XmlElement(name = "steuer", type = MehrwertsteuerTatbestand.class),
				@XmlElement(name = "erhoehungsSatz", type = GebuehrenErhoehungsTatbestand.class) })
		@XmlElementWrapper(name = "gebuehrenVorRuecknahme")
		//public GebuehrenTatbestand[] vorher = new GebuehrenTatbestand[0];
		public List<GebuehrenTatbestand> vorher = new ArrayList<GebuehrenTatbestand>();

		@XmlElements({ @XmlElement(name = "gebuehrenSatz", type = GebuehrenSatzTatbestand.class),
				@XmlElement(name = "auslagen", type = AuslagenTatbestand.class),
				@XmlElement(name = "steuer", type = MehrwertsteuerTatbestand.class),
				@XmlElement(name = "erhoehungsSatz", type = GebuehrenErhoehungsTatbestand.class) })
		@XmlElementWrapper(name = "gebuehrenNachRuecknahme")
		//public GebuehrenTatbestand[] nachher = new GebuehrenTatbestand[0];
		public List<GebuehrenTatbestand> nachher = new ArrayList<GebuehrenTatbestand>();
	}

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für die Registerkarte
	 * Streitgenossen
	 *
	 * @author Peter Schuster (setrok)
	 * @date 27.12.2014
	 *
	 */
	public static class Streitgenossen implements UnterContainerKlasse {

		@XmlAttribute(name = "widerklage")
		public boolean widerklage;

		@XmlAttribute(name = "p45i3GKG")
		public boolean wirtschaftlichIdentisch;

		@XmlElement(name = "klaeger")
		public BaumbachBeteiligtenListe klaeger = new BaumbachBeteiligtenListe();

		@XmlElements({ @XmlElement(name = "drittwiderbeklagter", type = BaumbachBeteiligter.class),
				@XmlElement(name = "widerbeklagtenGesamtschuldnerschaft", type = BaumbachGesamtschuldnerschaft.class) })
		@XmlElementWrapper(name = "drittwiderbeklagtenListe")
		public BaumbachBeteiligtenListe drittwiderbeklagte = new BaumbachBeteiligtenListe();

		@XmlElements({ @XmlElement(name = "beklagter", type = BaumbachBeteiligter.class),
				@XmlElement(name = "beklagtenGesamtschuldnerschaft", type = BaumbachGesamtschuldnerschaft.class) })
		@XmlElementWrapper(name = "beklagtenListe")
		public BaumbachBeteiligtenListe beklagte = new BaumbachBeteiligtenListe();
	}

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für die Registerkarte
	 * Staffelung
	 *
	 * @author Peter Schuster (setrok)
	 * @date 27.12.2014
	 *
	 */
	public static class ForderungsStaffelung implements UnterContainerKlasse {
		@XmlElement(name = "partei")
		@XmlElementWrapper(name = "parteien")
		public List<Beteiligter> parteien = new ArrayList<Beteiligter>();

		@XmlElements({ @XmlElement(name = "klage", type = KlageForderung.class),
				@XmlElement(name = "aufrechnung", type = AufrechnungsForderung.class) })
		@XmlElementWrapper(name = "forderungen")
		public ArrayList<Forderung> forderungen = new ArrayList<Forderung>();
	}

	/**
	 * Unterklasse von {@link VerfahrensDatenContainer} für die Registerkarte Freie
	 * Gebührenberechnung
	 *
	 * @author Peter Schuster (setrok)
	 * @date 27.12.2014
	 *
	 */
	public static class GebuehrenBerechnung implements UnterContainerKlasse {
		@XmlAttribute(name = "streitwert")
		public long streitwert;

		@XmlElements({ @XmlElement(name = "gebuehrenSatz", type = GebuehrenSatzTatbestand.class),
				@XmlElement(name = "auslagen", type = AuslagenTatbestand.class),
				@XmlElement(name = "steuer", type = MehrwertsteuerTatbestand.class),
				@XmlElement(name = "erhoehungsSatz", type = GebuehrenErhoehungsTatbestand.class) })
		@XmlElementWrapper(name = "gebuehren")
		//public GebuehrenTatbestand[] gebuehren = new GebuehrenTatbestand[0];
		public List<GebuehrenTatbestand> gebuehren = new ArrayList<GebuehrenTatbestand>();
	}

	@XmlElement(name = "programminfo")
	public Dateikopf dateikopf = new Dateikopf();

	@XmlElement(name = "allgemein")
	public Allgemein allgemein = new Allgemein();

	@XmlElement(name = "teilklageruecknahme")
	public Teilklageruecknahme teilklageruecknahme = new Teilklageruecknahme();

	@XmlElement(name = "streitgenossen")
	public Streitgenossen streitgenossen = new Streitgenossen();

	@XmlElement(name = "forderungsstaffelung")
	public ForderungsStaffelung staffelung = new ForderungsStaffelung();

	@XmlElement(name = "gebuehrenberechnung")
	public GebuehrenBerechnung gebuehrenBerechnung = new GebuehrenBerechnung();

}
