/**
 * VollstreckungsVerhaeltnis.java
 * eu.gronos.kostenrechner.calculation (Kostenrechner)
 */
package eu.gronos.kostenrechner.model.tenordaten;

import javax.xml.bind.JAXB;
import javax.xml.bind.annotation.XmlElement;

import eu.gronos.kostenrechner.model.baumbach.BaumbachVollstreckbarkeit;

/**
 * Klasse zum strukturierten Darstellen der vorläufigen Vollstreckbarkeit einer
 * Verurteilung
 *
 * @author Peter Schuster (setrok)
 * @date 28.07.2018
 *
 */
public class VollstreckungsVerhaeltnis extends ProzessVerhaeltnis {
	private Vollstreckbarkeit vollstreckbarkeit;

	/**
	 * Standardkonstruktor ohne Parameter für {@link JAXB}
	 * 
	 */
	public VollstreckungsVerhaeltnis() {
		super();
	}

	/**
	 * Der Konstruktor, mit dem man gleich alle Felder setzen kann.
	 * 
	 * @param glaeubiger        {@link #getGlaeubiger()}
	 * @param schuldner         {@link #getSchuldner()}
	 * @param vollstreckbarkeit {@link #getVollstreckbarkeit()}
	 */
	public VollstreckungsVerhaeltnis(Beteiligter glaeubiger, Beteiligter schuldner,
			Vollstreckbarkeit vollstreckbarkeit) {
		super(glaeubiger, schuldner, false);
		this.setVollstreckbarkeit(vollstreckbarkeit);
	}

	/**
	 * Ein Konstruktor, der sich zunutze macht, dass
	 * {@link BaumbachVollstreckbarkeit} fast alle Wete enthält, die für das
	 * Vollstreckungsverhältnis gebraucht werden
	 * 
	 * @param baumbach ein {@link BaumbachVollstreckbarkeit}
	 */
	public VollstreckungsVerhaeltnis(BaumbachVollstreckbarkeit baumbach) {
		this(baumbach.getGlaeubiger(), baumbach.getSchuldner(), new Vollstreckbarkeit(baumbach));
	}

	/**
	 * @return gibt {@link #vollstreckbarkeit} als VollstreckbarkeitsObjekt zurück.
	 */
	@XmlElement(name = "vollstreckbarkeit")
	public Vollstreckbarkeit getVollstreckbarkeit() {
		return vollstreckbarkeit;
	}

	/**
	 * @param vollstreckbarkeit d. {@link #vollstreckbarkeit}, d. gesetzt werden
	 *                          soll als VollstreckbarkeitsObjekt
	 */
	public void setVollstreckbarkeit(Vollstreckbarkeit vollstreckbarkeit) {
		this.vollstreckbarkeit = vollstreckbarkeit;
	}

	@Override
	public String toString() {
		if (getVollstreckbarkeit() == null)
			return super.toString();
		else
			return String.format("Vollstreckungsverhältnis %s gemäß %s", super.toString(), getVollstreckbarkeit().toString());
	}

}
