/**
 * AktionenJMenu.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.InfoAction;
import eu.gronos.kostenrechner.controller.system.BooleanConfigOptionHorcher;
import eu.gronos.kostenrechner.controller.system.LizenzAnzeigenAktion;
import eu.gronos.kostenrechner.controller.system.SchreibeEmailAllgemein;
import eu.gronos.kostenrechner.controller.system.SchreibeEmailXml;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Allgemein;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.gebuehren.GebuehrenBerechnungPanel;
import eu.gronos.kostenrechner.view.gebuehren.TeilklageruecknahmePanel;

/**
 * Das Menü Aktionen enthält die Aktionen für alle möglichen Menüpunkte
 *
 * @author Peter Schuster (setrok)
 * @date 20.06.2018
 *
 */
public class AktionenJMenu extends JMenu implements WerteLesbar<Allgemein> {

	private static final long serialVersionUID = 7935405681482100256L;
	private static final AktionenJMenu instance = new AktionenJMenu();
	private Kostenrechner kostenrechner;
	private final AbstractAction emailSchreibenXML = new SchreibeEmailXml("Fehlermeldung an Autor",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail mit Fehlerbehebungsdaten an den Programmierer zu verfassen</html>",
			KeyEvent.VK_T, "e-mail-plus");

	private final AbstractAction emailSchreibenAllgemein = new SchreibeEmailAllgemein("E-Mail an Autor",
			"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
			KeyEvent.VK_E, "e-mail");
	private final Action lizenzAnzeigenAktion = new LizenzAnzeigenAktion(this, "Lizenz",
			"Zeigt die Kurzform der Lizenzbedingungen an.", KeyEvent.VK_L);
	public JCheckBoxMenuItem mi92;
	private AbstractAction ueberAction = new InfoAction("Über " + Kostenrechner.KOSTENRECHNER_TITLE,
			"Zeigt Informationen über das Programm an.", KeyEvent.getExtendedKeyCodeForChar('Ü'), "about");

	/**
	 * Der private Konstruktor hat keine Parameter
	 * 
	 */
	private AktionenJMenu() {
		super("Aktionen");
	}

	/**
	 * Die Methode dient dazu, das Menü Aktionen mit Menüpunkten zu füllen
	 * 
	 */
	public void buildMenu() {
		setMnemonic(KeyEvent.VK_A);
		// TODO Tastenkürzel doppelt?
		mi92 = new JCheckBoxMenuItem("§ 92 Abs. 2 ZPO anwenden", true);
		mi92.setToolTipText(
				"Verhältnismäßig geringe Kostenquoten (unter 10%) werden nach § 92 II ZPO ignoriert und den anderen Beteiligten auferlegt.");
		mi92.setMnemonic(KeyEvent.VK_9);
		mi92.addItemListener(new BooleanConfigOptionHorcher("p92iiZPO", true, mi92));
		add(mi92);

		addSeparator();
		add(new JMenuItem(TeilklageruecknahmePanel.getInstance().klageruecknahmeTerminAction));
		addSeparator();
		add(new JMenuItem(StreitgenossenPanel.getInstance().baumbachAssistentAction));
		add(new JMenuItem(StreitgenossenPanel.getInstance().gesamtschuldnerschaftExtrahierenAction));
		add(new JMenuItem(StreitgenossenPanel.getInstance().gesamtschuldnerschaftDwbkExtrahierenAction));
		addSeparator();
		add(new JMenuItem(GebuehrenBerechnungPanel.getInstance().saeumnisGebuehrenAction));
		add(new JMenuItem(GebuehrenBerechnungPanel.getInstance().anerkenntnisGebuehrenAction));
		add(new JMenuItem(GebuehrenBerechnungPanel.getInstance().normaleGebuehrenAction));
		addSeparator();
		add(new JMenuItem(ueberAction));
		add(new JMenuItem(lizenzAnzeigenAktion));
		JMenu kontaktMenu = new JMenu("Kontakt");
		add(kontaktMenu);
		kontaktMenu.add(new JMenuItem(emailSchreibenXML));
		kontaktMenu.add(new JMenuItem(emailSchreibenAllgemein));
	}

	/**
	 * @return gibt {@link #instance} als AktionenJMenu zurück.
	 */
	public static AktionenJMenu getInstance() {
		return instance;
	}

	/**
	 * @param kostenrechner d. {@link #kostenrechner}, d. gesetzt werden soll als
	 *                      Kostenrechner
	 */
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * Die Methode dient dazu, die Oberflächenelemente des Panels mit Werten zu
	 * füllen, hier mit allgemeinen Programmeinstellungen, die nicht vom Container
	 * selbst gesetzt werden.
	 * 
	 * @param container
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#setzeWerte(eu.gronos.kostenrechner.controller.UnterContainerKlasse)
	 */
	@Override
	public void setzeWerte(Allgemein container) {
		mi92.setSelected(container.b92ii);
	}

	/**
	 * Die Methode dient dazu, allgemeine Programmeinstellungen, die nicht vom
	 * Container selbst gesetzt werden, in einen Container zu packen.
	 * 
	 * @return
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leseWerte()
	 */
	@Override
	public Allgemein leseWerte() {
		Allgemein container = new Allgemein();
		container.aktenzeichen = "k.A.";
		container.b92ii = mi92.isSelected();
		return container;
	}

	/**
	 * Wenn der leereWerte()-Befehl kommt, setzt die Klasse nur {@link #mi92} wieder
	 * auf <code>true</code>.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leereWerte()
	 */
	@Override
	public void leereWerte() {
		mi92.setSelected(true);
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	/**
	 * Die Methode stellt eine einfache "Info über"-Meldung dar.
	 * 
	 */
	public static void ueberKostenrechner() {
		JOptionPane.showMessageDialog(Kostenrechner.getInstance(),
				String.format("%s%s%n%s%n%nLogfile: %s", Kostenrechner.KOSTENRECHNER_TITLE,
						Kostenrechner.KOSTENRECHNER_VERSION, Kostenrechner.KOSTENRECHNER_COPY,
						Kostenrechner.getLogFileName()),
				"Über Kostenrechner", JOptionPane.INFORMATION_MESSAGE, RechnerhammerIcons.getRechnerhammer64());
	}

}
