/**
 * AktionenJMenu.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view;

import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.KeyStroke;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeendenAction;
import eu.gronos.kostenrechner.controller.TabellenZuruecksetzenAction;
import eu.gronos.kostenrechner.controller.files.LadeSkktxAction;
import eu.gronos.kostenrechner.controller.files.SpeichereSkktxAction;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;

/**
 * Das Menü Datei stellt die Aktionen für alle möglichen Dateioperationen zu
 * Menüpunkten zusammen.
 *
 * @author Peter Schuster (setrok)
 * @date 20.06.2018
 *
 */
public class DateiJMenu extends JMenu {

	private static final long serialVersionUID = 1481786221712533553L;
	private static final DateiJMenu instance = new DateiJMenu();
	private Kostenrechner kostenrechner;

	private final AbstractAction tabellenZuruecksetzenAction = new TabellenZuruecksetzenAction(
			"Neu / Alle Einträge zurücksetzen",
			"Leert die Tabelle der Gebührentatbestände und die Tabellen der Beteiligten", KeyEvent.VK_Z,
			"neue-berechnung", KeyStroke.getKeyStroke(KeyEvent.VK_N, Kostenrechner.ctrlMask));

	AbstractAction beendenAction = new BeendenAction("Ende", "Beendet Kostentenor", KeyEvent.VK_E, "quit");

	private Action skktxSpeichern = new SpeichereSkktxAction(this, "Berechnung speichern",
			"Speichert die Daten aller Registerkarten für spätere Berechnungen ab", KeyEvent.VK_S, "save",
			KeyStroke.getKeyStroke(KeyEvent.VK_S, Kostenrechner.ctrlMask));

	private Action skktxLaden = new LadeSkktxAction(getKostenrechner(), "Berechnung öffnen",
			"Lädt die Daten aller Registerkarten aus einer früher gespeicherten Berechnung", KeyEvent.VK_F, "open",
			KeyStroke.getKeyStroke(KeyEvent.VK_O, Kostenrechner.ctrlMask));

	/**
	 * Der private Konstruktor hat keine Parameter
	 * 
	 */
	private DateiJMenu() {
		super("Datei");
	}

	/**
	 * Die Methode dient dazu, das Dateimenü zusammenzubauen.
	 * 
	 */
	public void buildMenu() {
		setMnemonic(KeyEvent.VK_D);
		// TODO Tastenkürzel doppelt?
		add(new JMenuItem(tabellenZuruecksetzenAction));
		add(new JMenuItem(skktxLaden));
		add(new JMenuItem(StreitgenossenPanel.getInstance().xmlLesenAction));
		add(new JMenuItem(skktxSpeichern));
		addSeparator();
		add(new JMenuItem(beendenAction));
	}

	/**
	 * @return gibt {@link #instance} als AktionenJMenu zurück.
	 */
	public static DateiJMenu getInstance() {
		return instance;
	}

	/**
	 * @param kostenrechner d. {@link #kostenrechner}, d. gesetzt werden soll als
	 *                      Kostenrechner
	 */
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

}
