/**
 * BaumbachAssistentDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.baumbach;

import java.awt.BorderLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.InputEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSpinner;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.baumbach.AssistentBkStreitwertHorcher;
import eu.gronos.kostenrechner.controller.baumbach.AssistentBkUnterliegenHorcher;
import eu.gronos.kostenrechner.controller.baumbach.AssistentWbkStreitwertHorcher;
import eu.gronos.kostenrechner.controller.baumbach.AssistentWbkUnterliegenHorcher;
import eu.gronos.kostenrechner.controller.baumbach.AssistentWeiterAction;
import eu.gronos.kostenrechner.controller.files.VerfahrensDatenContainerUtil;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.baumbach.BaumbachGesamtschuldnerschaft;
import eu.gronos.kostenrechner.model.baumbach.ScalableBeteiligtenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Assistent zum Hinzufügen aller Beteiligten auf einmal. Mehrere Tabs/Schritte.
 * 1.) Zuerst müssen die einfachen (Baumbach?)Beteiligten für die Klage erstellt
 * werden, ohne das Unterliegen und den Streitwert festzulegen (ggf.
 * XJustiz-Import) bzw. mit einheitlichem Streitwert. 2.) Dann können die
 * Beteiligten der Widerklage hinzugefügt werden, wieder einheitlicher
 * Streitwert. 3.) Dann kann angegeben werden, welche Beteiligten
 * gesamtschuldnerisch und zu wieviel verurteilt wurden. Erst jetzt werden die
 * eigentlichen BaumbachBeteiligten initialisiert (oder jetzt bekommen sie ihre
 * Werte).
 * 
 * @author Peter Schuster (setrok)
 * @date 22.06.2014
 * 
 * 
 */
public class BaumbachAssistentDialog extends HinzufuegenDialog<VerfahrensDatenContainer> {

	private static final long serialVersionUID = -170526643502912253L;

	/* TODO: Lieber OkButton von super holen und manipulieren... */
	private AbstractAction weiter = new AssistentWeiterAction(this, "Weiter", "Weiter zum nächsten Reiter",
			KeyEvent.VK_W, "weiter", KeyStroke.getKeyStroke(KeyEvent.VK_W, InputEvent.ALT_DOWN_MASK));

	private boolean mitWiderklage;
	private int zustand;
	private JTabbedPane tabbedPane;
	private JPanel[] panels;
	public static final int HOECHSTER_ZUSTAND = 11;
	public static final int HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE = 6;
	private VerfahrensDatenContainer verfahrensDatenContainer;
	private JTextArea jta;
	private JSpinner spiKlaegerZahl;
	private JSpinner spiBeklagtenZahl;
	private ScalableBeteiligtenTableModel sbtmKlaeger;
	private Object[][] objectsKlaeger;
	private JTable tabKlaegerGenus;
	private JScrollPane scpKlaegerGenus;
	private Object[][] objectsBeklagte;
	private ScalableBeteiligtenTableModel sbtmBeklagte;
	private SimpleBeteiligtenTable tabBeklagtenGenus;
	private JScrollPane scpBeklagteGenus;
	private JCheckBox chbBkSwEinheitlich;
	JFormattedTextField ftfBkStreitwert;
	SimpleBeteiligtenTable tabBeklagtenSw;
	private JScrollPane scpBkSw;
	private JCheckBox chBkGesamtschuldner;
	JList<Beteiligter> liBkGesamtschuldner;
	private JScrollPane scpBeklagtenAuswahlListe;
	JFormattedTextField ftfBkGesamtUnterliegen;
	private SimpleBeteiligtenTable tbBkUnt;
	private JScrollPane scpBkUnt;
	private JList<Beteiligter> liBkWk;
	private JScrollPane scpBkWk;
	private JSpinner spiDwbkAnz;
	private ScalableBeteiligtenTableModel sbtmDrittwiderbeklagte;
	private Object[][] objectsDwbk;
	private SimpleBeteiligtenTable tbDwbkGenus;
	private JScrollPane scpDwbkGenus;
	private JCheckBox chbWkEinheitlich;
	JFormattedTextField ftfWkSw;
	SimpleBeteiligtenTable tabKlSw;
	private JScrollPane scpKlSw;
	SimpleBeteiligtenTable tabDwbkSw;
	private JScrollPane scpDwbkSw;
	private JCheckBox chbWkGesamtschuld;
	JList<Beteiligter> liWkGesamtschuldner;
	JFormattedTextField ftfWkUnterliegen;
	private SimpleBeteiligtenTable tbKlDarueber;
	private JScrollPane scpKlDarueber;
	private SimpleBeteiligtenTable tbDwbkDarueber;
	private JScrollPane scpDwbkDarueber;
	private int klaegerZahl;
	private int beklagtenZahl;
	private int dwbkZahl;
	private double bkStreitwert;
	private Beteiligter[] beteiligteBeklagte;
	private double bkUnterliegen = 0.0;
	private int[] bkGesamtIndices = new int[0];
	private double bkGesamtUnterliegen = 0;
	private int bkWiderklaegerIndex;
	private JCheckBox chbKlWk;
	private boolean bkGesamt = false;
	private double wkStreitwert;
	private boolean klWk = false;
	private Beteiligter[] beteiligteWiderbeklagte;
	private double wkGesamtUnterliegen = 0;
	private double wkUnterliegen = 0;
	private boolean wkGesamt = false;
	private int[] wkGesamtIndices = null;

	private JLabel lblBkStreitwert;

	private JLabel lblBkGesamtUnterliegen;

	private JLabel lblWkSw;

	private JLabel lblWkUnterliegen;

	/**
	 * Der Konstruktor für den Standardfall ohne Xjustiz-Import
	 * 
	 * @param owner         the JFrame from which the dialog is displayed
	 * @param mitWiderklage sollen auch die Schritte für die Widerklage durchgeführt
	 *                      werden?
	 */
	public BaumbachAssistentDialog(JFrame owner, boolean mitWiderklage) {
		super(owner, "Assistent für die Baumbach'sche Kostenformel");
		this.setMitWiderklage(mitWiderklage);
		this.setZustand(0);
		this.setContainer(null);
		klaegerZahl = 1;
		beklagtenZahl = 2;
		dwbkZahl = 0;
		// hier einen Default-Wert für den Streitwert nehmen
		bkStreitwert = 10000;
		setWkStreitwert(0);
		setOkButtonToBeFocussed(false);
	}

	/**
	 * Ein Konstruktor, der Verfahrensdaten vom XJustiz-Import annimmt und dann ein
	 * paar Felder schon einmal vorbelegen kann.
	 * 
	 * @param owner     the JFrame from which the dialog is displayed
	 * @param container ein KostenTenorContainer aus einem Xjustiz-Import
	 */
	public BaumbachAssistentDialog(JFrame owner, VerfahrensDatenContainer container) {
		super(owner, "Assistent für die Baumbach'sche Kostenformel (Xjustiz-Import)");
		if (container == null)
			throw new IllegalArgumentException("Leere Datei oder Datei konnte nicht gelesen werden!");
		this.setMitWiderklage(container.streitgenossen.widerklage);
		this.setZustand(0);
		this.setContainer(container);
		if (!VerfahrensDatenContainerUtil.gibtsKlaeger(container) || container.streitgenossen.klaeger.size() != 1)
			throw new IllegalArgumentException("Nur ein Kläger erlaubt!");
		klaegerZahl = container.streitgenossen.klaeger.size();
		setKlWk((isMitWiderklage() && container.streitgenossen.klaeger.enthaeltAnWiderklageBeteiligte()));
		/* Gibt's überhaupt Beklagte und wenn ja, maximal einen Widerkläger? */
		if (!VerfahrensDatenContainerUtil.gibtsBeklagte(container))
			throw new IllegalArgumentException("Datei enthält keine Beklagten!");
		else if (container.streitgenossen.beklagte.zaehleAnWiderklageBeteiligte() > 1)
			throw new IllegalArgumentException("Nur ein Widerkläger erlaubt!");
		beklagtenZahl = container.streitgenossen.beklagte.size();
		if (VerfahrensDatenContainerUtil.gibtsDrittwiderbeklagte(container))
			dwbkZahl = container.streitgenossen.drittwiderbeklagte.size();
		else
			dwbkZahl = 0;
		/*
		 * Gibt es Widerkläger ohne Widerbeklagte oder Widerbeklagte ohne Widerkläger?
		 */
		if (this.isMitWiderklage()) {
			if (container.streitgenossen.beklagte.zaehleAnWiderklageBeteiligte() > 0
					&& container.streitgenossen.klaeger.zaehleAnWiderklageBeteiligte() < 1 && dwbkZahl < 1)
				throw new IllegalArgumentException("Datei enthält Widerkläger ohne Widerbeklagte!");
			if ((container.streitgenossen.klaeger.zaehleAnWiderklageBeteiligte() > 0 || dwbkZahl > 0)
					&& container.streitgenossen.beklagte.zaehleAnWiderklageBeteiligte() < 1)
				throw new IllegalArgumentException("Datei enthält Widerbeklagte ohne Widerkläger!");
		}
		// hier einen Default-Wert für den Streitwert nehmen (z.B. Xjustiz)
		bkStreitwert = container.allgemein.streitwert;
		setWkStreitwert(container.allgemein.streitwert);
		setOkButtonToBeFocussed(false);
	}

	/**
	 * Die Methode setzt alle Tabs außer den aktuellen und den Start-Tab auf deaktiv
	 * 
	 * @param index Index des zu aktivierenden Tabs
	 */
	public void zustandAnzeigen(int index) {
		for (int i = 0; i < (getHoechsterZustand()) && panels[i] != null; i++) {
			if (i == 0) {
				tabbedPane.setEnabledAt(i, true);
			} else if (i == index) {
				tabbedPane.setEnabledAt(i, true);
				tabbedPane.setSelectedIndex(i);
			} else {
				tabbedPane.setEnabledAt(i, false);
			}
		}
		/* Solange die Eingabe noch nicht abgeschlossen ist, Ok ausblenden */
		if (index < getHoechsterZustand() - 1) {
			super.setOkActionEnabled(false);
		} else {
			super.setOkActionEnabled(true);
		}
	}

	/**
	 * Die Methode darf erst ganz zum Schluss true melden, abhängig davon also, ob
	 * eine Widerklage vorliegt oder nicht. Wenn keine Widerklage vorliegt, ist der
	 * Vorgang nach dem Panel 5 fertig, mit Widerklage nach dem Panel 10.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	public boolean parseEingabe() {
		if (isMitWiderklage()) {
			Kostenrechner.getLogger().info(String.format("Zustand: %d", getZustand()));
			// System.out.println(zustand);
			return getZustand() == HOECHSTER_ZUSTAND - 1 && parseEingabe(10);
		} else {
			return getZustand() == HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE - 1 && parseEingabe(5);
		}
	}

	/**
	 * Die Variante von parseEingabe, die nach Import von Xjustiz genutzt werden
	 * soll. Überprüft die Eingabe und übergibt sie an das nächste Tab, füllt dieses
	 * aber, wenn möglich mit Daten aus Xjustiz.
	 * 
	 * @param tab           die Nummer des Tabulators
	 * @param ueberspringen true = es sollen die bei Xjustiz-Import unnötigen Tabs
	 *                      übersprungen werden
	 * @return true, wenn zum nächsten, nicht zu überspringenden Tab gewechselt
	 *         werden kann
	 */
	public boolean parseEingabe(int tab, boolean ueberspringen) {
		if (!ueberspringen)
			return parseEingabe(tab);
		else {
			/*
			 * Der Konstruktor für vdc überprüft schon, ob genug Kläger und Beklagte da sind
			 * und setzt die Anzahl dieser Beteiligten.
			 * 
			 * Um das eigentliche Überspringen muss sich der Weiter-Button kümmern.
			 * 
			 * Bei den zu überspringenden Tabs werden die zweidimensionalen Object-Arrays
			 * gefüllt, die sbtm-Modelle angelegt, setModel bei den JTables ausgeführt und
			 * die jeweiligen Streitwert-Textfelder gefüllt.
			 */
			if (tab == 0) {
				boolean b = parseEingabe(tab);
				/*
				 * Klägerdaten auslesen (werden an der Oberfläche dann erstmal nicht gebraucht),
				 * Beklagtendaten auslesen und für Panel 3 setzen
				 */
				objectsKlaeger = konvertiere(getVerfahrensDatenContainer().streitgenossen.klaeger);
				objectsBeklagte = konvertiere(getVerfahrensDatenContainer().streitgenossen.beklagte);
				ftfBkStreitwert.setText(Double.toString(bkStreitwert));
				sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE, bkStreitwert);
				tabBeklagtenSw.setModel(sbtmBeklagte);
				chbBkSwEinheitlich.setSelected(false);
				chbBkSwEinheitlich.setSelected(true);
				return b;
			} else if (tab == 5) {
				boolean b = parseEingabe(tab);
				/*
				 * Drittwiderbeklagtendaten auslesen, Kläger- und Dwbk-Daten für Panel 8 setzen.
				 */
				ftfWkSw.setText(Double.toString(getWkStreitwert()));
				if (isKlWk()) {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER,
							getWkStreitwert());
				} else {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER, 0);
				}
				tabKlSw.setModel(sbtmKlaeger);
				if (VerfahrensDatenContainerUtil.gibtsDrittwiderbeklagte(getVerfahrensDatenContainer()))
					objectsDwbk = konvertiere(getVerfahrensDatenContainer().streitgenossen.drittwiderbeklagte);
				else
					objectsDwbk = new Object[0][4];
				sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
						getWkStreitwert());
				tabDwbkSw.setModel(sbtmDrittwiderbeklagte);
				chbWkEinheitlich.setSelected(false);
				chbWkEinheitlich.setSelected(true);
				return b;
			} else {
				return parseEingabe(tab);
			}
		}
	}

	/**
	 * @return gibt {@link #zustand} als int zurück.
	 */
	public int getZustand() {
		return zustand;
	}

	/**
	 * @param zustand d. {@link #zustand}, d. gesetzt werden soll als int
	 */
	public void setZustand(int zustand) {
		this.zustand = zustand;
	}

	/**
	 * Gibt abhängig von Widerklage oder nicht den tatsächlichen höchstmöglichen
	 * Zustand aus.
	 * 
	 * @return {@link #HOECHSTER_ZUSTAND} oder
	 *         {@link #HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE}
	 */
	public int getHoechsterZustand() {
		if (isMitWiderklage()) {
			return HOECHSTER_ZUSTAND;
		} else {
			return HOECHSTER_ZUSTAND_OHNE_WIDERKLAGE;
		}
	}

	/**
	 * @return gibt {@link #verfahrensDatenContainer} als VerfahrensDatenContainer
	 *         zurück.
	 */
	public VerfahrensDatenContainer getVerfahrensDatenContainer() {
		return verfahrensDatenContainer;
	}

	/**
	 * @return gibt {@link #mitWiderklage} als boolean zurück.
	 */
	public boolean isMitWiderklage() {
		return mitWiderklage;
	}

	/**
	 * @return gibt {@link #wkStreitwert} als double zurück.
	 */
	public double getWkStreitwert() {
		return wkStreitwert;
	}

	/**
	 * @return gibt {@link #klWk} als boolean zurück.
	 */
	public boolean isKlWk() {
		return klWk;
	}

	/**
	 * @return gibt {@link #wkGesamtUnterliegen} als double zurück.
	 */
	public double getWkGesamtUnterliegen() {
		return wkGesamtUnterliegen;
	}

	/**
	 * Diese Methode erstellt zum Abschluss BaumbachBeteiligtenListen
	 * 
	 * @return und einen KostenTenorContainer zurückgeben
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected VerfahrensDatenContainer baueRueckgabewert() {
		/*
		 * Erst wenn der letzte Zustand erreicht ist, darf gebaut werden (sollte
		 * eigentlich von parseEingabe() geprüft werden
		 */
		BaumbachBeteiligtenListe klaeger = konvertiere(objectsKlaeger);
		BaumbachBeteiligtenListe beklagte = konvertiere(objectsBeklagte);
		BaumbachBeteiligtenListe drittwiderbeklagte;
		String aktenzeichen;
		double streitwert;
		// Höchsten Streitwert ermitteln
		if (getWkStreitwert() > bkStreitwert)
			streitwert = getWkStreitwert();
		else
			streitwert = bkStreitwert;
		// streitwert = vdc.allgemein.streitwert;
		if (getVerfahrensDatenContainer() != null) {
			aktenzeichen = getVerfahrensDatenContainer().allgemein.aktenzeichen;
		} else {
			aktenzeichen = "";
		}
		if (isMitWiderklage()) {
			drittwiderbeklagte = konvertiere(objectsDwbk);
		} else {
			drittwiderbeklagte = null;
		}
		if (bkGesamt) {
			/* BaumbachGesamtschuldnerschaft bk bauen */
			// TODO Ist genusNumerus Plural?
			beklagte.add(new BaumbachGesamtschuldnerschaft(Beteiligter.BEKLAGTE,
					Beteiligter.MAENNLICH + Beteiligter.PLURAL, bkGesamtUnterliegen, false, bkGesamtIndices, beklagte));
		}
		/*
		 * BaumbachGesamtschuldnerschaft kl+dbwk bauen, sofern Voraussetzungen. Zur
		 * Sicherheit (auch wenn's eigentlich schon früher abgefragt wird) auch
		 * Bedingungen gegen NullPointerExceptions
		 */
		if (wkGesamt && drittwiderbeklagte != null && drittwiderbeklagte.size() > 0) {
			BaumbachBeteiligtenListe zwischen = new BaumbachBeteiligtenListe();
			zwischen.addAll(klaeger);
			zwischen.addAll(drittwiderbeklagte);
			drittwiderbeklagte.add(new BaumbachGesamtschuldnerschaft(Beteiligter.DRITTWIDERBEKLAGTE,
					Beteiligter.MAENNLICH + Beteiligter.PLURAL, getWkGesamtUnterliegen(), true, wkGesamtIndices,
					zwischen));
		}
		return VerfahrensDatenContainerUtil.erstelleContainer(klaeger, drittwiderbeklagte, beklagte, streitwert,
				aktenzeichen);
	}

	/**
	 * Die Methode initialisiert die allgemeinen JComponents für den Inhaltsbereich
	 * des Dialogs und wird vom Konstruktor selbsttätig aufgerufen. In der
	 * abgeleiteten Klasse baut die Methode eine {@link javax.swing.JTabbedPane}
	 * auf, mit so vielen Tabs wie Zuständen, die in einem Array JPanel[] panels
	 * aufbewahrt werden.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueContentPane()
	 */
	@Override
	protected void baueContentPane() {
		tabbedPane = new JTabbedPane(JTabbedPane.TOP);
		// BorderLayout.NORTH führt zu Darstellungsfehlern beim Vergrößern
		getContentPane().add(tabbedPane, BorderLayout.CENTER);
		tabbedPane.setBorder(BORDER);
		panels = new JPanel[HOECHSTER_ZUSTAND];
		// for-Schleife zum Anlegen der Reiter
		for (int i = 0; i < panels.length; i++) {
			panels[i] = new JPanel();
			panels[i].setBorder(BORDER);
			panels[i].setLayout(new GridBagLayout());
		}
	}

	/**
	 * Füllt die Tabs mit Oberflächenelemente
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		// Jeder Schritt ist ein einzelnes JPanel im tabbedPane

		getButtonPane().add(new JButton(weiter));
		addToRootInputMap(weiter);

		// Nur je eine sbtm-Variable für kl, bk, dwbk!

		// Panel 0:
		tabbedPane.addTab("Start", null, panels[0], "Willkommenstext anzeigen");
		String willkommen = "Willkommen!\n\n" + "Der Assistent führt Sie durch die notwendigen Schritte, \n"
				+ "um den Baumbach'schen Kostentenor berechnen zu können.\n\n"
				+ "Klicken Sie \"Weiter\", um zu beginnen.";
		if (getVerfahrensDatenContainer() != null)
			willkommen = "Willkommen!\n\n" + "Sie möchten die Beteiligtendaten aus dem Verfahren "
					+ getVerfahrensDatenContainer().allgemein.aktenzeichen + " importieren.\n"
					+ "Der Assistent unternimmt mit Ihnen die noch verbleibenden Schritte, um die noch fehlenden Daten zu erfragen, \n"
					+ "mit denen ein Baumbach'scher Kostentenor berechnet werden kann.\n\n"
					+ "Klicken Sie \"Weiter\", um zu beginnen.";
		jta = new JTextArea(willkommen);
		jta.setEditable(false);
		panels[0].add(jta, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 1:
		// Kann bei Import übersprungen werden
		tabbedPane.addTab("Beteiligtenanzahl (1)", null, panels[1], "Anzahl der Kläger und Beklagten");
		JLabel lblKlaegerZahl = new JLabel("Anzahl der Kläger: ", SwingConstants.RIGHT);
		panels[1].add(lblKlaegerZahl,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		spiKlaegerZahl = new JSpinner(new SpinnerNumberModel(klaegerZahl, 0, 100, 1));
		setMnemonicLabelFor(lblKlaegerZahl, spiKlaegerZahl, KeyEvent.VK_K);
		spiKlaegerZahl.setEnabled(false);
		panels[1].add(spiKlaegerZahl,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblBeklagtenZahl = new JLabel("Anzahl der Beklagten: ", SwingConstants.RIGHT);
		panels[1].add(lblBeklagtenZahl,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		spiBeklagtenZahl = new JSpinner(new SpinnerNumberModel(beklagtenZahl, 0, 100, 1));
		setMnemonicLabelFor(lblBeklagtenZahl, spiBeklagtenZahl, KeyEvent.VK_B);
		panels[1].add(spiBeklagtenZahl,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Panel 2
		// kann bei Import übersprungen werden
		tabbedPane.addTab("-genus (2)", null, panels[2], "Bezeichnung von Kläger und Beklagten");

		JLabel lblGenusKlaeger = new JLabel("Bezeichnung der Kläger");
		panels[2].add(lblGenusKlaeger,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsKlaeger = new Object[1][4];
		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER);
		tabKlaegerGenus = new SimpleBeteiligtenTable(sbtmKlaeger, Beteiligter.KLAEGER);
		scpKlaegerGenus = new JScrollPane(tabKlaegerGenus);
		setMnemonicLabelFor(lblGenusKlaeger, scpKlaegerGenus, KeyEvent.VK_K);
		panels[2].add(scpKlaegerGenus,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, false));

		JLabel lblBeklagtenGenus = new JLabel("Bezeichnung der Beklagten");
		panels[2].add(lblBeklagtenGenus,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsBeklagte = new Object[2][3];
		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE);
		tabBeklagtenGenus = new SimpleBeteiligtenTable(sbtmBeklagte, Beteiligter.BEKLAGTE);
		scpBeklagteGenus = new JScrollPane(tabBeklagtenGenus);
		setMnemonicLabelFor(lblBeklagtenGenus, scpBeklagteGenus, KeyEvent.VK_B);
		panels[2].add(scpBeklagteGenus,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 3
		tabbedPane.addTab("Klagewert (3)", null, panels[3], "Streitwert der Beklagten (für alle einheitlich?)");
		chbBkSwEinheitlich = new JCheckBox("Die Beklagten wurden alle in derselben Höhe in Anspruch genommen", true);
		chbBkSwEinheitlich.setMnemonic(KeyEvent.VK_H);
		panels[3].add(chbBkSwEinheitlich,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		lblBkStreitwert = new JLabel("Streitwert: ", SwingConstants.RIGHT);
		panels[3].add(lblBkStreitwert,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfBkStreitwert = new JFormattedTextField(formatter);
		// Vorbelegen mit dem Wert aus Xjustiz, sofern bekannt
		ftfBkStreitwert.setText(Double.toString(bkStreitwert));
		ftfBkStreitwert.setColumns(40);
		setMnemonicLabelFor(lblBkStreitwert, ftfBkStreitwert, KeyEvent.VK_S);
		// "Weiter"-Action der InputMap hinzufügen und mit VK_ENTER verknüpfen
		setEnterAction(ftfBkStreitwert, weiter);
		panels[3].add(ftfBkStreitwert,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE, 10000.0);
		tabBeklagtenSw = new SimpleBeteiligtenTable(sbtmBeklagte, Beteiligter.BEKLAGTE);
		chbBkSwEinheitlich.addChangeListener(new AssistentBkStreitwertHorcher(ftfBkStreitwert, tabBeklagtenSw));
		scpBkSw = new JScrollPane(tabBeklagtenSw);
		panels[3].add(scpBkSw, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));
		/*
		 * Noch einmal auf ausgewählt setzen, damit der ChangeListener ausgelöst wird
		 */
		chbBkSwEinheitlich.setSelected(true);

		// Panel 4
		tabbedPane.addTab("Gesamtschuld (4)", null, panels[4],
				"Wurden die Beklagten (ganz oder z.T.) gesamtschuldnerisch verurteilt? Auswahl der Beklagten");
		chBkGesamtschuldner = new JCheckBox("Die Beklagten wurden ganz oder z.T. gesamtschuldnerisch verurteilt", true);
		chBkGesamtschuldner.setMnemonic(KeyEvent.VK_G);
		panels[4].add(chBkGesamtschuldner,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblBkGesamtschuldner = new JLabel("Gesamtschuldnerisch verurteilte Beklagten auswählen");
		panels[4].add(lblBkGesamtschuldner,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Hier muss ich wohl die erste Spalte nehmen...
		liBkGesamtschuldner = new JList<Beteiligter>(new Beteiligter[0]);
		liBkGesamtschuldner.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		liBkGesamtschuldner.setCellRenderer(new BeteiligtenRenderer());
		scpBeklagtenAuswahlListe = new JScrollPane(liBkGesamtschuldner);
		setMnemonicLabelFor(lblBkGesamtschuldner, scpBeklagtenAuswahlListe, KeyEvent.VK_V);
		panels[4].add(scpBeklagtenAuswahlListe,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));

		lblBkGesamtUnterliegen = new JLabel("Höhe der Verurteilung", SwingConstants.RIGHT);
		panels[4].add(lblBkGesamtUnterliegen,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfBkGesamtUnterliegen = new JFormattedTextField(formatter);
		ftfBkGesamtUnterliegen.setText("10");
		ftfBkGesamtUnterliegen.setColumns(40);
		setMnemonicLabelFor(lblBkGesamtUnterliegen, ftfBkGesamtUnterliegen, KeyEvent.VK_H);
		setEnterAction(ftfBkGesamtUnterliegen, weiter);
		/*
		 * Muss nach dem Mnemonic, weil das Zuweisen eines Mnemonic wohl das ChangeEvent
		 * auch auslöst. Und auch nach der Initialisierung der Oberflächenelemente.
		 */
		chBkGesamtschuldner
				.addChangeListener(new AssistentBkUnterliegenHorcher(liBkGesamtschuldner, ftfBkGesamtUnterliegen));
		panels[4].add(ftfBkGesamtUnterliegen,
				GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		/* Und einmal die Auswahl setzen, um ein ChangeEvent auszulösen */
		chBkGesamtschuldner.setSelected(true);

		// Panel 5
		// ==> sonst Unterliegen 0
		tabbedPane.addTab("Darüber hinaus (5)", null, panels[5],
				"Wurden einige Beklagte darüber hinaus verurteilt? Welche, zu wie viel?");

		JLabel lblBkUnt = new JLabel("Folgende Beklagte wurden darüber hinaus zu weiterer Zahlung verurteilt:");
		panels[5].add(lblBkUnt,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE, 10000.0, 0.0);
		tbBkUnt = new SimpleBeteiligtenTable(sbtmBeklagte, Beteiligter.BEKLAGTE);
		scpBkUnt = new JScrollPane(tbBkUnt);
		setMnemonicLabelFor(lblBkUnt, scpBkUnt, KeyEvent.VK_D);
		panels[5].add(scpBkUnt, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));

		// AB HIER NUR WENN (mitWiderklage == true)
		if (isMitWiderklage() == false) {
			zustandAnzeigen(getZustand());
			panelsAusblendenBeiImport();
			return;
		}

		// Panel 6
		// kann bei Import übersprungen werden
		tabbedPane.addTab("Widerklage (6)", null, panels[6], "Welcher Beklagte klagt? Wie viele Drittwiderbeklagten?");
		JLabel lblBkWk = new JLabel("Bitte wählen Sie den Beklagten aus, der auch Widerkläger ist.");
		panels[6].add(lblBkWk,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 3, 1, GridBagConstraints.HORIZONTAL, false));

		liBkWk = new JList<Beteiligter>();
		liBkWk.setCellRenderer(new BeteiligtenRenderer());
		liBkWk.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		scpBkWk = new JScrollPane(liBkWk);
		setMnemonicLabelFor(lblBkWk, scpBkWk, KeyEvent.VK_B);
		panels[6].add(scpBkWk, GitterBeutelBeschraenkungen.getInstance(0, 1, 3, 1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkAnz = new JLabel("Wie viele Drittwiderbeklagte gibt es?", SwingConstants.RIGHT);
		panels[6].add(lblDwbkAnz,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));
		spiDwbkAnz = new JSpinner(new SpinnerNumberModel(dwbkZahl, 0, 100, 1));
		setMnemonicLabelFor(lblDwbkAnz, spiDwbkAnz, KeyEvent.VK_D);
		panels[6].add(spiDwbkAnz,
				GitterBeutelBeschraenkungen.getInstance(1, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Abfragen, ob Wk auch gegen den Kläger geht
		chbKlWk = new JCheckBox("auch der Kläger ist Widerbeklagter", true);
		chbKlWk.setMnemonic(KeyEvent.VK_U);
		panels[6].add(chbKlWk,
				GitterBeutelBeschraenkungen.getInstance(2, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Panel 7
		// kann bei Import übersprungen werden
		tabbedPane.addTab("Drittwiderklage (7)", null, panels[7], "Bezeichnung der Drittwiderbeklagten");
		JLabel lblDwbkGenus = new JLabel("Genus der Drittwiderbeklagten");
		panels[7].add(lblDwbkGenus,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		objectsDwbk = new Object[dwbkZahl][4];
		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE);
		tbDwbkGenus = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte, Beteiligter.DRITTWIDERBEKLAGTE);
		scpDwbkGenus = new JScrollPane(tbDwbkGenus);
		setMnemonicLabelFor(lblDwbkGenus, scpDwbkGenus, KeyEvent.VK_G);
		panels[7].add(scpDwbkGenus,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 8
		tabbedPane.addTab("Widerklagewert (8)", null, panels[8],
				"Wurden Widerbeklagte in derselben Höhe verklagt? Welche (jeweils)?");
		chbWkEinheitlich = new JCheckBox("Alle Widerbeklagten wurden in derselben Höhe in Anspruch genommen", true);
		chbWkEinheitlich.setMnemonic(KeyEvent.VK_H);
		panels[8].add(chbWkEinheitlich,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		lblWkSw = new JLabel("Streitwert der Widerklage:", SwingConstants.RIGHT);
		panels[8].add(lblWkSw,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfWkSw = new JFormattedTextField(formatter);
		ftfWkSw.setText(Double.toString(getWkStreitwert()));
		setMnemonicLabelFor(lblWkSw, ftfWkSw, KeyEvent.VK_S);
		setEnterAction(ftfWkSw, weiter);
		panels[8].add(ftfWkSw,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblKlSw = new JLabel("Streitwert für den Kläger:");
		panels[8].add(lblKlSw,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER, 0.0);
		tabKlSw = new SimpleBeteiligtenTable(sbtmKlaeger, Beteiligter.KLAEGER);
		scpKlSw = new JScrollPane(tabKlSw);
		setMnemonicLabelFor(lblKlSw, scpKlSw, KeyEvent.VK_K);
		panels[8].add(scpKlSw, GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkSw = new JLabel("Streitwert bzgl. der Drittwiderbeklagten:");
		panels[8].add(lblDwbkSw,
				GitterBeutelBeschraenkungen.getInstance(1, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE, 0.0);
		tabDwbkSw = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte, Beteiligter.DRITTWIDERBEKLAGTE);
		chbWkEinheitlich.addChangeListener(new AssistentWbkStreitwertHorcher(this, ftfWkSw, tabKlSw, tabDwbkSw));
		scpDwbkSw = new JScrollPane(tabDwbkSw);
		setMnemonicLabelFor(lblDwbkSw, scpDwbkSw, KeyEvent.VK_D);
		panels[8].add(scpDwbkSw, GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.BOTH, false));

		// Panel 9
		tabbedPane.addTab("Gesamtschuld (9)", null, panels[9],
				"Wurden die Widerbeklagten z.T. gesamtschuldnerisch verurteilt? Welche Widerbeklagten, in welcher Höhe?");
		chbWkGesamtschuld = new JCheckBox("Einige Widerbeklagte wurden gesamtschuldnerisch verurteilt.", true);
		chbWkGesamtschuld.setMnemonic(KeyEvent.VK_E);
		panels[9].add(chbWkGesamtschuld,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblWkGesamtschuldner = new JLabel("Folgende Widerbeklagte wurde gesamtschuldnerisch verurteilt");
		panels[9].add(lblWkGesamtschuldner,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 2, 1, GridBagConstraints.HORIZONTAL, false));

		liWkGesamtschuldner = new JList<Beteiligter>();
		liWkGesamtschuldner.setCellRenderer(new BeteiligtenRenderer());
		setMnemonicLabelFor(lblWkGesamtschuldner, liWkGesamtschuldner, KeyEvent.VK_F);
		liWkGesamtschuldner.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
		panels[9].add(liWkGesamtschuldner,
				GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));

		lblWkUnterliegen = new JLabel("Höhe der Verurteilung", SwingConstants.RIGHT);
		panels[9].add(lblWkUnterliegen,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfWkUnterliegen = new JFormattedTextField(formatter);
		ftfWkUnterliegen.setText(Double.toString(getWkGesamtUnterliegen()));
		setMnemonicLabelFor(lblWkUnterliegen, ftfWkUnterliegen, KeyEvent.VK_H);
		chbWkGesamtschuld
				.addChangeListener(new AssistentWbkUnterliegenHorcher(this, liWkGesamtschuldner, ftfWkUnterliegen));
		setEnterAction(ftfWkUnterliegen, weiter);
		panels[9].add(ftfWkUnterliegen,
				GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));

		// Panel 10
		tabbedPane.addTab("Darüber hinaus (10)", null, panels[10],
				"Wurden einzelne Widerbeklagten darüber hinaus einzeln verurteilt? Welche, in welcher Höhe?");
		JLabel lblKlDarueber = new JLabel("Folgende Kläger wurden darüber hinaus verurteilt:");
		panels[10].add(lblKlDarueber,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER, getWkStreitwert(),
				wkUnterliegen);
		tbKlDarueber = new SimpleBeteiligtenTable(sbtmKlaeger, Beteiligter.KLAEGER);
		scpKlDarueber = new JScrollPane(tbKlDarueber);
		setMnemonicLabelFor(lblKlDarueber, scpKlDarueber, KeyEvent.VK_K);
		panels[10].add(scpKlDarueber,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, false));

		JLabel lblDwbkDarueber = new JLabel("Folgende Drittwiderbeklagte wurden darüber hinaus verurteilt:");
		panels[10].add(lblDwbkDarueber,
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
				getWkStreitwert(), wkUnterliegen);
		tbDwbkDarueber = new SimpleBeteiligtenTable(sbtmDrittwiderbeklagte, Beteiligter.DRITTWIDERBEKLAGTE);
		scpDwbkDarueber = new JScrollPane(tbDwbkDarueber);
		setMnemonicLabelFor(lblDwbkDarueber, scpDwbkDarueber, KeyEvent.VK_D);
		panels[10].add(scpDwbkDarueber,
				GitterBeutelBeschraenkungen.getInstance(1, 1, 1, 1, GridBagConstraints.BOTH, false));

		zustandAnzeigen(getZustand());
		panelsAusblendenBeiImport();
	}

	/**
	 * Überprüft die Eingabe und übergibt sie an das nächste Tab
	 * 
	 * @param tab die Nummer des Tabulators
	 * @return true, wenn das Überprüfen und Übergeben klappt
	 */
	private boolean parseEingabe(int tab) {
		/*
		 * Zu jedem Panel überlegen, was ausgelesen und was in das nächste Panel
		 * geschrieben werden muss. Mögliche Exceptions und RuntimeExceptions müssen
		 * abgefangen werden
		 */
		boolean b = false;
		switch (tab) {
		case 0:
		default:
			// Willkommen: nix zu prüfen
			b = true;
			break;
		case 1:
			// Anzahl Kläger und Beklagte
			try {
				klaegerZahl = ((SpinnerNumberModel) spiKlaegerZahl.getModel()).getNumber().intValue();
				// Integer.parseInt(spiKlaegerZahl.getText());
				beklagtenZahl = ((SpinnerNumberModel) spiBeklagtenZahl.getModel()).getNumber().intValue();
				// Integer.parseInt(spBeklagtenZahl.getText());
				if (klaegerZahl == 1 && beklagtenZahl > 0) {
					objectsKlaeger = new Object[klaegerZahl][4];
					objectsBeklagte = new Object[beklagtenZahl][4];
					sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER);
					tabKlaegerGenus.setModel(sbtmKlaeger);
					// ((BeteiligtenTable) tabKlaegerGenus).setWidth();
					sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE);
					tabBeklagtenGenus.setModel(sbtmBeklagte);
					// ((BeteiligtenTable) tabBeklagtenGenus).setWidth();
					b = true;
				} else
					b = false;
			} catch (IllegalArgumentException e) {
				if (e instanceof NumberFormatException)
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLENFORMAT_KL_BK,
							e);
				else
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_BEIM_ANLEGEN_KL_BK,
							e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 2:
			/* Bezeichnung von Kläger und Beklagten (nix zu beachten) */
			objectsKlaeger = sbtmKlaeger.getAllValues();
			objectsBeklagte = sbtmBeklagte.getAllValues();
			ftfBkStreitwert.setText(Double.toString(bkStreitwert));
			sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE, bkStreitwert);
			tabBeklagtenSw.setModel(sbtmBeklagte);
			chbBkSwEinheitlich.setSelected(false);
			chbBkSwEinheitlich.setSelected(true);
			// Kläger werden für eine der Widerklage-Tabs gebraucht!
			b = true;
			break;
		case 3:
			/*
			 * Streitwert der Beklagten (für alle einheitlich?) Prüfen, ob einheitlich oder
			 * nicht
			 */
			boolean bkEinheitlich = chbBkSwEinheitlich.isSelected();
			try {
				if (bkEinheitlich) {
					/*
					 * wenn einheitlich: speichern und auf alle Bk übernehmen und alle Werte ziehen
					 * --> für Panel 5
					 */
					bkStreitwert = doubleAusFormatter(ftfBkStreitwert, lblBkStreitwert.getText());// Double.parseDouble(ftfBkStreitwert.getText());
					sbtmBeklagte.setEachStreitwert(bkStreitwert, true);
					sbtmBeklagte.fireTableRowsUpdated(0, sbtmBeklagte.getRowCount() - 1);
					objectsBeklagte = sbtmBeklagte.getAllValues();
				} else {
					/*
					 * sonst: getValues und höchsten Streitwert für schlechte Tage zur Seite legen
					 * --> für den zurückzugebenden KostenTenorContainer
					 */
					objectsBeklagte = sbtmBeklagte.getAllValues();
					bkStreitwert = sbtmBeklagte.getHoechstenStreitwert();
					ftfBkStreitwert.setText(Double.toString(bkStreitwert));
				}
				/*
				 * Panel 4 vorbereiten: Array Beteiligter[], nur erste Spalte --> wird auch für
				 * Panel 6 gebraucht
				 */
				beteiligteBeklagte = sbtmBeklagte.getBeteiligterColumn();
				liBkGesamtschuldner.setListData(beteiligteBeklagte);
				ftfBkGesamtUnterliegen.setText(Double.toString(bkGesamtUnterliegen));
				/* Panel 5 schon einmal vorbereiten */
				sbtmBeklagte = new ScalableBeteiligtenTableModel(objectsBeklagte, Beteiligter.BEKLAGTE, bkStreitwert,
						bkUnterliegen);
				tbBkUnt.setModel(sbtmBeklagte);
				b = true;
			} catch (NumberFormatException e) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLENFORMAT_SW, e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 4:
			bkGesamt = chBkGesamtschuldner.isSelected();
			/*
			 * Prüfen: Häkchen bei gesamtschuldnerischer Verurteilung gesetzt? Wenn ja, dann
			 * muss das Unterliegen auch über 0 sein, sonst ist die Eingabe kappes.
			 */
			if (bkGesamt) {
				try {
					bkGesamtIndices = liBkGesamtschuldner.getSelectedIndices();
					bkGesamtUnterliegen = doubleAusFormatter(ftfBkGesamtUnterliegen, lblBkGesamtUnterliegen.getText());// Double.parseDouble(ftfBkGesamtUnterliegen.getText());
					if (bkGesamtIndices != null && bkGesamtIndices.length > 1 && bkGesamtUnterliegen >= 0
							&& bkGesamtUnterliegen <= bkStreitwert)
						b = true;
					else {
						FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
								HinzufuegenDialog.FEHLER_BK_GESAMTSCHULDNERSCHAFT,
								new IllegalArgumentException(HinzufuegenDialog.FEHLER_BK_GESAMTSCHULDNERSCHAFT));
					}
				} catch (NumberFormatException e) {
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLENFORMAT, e);
					// e.printStackTrace();
				}
			} else {
				// sonst: alles egal, keine Plausi zu prüfen
				b = true;
			}
			break;
		case 5:
			/*
			 * Wurden einige Beklagte darüber hinaus verurteilt? Welche, zu wie viel?
			 */
			/*
			 * Prüfen, ob einer der Beteiligten ein höheres Unterliegen als der streitwert
			 * hat (ggfs. zusammen mit der gesamtschuldnerischen Verurteilung)
			 */
			b = true;
			for (int row = 0; row < sbtmBeklagte.getRowCount(); row++) {
				double d = ((Double) sbtmBeklagte.getValueAt(row, 3)).doubleValue();
				if (d > bkStreitwert)
					b = false;
				if (chBkGesamtschuldner.isSelected()
						// && Arrays.asList(bkGesamtIndices).contains(row)
						&& contains(bkGesamtIndices, row) && (d + bkGesamtUnterliegen) > bkStreitwert)
					b = false;
			}
			if (!b)
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT,
						new IllegalArgumentException(HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT));
			else {
				/*
				 * Für die Rückgabe muss anWiderklageBeteiligt erst einmal auf false gesetzt
				 * werden, wo es nicht gesetzt wurde
				 */
				sbtmBeklagte.setEachAnwiderklagebeteiligt(false, false);
				// Dann Werte auslesen
				objectsBeklagte = sbtmBeklagte.getAllValues();
				if (isMitWiderklage()) {
					// Wenn Widerklage
					if (getVerfahrensDatenContainer() == null) {
						// (und sofern nicht xjustiz-import) Panel 6 bauen
						liBkWk.setListData(beteiligteBeklagte);
						// spiDwbkAnz.setText(Integer.toString(dwbkZahl));
						((SpinnerNumberModel) spiDwbkAnz.getModel()).setValue(new Integer(dwbkZahl));
					} else {
						// Wenn xjustiz-import: nix, Panel 6 schon angelegt
					}
				} else {
					// wenn keine Widerklage vorhanden => hier fertig!
				}
			}
			break;
		case 6:
			// Welcher Beklagte klagt? Wie viele Drittwiderbeklagten?
			try {
				bkWiderklaegerIndex = liBkWk.getSelectedIndex();
				dwbkZahl = ((SpinnerNumberModel) spiDwbkAnz.getModel()).getNumber().intValue();
				// Integer.parseInt(spiDwbkAnz.getText());
				setKlWk(chbKlWk.isSelected());
				/*
				 * Mindestens ein Bk muss ausgewählt sein, dwbkZahl darf nicht negativ sein und
				 * wenn schon mitWiderklage, dann muss der Kläger widerbeklagt sein oder
				 * wenigstens ein Dwbk vorhanden sein
				 */
				if (bkWiderklaegerIndex > -1 && dwbkZahl > -1 && (isKlWk() || dwbkZahl > 0)) {
					/*
					 * Beim markierten Beklagten die vierte Spalte (index 3, anWiderklageBeteiligt)
					 * auf true setzen. Hier gehe ich direkt über die Werte und nicht den Umweg über
					 * ein sbtm, weil an dieser Stelle keine Tabelle angezeigt wird.
					 */
					objectsBeklagte[bkWiderklaegerIndex][3] = Boolean.TRUE;
					/*
					 * Schreiben, ob Wk auch gegen den Kläger geht. Auch hier direkt über die Werte,
					 * weil eine Tabelle dazu gerade nicht angezeigt wird.
					 */
					objectsKlaeger[0][3] = new Boolean(isKlWk());
					objectsDwbk = new Object[dwbkZahl][4];
					sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk,
							Beteiligter.DRITTWIDERBEKLAGTE);
					// Drittwiderbeklagte sind immer an der Wk beteiligt
					sbtmDrittwiderbeklagte.setEachAnwiderklagebeteiligt(true, false);
					tbDwbkGenus.setModel(sbtmDrittwiderbeklagte);
					b = true;
				} else {
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
							HinzufuegenDialog.FEHLER_NIX_WIDERKLAEGER_WIDERBEKLAGTER,
							new IllegalArgumentException(HinzufuegenDialog.FEHLER_NIX_WIDERKLAEGER_WIDERBEKLAGTER));
					b = false;
				}
			} catch (NumberFormatException e) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLFORMAT_DWBK, e);
				// e.printStackTrace();
				b = false;
			}
			break;
		case 7:
			/* Bezeichnung der Drittwiderbeklagten (nix zu prüfen) */
			/*
			 * Denkt auch daran, dass man Dwbk auch weglassen kann
			 */
			if (dwbkZahl > 0) {
				objectsDwbk = sbtmDrittwiderbeklagte.getAllValues();
				sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk, Beteiligter.DRITTWIDERBEKLAGTE,
						getWkStreitwert());
				tabDwbkSw.setModel(sbtmDrittwiderbeklagte);
			} else {
				// schon gut
			}
			b = true;
			/*
			 * Für Panel 8 werden dann wieder die Klägerdaten gebraucht: angezeigt werden
			 * sollen sie immer. Der ChangeListener der checkbox sorgt dafür, dass die
			 * Kläger nur bearbeitet werden können, wenn der Kläger was mit der Wk am Hut
			 * hat.
			 */
			if (isKlWk()) {
				sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER, getWkStreitwert());
			} else {
				sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER, 0);
			}
			tabKlSw.setModel(sbtmKlaeger);
			/*
			 * Einmal den ChangeListener aufmerksam machen, der schreibt auch den
			 * wkStreitwert ins Textfeld.
			 */
			chbWkEinheitlich.setSelected(false);
			chbWkEinheitlich.setSelected(true);
			break;
		case 8:
			/*
			 * Wurden Widerbeklagte in derselben Höhe verklagt? Welche (jeweils): Hier
			 * achtet die Prüfung immer darauf, dass es einen Kläger geben kann, der nicht
			 * widerbeklagt ist bzw. dass es möglicherweise keine Dwbk gibt.
			 */
			boolean wkEinheitlich = chbWkEinheitlich.isSelected();
			try {
				if (wkEinheitlich) {
					/* wenn einheitlich */
					setWkStreitwert(doubleAusFormatter(ftfWkSw, lblWkSw.getText()));// Double.parseDouble(ftfWkSw.getText()));
					if (isKlWk()) {
						// Wenn nicht bei Wk dabei, dann auch nichts setzen
						sbtmKlaeger.setEachStreitwert(getWkStreitwert(), true);
						objectsKlaeger = sbtmKlaeger.getAllValues();
					}
					if (dwbkZahl > 0) {
						// Man kann Dwbk auch weglassen
						sbtmDrittwiderbeklagte.setEachStreitwert(getWkStreitwert(), true);
						objectsDwbk = sbtmDrittwiderbeklagte.getAllValues();
					}
				} else {
					/* wenn nicht einheitlich */
					double klWkStreitwert = 0;
					double dwbkStreitwert = 0;
					if (isKlWk()) {
						// Wenn nicht bei Wk dabei, dann auch nichts lesen
						objectsKlaeger = sbtmKlaeger.getAllValues();
						klWkStreitwert = sbtmKlaeger.getHoechstenStreitwert();
					}
					if (dwbkZahl > 0) {
						// Man kann Dwbk auch weglassen
						objectsDwbk = sbtmDrittwiderbeklagte.getAllValues();
						dwbkStreitwert = sbtmDrittwiderbeklagte.getHoechstenStreitwert();
					}
					// Höheren Streitwert von beiden speichern
					setWkStreitwert((klWkStreitwert > dwbkStreitwert) ? klWkStreitwert : dwbkStreitwert);
					ftfWkSw.setText(Double.toString(getWkStreitwert()));
				}
				if (getWkStreitwert() <= 0) {
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_WK_STREITWERT,
							new IllegalArgumentException(HinzufuegenDialog.FEHLER_WK_STREITWERT));
					b = false;
					break;
				} else {
					b = true;
				}
				/*
				 * Immer eine Liste aus Kl (auch wenn nicht wbk) und Dwbk bauen, damit die
				 * Indexe beim Bauen einer BaumbachGesamtschuldnerschaft stimmen. Umständlicher
				 * Weg mit einer for-each-Schleife, aber Arrays.asList() gibt eine
				 * unveränderliche Liste zurück, sodass der erste Weg nicht geklappt hat...
				 */
				ArrayList<Beteiligter> tempWiderbeklagte = new ArrayList<Beteiligter>();
				for (Beteiligter bb : sbtmKlaeger.getBeteiligterColumn())
					tempWiderbeklagte.add(bb);
				for (Beteiligter bb : sbtmDrittwiderbeklagte.getBeteiligterColumn())
					tempWiderbeklagte.add(bb);
				beteiligteWiderbeklagte = tempWiderbeklagte.toArray(new Beteiligter[tempWiderbeklagte.size()]);
				liWkGesamtschuldner.setListData(beteiligteWiderbeklagte);
				ftfWkUnterliegen.setText(Double.toString(getWkGesamtUnterliegen()));
				if (isKlWk()) {
					sbtmKlaeger = new ScalableBeteiligtenTableModel(objectsKlaeger, Beteiligter.KLAEGER,
							getWkStreitwert(), wkUnterliegen);
					tbKlDarueber.setModel(sbtmKlaeger);
					tbKlDarueber.setEnabled(true);
				} else {
					/*
					 * Wenn Kl nicht wbk, dann darf man für ihn auch nichts eingeben
					 */
					tbKlDarueber.setEnabled(false);
				}
				if (dwbkZahl > 0) {
					sbtmDrittwiderbeklagte = new ScalableBeteiligtenTableModel(objectsDwbk,
							Beteiligter.DRITTWIDERBEKLAGTE, getWkStreitwert(), wkUnterliegen);
					tbDwbkDarueber.setModel(sbtmDrittwiderbeklagte);
					tbDwbkDarueber.setEnabled(true);
				} else {
					tbDwbkDarueber.setEnabled(false);
				}
				chbWkGesamtschuld.setSelected(false);
				chbWkGesamtschuld.setSelected(true);
				chbWkGesamtschuld.setSelected(beteiligteWiderbeklagte.length > 1);
			} catch (NumberFormatException e) {
				b = false;
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLENFORMAT_SW, e);
				// e.printStackTrace();
			}
			break;
		case 9:
			/*
			 * Wurden die Widerbeklagten z.T. gesamtschuldnerisch verurteilt? Welche
			 * Widerbeklagten, in welcher Höhe? Prüfen: Häkchen bei gesamtschuldnerischer
			 * Verurteilung gesetzt? Wenn ja, dann muss das Unterliegen auch über 0 sein,
			 * sonst ist die Eingabe kappes.
			 */
			wkGesamt = chbWkGesamtschuld.isSelected();
			if (wkGesamt) {
				try {
					wkGesamtIndices = liWkGesamtschuldner.getSelectedIndices();
					setWkGesamtUnterliegen(doubleAusFormatter(ftfWkUnterliegen, lblWkUnterliegen.getText()));// Double.parseDouble(ftfWkUnterliegen.getText()));
					if (wkGesamtIndices != null && wkGesamtIndices.length > 1 && getWkGesamtUnterliegen() > 0
							&& getWkGesamtUnterliegen() <= getWkStreitwert())
						b = true;
					else {
						b = false;
						FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
								HinzufuegenDialog.FEHLER_WK_GESAMTSCHULDNERSCHAFT,
								new IllegalArgumentException(HinzufuegenDialog.FEHLER_WK_GESAMTSCHULDNERSCHAFT));
					}
					/*
					 * Wenn der Kläger nicht widerbeklagt ist, darf er nicht ausgewählt werden
					 */
					if (!isKlWk() && contains(wkGesamtIndices, 0)) {
						b = false;
						FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_KLAEGER_NIX_WBK,
								new IllegalArgumentException(HinzufuegenDialog.FEHLER_KLAEGER_NIX_WBK));
					}
				} catch (NumberFormatException e) {
					b = false;
					FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_ZAHLENFORMAT, e);
					// e.printStackTrace();
				}
			} else {
				/*
				 * Wenn keine Gesamtschuldnerschaft angehakt, keine Plausis zu prüfen.
				 */
				b = true;
			}
			/* Panel 10 ist schon unter case 8 gebaut worden. */
			break;
		case 10:
			/*
			 * Wurden einzelne Widerbeklagten darüber hinaus einzeln verurteilt? Welche, in
			 * welcher Höhe?
			 */

			/*
			 * Prüfen, ob einer der Beteiligten ein höheres Unterliegen als der streitwert
			 * hat (ggfs. zusammen mit der gesamtschuldnerischen Verurteilung)
			 */
			b = true;
			/*
			 * Wenn Kl nicht wbk, dann darf man für ihn auch nichts eingeben, sonst prüfen,
			 * ob Unterliegen den Streitwert überschreitet
			 */
			if (isKlWk())
				for (int row = 0; row < sbtmKlaeger.getRowCount(); row++) {
					double d = ((Double) sbtmKlaeger.getValueAt(row, 3)).doubleValue();
					if (d > getWkStreitwert())
						b = false;
					else if (chbWkGesamtschuld.isSelected() && contains(wkGesamtIndices, row)
							&& (d + getWkGesamtUnterliegen()) > getWkStreitwert())
						b = false;
				}
			/* Dran denken, dass man Dwbk auch weglassen kann */
			if (dwbkZahl > 0)
				for (int row = 0; row < sbtmDrittwiderbeklagte.getRowCount(); row++) {
					int absRow = row + klaegerZahl;
					double d = ((Double) sbtmDrittwiderbeklagte.getValueAt(row, 3)).doubleValue();
					if (d > getWkStreitwert())
						b = false;
					else if (chbWkGesamtschuld.isSelected() && contains(wkGesamtIndices, absRow)
							&& (d + getWkGesamtUnterliegen()) > getWkStreitwert())
						b = false;
				}
			if (!b)
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT,
						new IllegalArgumentException(HinzufuegenDialog.FEHLER_UNTERLIEGEN_GT_STREITWERT));
			// Wenn Widerklage erhoben: hier fertig!
			break;
		}
		return b;
	}

	/**
	 * Die Methode sorgt dafür, dass bei Xjustiz-Import die nicht benötigten
	 * Panels/Tabs auch nicht stören. Zwar kann man diese Tabs nicht komplett
	 * ausblenden, aber zumindest werden die Titles auf null gesetzt, so dass sie
	 * leer sind. Der Assistent erscheint dadurch nicht mehr so groß.
	 * 
	 */
	private void panelsAusblendenBeiImport() {
		if (getVerfahrensDatenContainer() == null)
			return;
		panels[1].setVisible(false);
		tabbedPane.setTitleAt(1, null);
		panels[2].setVisible(false);
		tabbedPane.setTitleAt(2, null);
		if (isMitWiderklage()) {
			tabbedPane.setTitleAt(6, null);
			panels[6].setVisible(false);
			tabbedPane.setTitleAt(7, null);
			panels[7].setVisible(false);
		}
	}

	/**
	 * Die Methode prüft, ob in einem Array int[] ein bestimmter int enthalten ist.
	 * 
	 * @param indices das zu durchsuchende Array int[]
	 * @param index   der int-Wert, der enthalten sein könnte
	 * @return true, wenn index in indices enthalten ist.
	 */
	private boolean contains(int[] indices, int index) {
		boolean contains = false;
		for (int i : indices)
			if (i == index)
				contains = true;
		return contains;
	}

	/**
	 * Konvertiert einen BaumbachBeteiligten in ein Object-Array aus Beteiligter,
	 * null, null, Boolean. Streitwert und Unterliegen werden hier bewusst mit null
	 * aufgefüllt, weil die auf jeden Fall vom Anwender gesetzt werden sollen.
	 * 
	 * @param bb ein BaumbachBeteiligter
	 * @return ein Object-Array aus Beteiligter, null, null, Boolean
	 */
	private Object[] konvertiere(BaumbachBeteiligter bb) {
		if (bb != null)
			return new Object[] { new Beteiligter(bb), (Double) null, (Double) null,
					new Boolean(bb.isAnWiderklageBeteiligt()) };
		else
			return null;
	}

	/**
	 * Konvertiert ein Array Object[4] aus Beteiligter, Double, Double, Boolean in
	 * einen BaumbachBeteiligten und liest dabei alles aus.
	 * 
	 * @param obb ein Array Object[4] aus Beteiligter, Double, Double, Boolean
	 * @return einen BaumbachBeteiligten
	 */
	private BaumbachBeteiligter konvertiere(Object[] obb) {
		if (obb != null && obb.length == 4) {
			double sw = obb[1] == null ? 0.0 : ((Double) obb[1]).doubleValue();
			double unt = obb[2] == null ? 0.0 : ((Double) obb[2]).doubleValue();
			boolean isWk = obb[3] == null ? false : ((Boolean) obb[3]).booleanValue();
			return new BaumbachBeteiligter((Beteiligter) obb[0], sw, unt, isWk);
		} else
			return null;
	}

	/**
	 * Konvertiert eine BaumbachBeteiligtenListe in ein Array Object[][] aus
	 * Beteiligtendaten
	 * 
	 * @param bbl eine BaumbachBeteiligtenListe
	 * @return ein Array Object[][] aus Beteiligtendaten
	 */
	private Object[][] konvertiere(BaumbachBeteiligtenListe bbl) {
		if (bbl != null) {
			Object[][] zwischen = new Object[bbl.size()][4];
			for (int index = 0; index < bbl.size(); index++) {
				Object[] o = konvertiere(bbl.get(index));
				if (o != null)
					zwischen[index] = o;
			}
			return zwischen;
		} else
			return null;
	}

	/**
	 * Konvertiert ein Array Object[][] aus Beteiligtendaten in eine
	 * BaumbachBeteiligtenListe
	 * 
	 * @param obbl ein Array Object[][] aus Beteiligtendaten, bei dem für jede Zeile
	 *             die Anforderungen von
	 *             {@link eu.gronos.kostenrechner.view.baumbach.BaumbachAssistentDialog#konvertiere(Object[])
	 *             konvertiere(Object[])} erfüllt sind
	 * @return eine BaumbachBeteiligtenListe
	 */
	private BaumbachBeteiligtenListe konvertiere(Object[][] obbl) {
		if (obbl != null) {
			BaumbachBeteiligtenListe zwischen = new BaumbachBeteiligtenListe();
			for (Object[] o : obbl) {
				BaumbachBeteiligter bb = konvertiere(o);
				if (bb != null)
					zwischen.add(bb);
			}
			return zwischen;
		} else
			return null;
	}

	/**
	 * @param container d. {@link #verfahrensDatenContainer}, d. gesetzt werden soll
	 *                  als VerfahrensDatenContainer
	 */
	private void setContainer(VerfahrensDatenContainer container) {
		this.verfahrensDatenContainer = container;
	}

	/**
	 * @param mitWiderklage d. {@link #mitWiderklage}, d. gesetzt werden soll als
	 *                      boolean
	 */
	private void setMitWiderklage(boolean mitWiderklage) {
		this.mitWiderklage = mitWiderklage;
	}

	/**
	 * @param wkStreitwert d. {@link #wkStreitwert}, d. gesetzt werden soll als
	 *                     double
	 */
	private void setWkStreitwert(double wkStreitwert) {
		this.wkStreitwert = wkStreitwert;
	}

	/**
	 * @param klWk d. {@link #klWk}, d. gesetzt werden soll als boolean
	 */
	private void setKlWk(boolean klWk) {
		this.klWk = klWk;
	}

	/**
	 * @param wkGesamtUnterliegen d. {@link #wkGesamtUnterliegen}, d. gesetzt werden
	 *                            soll als double
	 */
	private void setWkGesamtUnterliegen(double wkGesamtUnterliegen) {
		this.wkGesamtUnterliegen = wkGesamtUnterliegen;
	}
}
