package eu.gronos.kostenrechner.view.baumbach;

import java.text.NumberFormat;

import javax.swing.DefaultCellEditor;
import javax.swing.JComboBox;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.table.TableModel;
import javax.swing.text.NumberFormatter;

import eu.gronos.kostenrechner.model.baumbach.ScalableBeteiligtenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.view.EuroEditor;
import eu.gronos.kostenrechner.view.EuroRenderer;

public class SimpleBeteiligtenTable extends JTable {
	private static final long serialVersionUID = -8877232808960976183L;
	private static NumberFormat format = NumberFormat.getInstance();
	static {
		format.setGroupingUsed(false);
	}
	private static NumberFormatter formatter = new NumberFormatter(format);

	/**
	 * Eine einfache Erweiterung von JTable, um Beteiligtendaten anzuzeigen und
	 * zu bearbeiten
	 * 
	 * @param dm
	 *            ein {@link ScalableBeteiligtenTableModel}
	 * @param beteiligtenTyp
	 *            eine der Konstanten {@link Beteiligter#KLAEGER},
	 *            {@link Beteiligter#BEKLAGTE} oder
	 *            {@link Beteiligter#DRITTWIDERBEKLAGTE}
	 */
	protected SimpleBeteiligtenTable(TableModel dm, int beteiligtenTyp) {
		super(dm);
		JComboBox<Beteiligter> comboBox = new JComboBox<Beteiligter>(
				Beteiligter.getBeteiligtenAuswahlListe(beteiligtenTyp, false));
		comboBox.setRenderer(new BeteiligtenRenderer(true));
		setDefaultEditor(Beteiligter.class, new DefaultCellEditor(comboBox));
		setDefaultRenderer(Beteiligter.class, new BeteiligtenRenderer());
		setDefaultRenderer(Double.class, new EuroRenderer());
		setDefaultEditor(Double.class, new EuroEditor(formatter));
		setWidth();
		setSelectionModel();
	}

	/**
	 * Setzt das {@link TableModel} für die {@link JTable} und dabei auch eine
	 * schöne Breite für die Spalten.
	 * 
	 * @param dm
	 *            ein {@link ScalableBeteiligtenTableModel}
	 * 
	 * @see javax.swing.JTable#setModel(javax.swing.table.TableModel)
	 * @see #setWidth()
	 */
	@Override
	public void setModel(TableModel dm) {
		super.setModel(dm);
		setWidth();
	}

	/**
	 * Setzt die Standardwerte für Auswahlmodus usw.
	 * 
	 */
	void setSelectionModel() {
		setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		setRowSelectionAllowed(true);
		setColumnSelectionAllowed(false);
	}

	/**
	 * Setzt eine angenehme Breite für die erste Spalte
	 * 
	 */
	void setWidth() {
		int maxWidth = getColumnModel().getColumn(1).getWidth() / 2;
		if (maxWidth < getColumnModel().getColumn(1).getMinWidth())
			maxWidth = getColumnModel().getColumn(1).getMinWidth();
		getColumnModel().getColumn(0).setMaxWidth(maxWidth);
	}

}