/**
 * BaumbachPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.baumbach;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.KeyEvent;

import javax.swing.AbstractAction;
import javax.swing.DefaultCellEditor;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.EintragEntfernenAktion;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenAssistentAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenBaumbachAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenBeteiligterNeuAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenGesamtschuldBkExtractAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenGesamtschuldDwbkExtractAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenGesamtschuldnerBkAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenGesamtschuldnerDwbkAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenWiderklageAction;
import eu.gronos.kostenrechner.controller.baumbach.StreitgenossenXJustizLesenAction;
import eu.gronos.kostenrechner.interfaces.StreitgenossenTabellenBehandelnd;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligtenListe;
import eu.gronos.kostenrechner.model.baumbach.BeteiligtenTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Streitgenossen;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.KostenJPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Diese von {@link KostenJPanel} abgeleitete Klasse nimmt die
 * Oberflächenelemente für die Berechnung der Baumbachschen Formel auf.
 *
 * @author Peter Schuster (setrok)
 * @date 13.06.2018
 *
 */
public class StreitgenossenPanel extends KostenJPanel implements WerteLesbar<VerfahrensDatenContainer.Streitgenossen>,
		StreitgenossenTabellenBehandelnd  {

	private static final long serialVersionUID = 1265694381131490890L;
	private static final StreitgenossenPanel instance = new StreitgenossenPanel();
	private Kostenrechner kostenrechner;
	private boolean widerKlageAnzeigen;
	public JTable tabKlaeger;
	public BeteiligtenTableModel btmKlaeger;
	public JTable tabBeklagte;
	public BeteiligtenTableModel btmBeklagte;

	private final AbstractAction neuerKlaegerAction = new StreitgenossenBeteiligterNeuAction(this, Beteiligter.KLAEGER,
			"Kläger neu", "<html><h1>Kläger neu</h1><p>Fügt einen neuen Kläger hinzu</p></p></html>",
			KeyEvent.VK_K, "klaeger-neu");
	private final AbstractAction neuerBeklagterAction = new StreitgenossenBeteiligterNeuAction(this, Beteiligter.BEKLAGTE,
			"Beklagter neu", "<html><h1>Beklagter neu</h1><p>Fügt einen neuen (einzelnen) Beklagten hinzu.</p></p></html>",
			KeyEvent.VK_B,
			"beklagter-neu");
	private final AbstractAction neuerDrittwiderbeklagterAction = new StreitgenossenBeteiligterNeuAction(this, Beteiligter.DRITTWIDERBEKLAGTE,
			"Drittwiderbeklagter neu", "<html><h1>Drittwiderbeklagter neu</h1><p>Fügt einen neuen Drittwiderbeklagten hinzu.</p></p></html>",
			KeyEvent.VK_D,
			"drittwiderbeklagter-neu");
	public JTable tabDrittwiderbeklagte;
	public BeteiligtenTableModel btmDrittwiderbeklagte;
	private JScrollPane scpDwbk;
	private JButton btnWiderklageAnzeigen;
	private JButton btnDrittwiderbeklagterNeu;
	private JButton btnKlDwblGesamtsch;
	private final AbstractAction klDwbkGesamtschAction = new StreitgenossenGesamtschuldnerDwbkAction(this,
			"Gesamtschuldnerische Kl./Drwbk. neu",
			"<html><h1>Gesamtschuldnerische Kläger und Drittwiderbeklagte neu</h1><p>Fügt eine gesamtschuldnerische Verurteilung auf Kläger-/Drittwiderbeklagtenseite hinzu."
					+ "<br>Vorher bitte die einzelnen Beteiligten hinzufügen.</p></html>",
			KeyEvent.VK_L, "beklagte-gesamtschuldner");
	private final AbstractAction widerKlageAktivierenAction = new StreitgenossenWiderklageAction(this,
			"Widerklagen aktivieren",
			"<html><h1>Widerklage</h1><p>Aktiviert die Eingabeschnittstellen für Widerklagen</p></html>", KeyEvent.VK_W,
			"widerklage-aktivieren");
	private AbstractAction bkGesamtschuldnerAction = new StreitgenossenGesamtschuldnerBkAction(this,
			"Gesamtschuldnerische Beklagte neu",
			"<html><h1>Gesamtschuldnerische Beklagten neu</h1><p>Fügt eine gesamtschuldnerische Verurteilung zur Tabelle der Beklagten hinzu.</p></html>",
			KeyEvent.VK_G, "gesamtschuldnerschaft-beklagte");
	private final AbstractAction baumbachAction = new StreitgenossenBaumbachAction(this,
			"Tenor ermitteln (Baumbach'sche Kostenformel)", // "Baumbach'schen Kostentenor berechnen",
			"<html><h1>Baumbach'schen Kostentenor berechnen</h1><p>Erzeugt einen Kostentenor nach der Baumbach'schen Kostenformel.</p></html>",
			KeyEvent.VK_T, "baumbach-tenor");
	private JPopupMenu pmBeklagtenKontextmenu;
	private JPopupMenu pmKlaegerKontextmenu;
	private JPopupMenu pmDwbkKontextmenu;
	public final AbstractAction gesamtschuldnerschaftExtrahierenAction = new StreitgenossenGesamtschuldBkExtractAction(
			this, "Gesamtschuldnerische Verurteilung ermitteln",
			"Geht die Beteiligten einer Seite durch und guckt, ob sie z.T. gesamtschuldnerisch verurteilt wurden",
			KeyEvent.VK_V, "extrahiere-gesamtschuld");

	public final AbstractAction gesamtschuldnerschaftDwbkExtrahierenAction = new StreitgenossenGesamtschuldDwbkExtractAction(
			this, "Gesamtschuldnerische Verurteilung (Widerbeklagte) ermitteln",
			"Geht die Beteiligten einer Seite durch und guckt, ob sie z.T. gesamtschuldnerisch verurteilt wurden",
			KeyEvent.VK_W, "extrahiere-widerklage-gesamtschuld");
	public AbstractAction baumbachAssistentAction = new StreitgenossenAssistentAction(this,
			"Beteiligte anlegen über Assistenten",
			"Der Assistent vereinfacht das Hinzufügen von Beteiligten durch eine geführte Abfrage der Daten.",
			KeyEvent.VK_A, "baumbach-assistent");
	private boolean wirtschaftlichIdentisch = false;
	public static final String FEHLER_BERECHNUNG = "Fehler bei der Berechnung";
	public AbstractAction xmlLesenAction = new StreitgenossenXJustizLesenAction(this,
			getKostenrechner(), "XJustiz importieren über Assistenten", "Liest einen XJustiz-Datensatz (XML) ein", KeyEvent.VK_X,
			"xjustiz", KeyStroke.getKeyStroke(KeyEvent.VK_O, Kostenrechner.ctrlMask + ActionEvent.SHIFT_MASK));
	private static final String FEHLER_GESAMTSCHULD_NIX_LOESCHEN = "Einträge können nicht entfernt werden, wenn bereits eine Gesamtschuldnerschaft vorhanden ist.";
	public static final String FEHLER_GESAMTSCHULDNERSCHAFT = "Fehler beim Hinzufügen einer Gesamtschuldnerschaft";
	public static final String FEHLER_NIX_PARTEIEN = "Fehler: Keine Kläger und/oder Beklagten vorhanden";
	public static final String FEHLER_NIX_PARTEIEN_LANG = "Die Berechnung erfordert einen Kläger und mindestens einen Beklagten.";
	private static final WiderklageTableRendererEditor WIDERKLAGE_RENDERER_EDITOR = new WiderklageTableRendererEditor();
	public static final BeteiligtenRenderer BETEILIGTEN_RENDERER_OHNE_LFDNR = new BeteiligtenRenderer(true);
	public static final BeteiligtenRenderer BETEILIGTEN_RENDERER = new BeteiligtenRenderer();
	public static final String DRITTWIDERBEKLAGTE_POPUPLABEL = "Drittwiderbeklagte...";
	public static final String KLAEGER_POPUPLABEL = "Kläger...";
	public static final String BEKLAGTE_POPUPLABEL = "Beklagte...";

	/**
	 * Der Konstruktor ruft nur die Oberklasse auf.
	 * 
	 */
	private StreitgenossenPanel() {
		super();
	}

	/**
	 * @return gibt {@link #instance} als BaumbachPanel zurück.
	 */
	public static StreitgenossenPanel getInstance() {
		return instance;
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 2
	 * Streitgenossen
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		setWiderKlageAnzeigen(false);

		leereWerte();

		tabKlaeger = new JTable(btmKlaeger);
		setzeBeteiligtenRenderer(tabKlaeger);
		JScrollPane scpKlaeger = new JScrollPane(tabKlaeger);
		add(scpKlaeger, GitterBeutelBeschraenkungen.getInstance(0, 0, 2, 1, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabKlaeger);

		tabBeklagte = new JTable(btmBeklagte);
		setzeBeteiligtenRenderer(tabBeklagte);
		JScrollPane scpBeklagte = new JScrollPane(tabBeklagte);
		add(scpBeklagte, GitterBeutelBeschraenkungen.getInstance(2, 0, 2, 3, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabBeklagte);

		JButton btnKlgerNeu = new JButton(neuerKlaegerAction);
		add(btnKlgerNeu, GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		tabDrittwiderbeklagte = new JTable(btmDrittwiderbeklagte);
		setzeBeteiligtenRenderer(tabDrittwiderbeklagte);
		scpDwbk = new JScrollPane(tabDrittwiderbeklagte);
		add(scpDwbk, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, false));
		entferneTastaturFalle(tabDrittwiderbeklagte);

		btnDrittwiderbeklagterNeu = new JButton(neuerDrittwiderbeklagterAction);
		add(btnDrittwiderbeklagterNeu,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBeklagterNeu = new JButton(neuerBeklagterAction);
		add(btnBeklagterNeu, GitterBeutelBeschraenkungen.getInstance(2, 3, 2, 1, GridBagConstraints.HORIZONTAL, false));

		btnKlDwblGesamtsch = new JButton(klDwbkGesamtschAction);
		add(btnKlDwblGesamtsch,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		btnWiderklageAnzeigen = new JButton(widerKlageAktivierenAction);
		add(btnWiderklageAnzeigen,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		setWiderKlageAnzeigen(false);

		JButton btnBkGesamtsch = new JButton(bkGesamtschuldnerAction);
		add(btnBkGesamtsch, GitterBeutelBeschraenkungen.getInstance(2, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBerechnen = new JButton(baumbachAction);
		add(btnBerechnen, GitterBeutelBeschraenkungen.getInstance(0, 5, 4, 1, GridBagConstraints.HORIZONTAL, false));

		pmBeklagtenKontextmenu = new JPopupMenu(BEKLAGTE_POPUPLABEL);
		pmBeklagtenKontextmenu.add(new JMenuItem(BEKLAGTE_POPUPLABEL));
		pmBeklagtenKontextmenu.addSeparator();
		pmBeklagtenKontextmenu.add(new JMenuItem(neuerBeklagterAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(bkGesamtschuldnerAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(gesamtschuldnerschaftExtrahierenAction));
		pmBeklagtenKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabBeklagte.addMouseListener(new KontextMouseListener(tabBeklagte, pmBeklagtenKontextmenu));

		pmKlaegerKontextmenu = new JPopupMenu(KLAEGER_POPUPLABEL);
		pmKlaegerKontextmenu.add(new JMenuItem(KLAEGER_POPUPLABEL));
		pmKlaegerKontextmenu.addSeparator();
		pmKlaegerKontextmenu.add(new JMenuItem(neuerKlaegerAction));
		pmKlaegerKontextmenu.add(new JMenuItem(klDwbkGesamtschAction));
		pmKlaegerKontextmenu.add(new JMenuItem(gesamtschuldnerschaftDwbkExtrahierenAction));
		pmKlaegerKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabKlaeger.addMouseListener(new KontextMouseListener(tabKlaeger, pmKlaegerKontextmenu));

		pmDwbkKontextmenu = new JPopupMenu(DRITTWIDERBEKLAGTE_POPUPLABEL);
		pmDwbkKontextmenu.add(new JMenuItem(DRITTWIDERBEKLAGTE_POPUPLABEL));
		pmDwbkKontextmenu.addSeparator();
		pmDwbkKontextmenu.add(new JMenuItem(neuerDrittwiderbeklagterAction));
		pmDwbkKontextmenu.add(new JMenuItem(klDwbkGesamtschAction));
		pmDwbkKontextmenu.add(new JMenuItem(gesamtschuldnerschaftDwbkExtrahierenAction));
		pmDwbkKontextmenu.add(new JMenuItem(eintragEntfernenAktion));
		tabDrittwiderbeklagte.addMouseListener(new KontextMouseListener(tabDrittwiderbeklagte, pmDwbkKontextmenu));
		// setTransferHandler(new XmlTransferHandler(this));
	}

	/**
	 * Die Methode leert die Baumbach-Tabellen und füllt sie mit den Daten aus dem
	 * Untercontainer
	 * {@link eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Streitgenossen}
	 * 
	 * @param container ein Untercontainer
	 *                  {@link eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Streitgenossen}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.StreitgenossenTabellenBehandelnd#setzeWerte(eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Streitgenossen)
	 */
	@Override
	public void setzeWerte(final VerfahrensDatenContainer.Streitgenossen container) {
		setWiderKlageAnzeigen(container.widerklage);
		setWirtschaftlichIdentisch(container.wirtschaftlichIdentisch);
		btmKlaeger = new BeteiligtenTableModel(Beteiligter.KLAEGER, isWiderKlageAnzeigen(), container.klaeger);
		tabKlaeger.setModel(btmKlaeger);
		btmDrittwiderbeklagte = new BeteiligtenTableModel(Beteiligter.DRITTWIDERBEKLAGTE, isWiderKlageAnzeigen(),
				container.drittwiderbeklagte);
		tabDrittwiderbeklagte.setModel(btmDrittwiderbeklagte);
		btmBeklagte = new BeteiligtenTableModel(Beteiligter.BEKLAGTE, isWiderKlageAnzeigen(), container.beklagte);
		tabBeklagte.setModel(btmBeklagte);
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return den Untercontainer von
	 *         {@link VerfahrensDatenContainer#streitgenossen}
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#leseWerte()
	 */
	@Override
	public Streitgenossen leseWerte() {
		Streitgenossen container = new Streitgenossen();
		container.widerklage = isWiderKlageAnzeigen();
		container.wirtschaftlichIdentisch = isWirtschaftlichIdentisch();
		container.klaeger = btmKlaeger.getAllValues();
		if (isWiderKlageAnzeigen())
			container.drittwiderbeklagte = btmDrittwiderbeklagte.getAllValues();
		else
			container.drittwiderbeklagte = new BaumbachBeteiligtenListe();
		container.beklagte = btmBeklagte.getAllValues();
		return container;
	}

	/**
	 * Die Methode setWiderklageAnzeigen dient zum Aktivieren der Widerklagen. Bei
	 * allen TableModels widerKlageAnzeigen setzen, Dwbk-Tabelle und Schaltflächen
	 * wieder auf Visible setzen, eigene Schaltflächen ausblenden.
	 * 
	 * @param widerKlageAnzeigen der Wert für widerKlageAnzeigen, der gesetzt werden
	 *                           soll.
	 */
	public void setWiderKlageAnzeigen(boolean widerKlageAnzeigen) {
		this.widerKlageAnzeigen = widerKlageAnzeigen;
		if (scpDwbk != null)
			scpDwbk.setVisible(widerKlageAnzeigen);
		if (btnDrittwiderbeklagterNeu != null)
			btnDrittwiderbeklagterNeu.setVisible(widerKlageAnzeigen);
		if (btnKlDwblGesamtsch != null)
			btnKlDwblGesamtsch.setVisible(widerKlageAnzeigen);
		if (btnWiderklageAnzeigen != null)
			btnWiderklageAnzeigen.setVisible(!widerKlageAnzeigen);
	}

	/**
	 * Die Methode leert die Tabellen der BaumbachBeteiligten
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#leereWerte()
	 */
	@Override
	public void leereWerte() {
		// setWiderKlageAnzeigen(false);
		btmKlaeger = new BeteiligtenTableModel(Beteiligter.KLAEGER, isWiderKlageAnzeigen(),
				new BaumbachBeteiligtenListe());
		btmDrittwiderbeklagte = new BeteiligtenTableModel(Beteiligter.DRITTWIDERBEKLAGTE, isWiderKlageAnzeigen());
		btmBeklagte = new BeteiligtenTableModel(Beteiligter.BEKLAGTE, isWiderKlageAnzeigen(),
				new BaumbachBeteiligtenListe());
		if (tabKlaeger != null)
			tabKlaeger.setModel(btmKlaeger);
		if (tabBeklagte != null)
			tabBeklagte.setModel(btmBeklagte);
		if (tabDrittwiderbeklagte != null)
			tabDrittwiderbeklagte.setModel(btmDrittwiderbeklagte);
	}

	/**
	 * Die Methode setzt die DefaultRenderer für Beteiligten-JTables
	 * (BeteiligtenRenderer und EuroRenderer) und bestimmt, dass sie den Viewport
	 * ausfüllen müssen.
	 * 
	 * @param table JTable für Beteiligtendaten
	 * 
	 */
	private void setzeBeteiligtenRenderer(JTable table) {
		// table.setFillsViewportHeight(true);
		table.setDefaultRenderer(Beteiligter.class, BETEILIGTEN_RENDERER);
		JComboBox<Beteiligter> comboBox = new JComboBox<Beteiligter>(Beteiligter
				.getBeteiligtenAuswahlListe(((BeteiligtenTableModel) table.getModel()).getBeteiligtenTyp(), false));
		comboBox.setRenderer(BETEILIGTEN_RENDERER_OHNE_LFDNR);
		table.setDefaultEditor(Beteiligter.class, new DefaultCellEditor(comboBox));
		table.setDefaultRenderer(Double.class, EURO_RENDERER);
		table.setDefaultEditor(Double.class, EURO_EDITOR);
		table.setDefaultRenderer(Boolean.class, WIDERKLAGE_RENDERER_EDITOR);
		table.setDefaultEditor(Boolean.class, WIDERKLAGE_RENDERER_EDITOR);
		table.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		table.getTableHeader().setDefaultRenderer(HEADER_RENDERER);
	}

	/**
	 * Die Methode prüft, ob die Tabellen für die BaumbachBeteiligten auf Kläger-
	 * und auf Beklagtenseite Werte enthalten.
	 * 
	 * @return true, wenn btmKlaeger und btmBeklagte Beteiligte enthalten; false,
	 *         wenn auch nur eine der beiden leer ist.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.TabellenWertePruefend#pruefeTabellen()
	 */
	@Override
	public boolean pruefeTabellen() {
		return btmKlaeger.getRowCount() > 0 && btmBeklagte.getRowCount() > 0;
	}

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * Entfernt eine Zeile aus einer Beteiligten-Tabelle. Es wird zuvor geprüft, ob
	 * die entsprechende Tabelle bereits Gesamtschuldnerschaften enthält; dann ist
	 * ein Entfernen aus Gründen möglicher Verweise nicht möglicht. Es sei denn: die
	 * Gesamtschuldnerschaft selbst wird entfernt.
	 * 
	 * @param table die JTable, aus der die Zeile gelöscht werden soll
	 * @param model das BeteiligtenTableModel, aus der die Zeile gelöscht werden
	 *              soll.
	 * 
	 */
	public void entferneBeteiligtenEintrag(JTable table, BeteiligtenTableModel model) {
		if (model == null)
			return;
		if (model.getAllValues() != null && model.getAllValues().enthaeltGesamtschuldner()) {
			if (table.getSelectedRow() > -1 && !model.getRow(table.getSelectedRow()).isGesamtschuldnerschaft()) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_GESAMTSCHULD_NIX_LOESCHEN,
						new IllegalArgumentException(FEHLER_GESAMTSCHULD_NIX_LOESCHEN));
				return;
			}
		} else if (model.getBeteiligtenTyp() == Beteiligter.KLAEGER && (btmDrittwiderbeklagte.getAllValues() != null
				&& btmDrittwiderbeklagte.getAllValues().enthaeltGesamtschuldner())) {
			if (table.getSelectedRow() > -1 && !model.getRow(table.getSelectedRow()).isGesamtschuldnerschaft()) {
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, FEHLER_GESAMTSCHULD_NIX_LOESCHEN,
						new IllegalArgumentException(FEHLER_GESAMTSCHULD_NIX_LOESCHEN));
				return;
			}
		}
		((EintragEntfernenAktion) eintragEntfernenAktion).entferneTabellenEintrag(table, model);
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	/**
	 * @return gibt {@link #wirtschaftlichIdentisch} als boolean zurück.
	 */
	public boolean isWirtschaftlichIdentisch() {
		return wirtschaftlichIdentisch;
	}

	/**
	 * @param wirtschaftlichIdentisch d. {@link #wirtschaftlichIdentisch}, d.
	 *                                gesetzt werden soll als boolean
	 */
	public void setWirtschaftlichIdentisch(boolean wirtschaftlichIdentisch) {
		this.wirtschaftlichIdentisch = wirtschaftlichIdentisch;
	}

	/**
	 * @return gibt {@link #widerKlageAnzeigen} als boolean zurück.
	 */
	public boolean isWiderKlageAnzeigen() {
		return widerKlageAnzeigen;
	}

	/**
	 * Die Methode gibt das zu einem {@link Beteiligter#getTyp()} zugehörige
	 * {@link BeteiligtenTableModel} zurück.
	 * 
	 * @param typ {@link Beteiligter#KLAEGER}, {@link Beteiligter#BEKLAGTE} oder
	 *            {@link Beteiligter#DRITTWIDERBEKLAGTE}
	 * @return das dazu gehörige {@link BeteiligtenTableModel}
	 * 
	 * @see eu.gronos.kostenrechner.view.baumbach.TableModelGebend#getTableModel(int)
	 */
	@Override
	public BeteiligtenTableModel getTableModel(int typ) {
		switch (typ) {
		case Beteiligter.KLAEGER:
			return btmKlaeger;
		case Beteiligter.DRITTWIDERBEKLAGTE:
			return btmDrittwiderbeklagte;
		case Beteiligter.BEKLAGTE:
			return btmBeklagte;
		default:
			return null;
		}
	}
}
