/**
 * WiderklageHinzufuegenDialog.java
 * eu.gronos.kostenrechner.view.panel (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.baumbach;

import java.awt.GridBagConstraints;
import java.awt.HeadlessException;

import javax.swing.JCheckBox;
import javax.swing.JDialog;
import javax.swing.JFrame;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.model.baumbach.Widerklage;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;

/**
 * Ein kleiner HinzufuegenDialog, der auch nach wirtschaftlicher Identität fragt
 *
 * @author Peter Schuster (setrok)
 * @date 10.11.2018
 *
 */
public class WiderklageHinzufuegenDialog extends HinzufuegenDialog<Widerklage> {
	private static final long serialVersionUID = -9111863513392608271L;
	private JCheckBox chbWiderklage;
	private JCheckBox chbWirtschaftlichIdentisch;

	/**
	 * Konstruktor.
	 * 
	 * @param owner ein {@link JFrame}, im Zweifel ein Verweis auf
	 *              {@link Kostenrechner}, wird von {@link JDialog} für modale
	 *              Fenster gebraucht
	 * @param title der {@link String} für den Titel.
	 */
	public WiderklageHinzufuegenDialog(JFrame owner, String title) throws HeadlessException {
		super(owner, title);
	}

	/**
	 * Die Methode baut ein {@link Widerklage}-Objekt aus den Werten der beiden
	 * {@link JCheckBox}-Elemente des Dialogs
	 * 
	 * @return ein Objekt {@link Widerklage}
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Widerklage baueRueckgabewert() {
		// return new boolean[] { chbWiderklage.isSelected(),
		// chbWirtschaftlichIdentisch.isSelected() };
		return new Widerklage(chbWiderklage.isSelected(), chbWirtschaftlichIdentisch.isSelected());
	}

	@Override
	protected void fuelleContentPane() {
		chbWiderklage = new JCheckBox("Widerklage aktivieren", true);
		contentPanel.add(chbWiderklage,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		chbWirtschaftlichIdentisch = new JCheckBox("Widerklage wirtschaftlich identisch", false);
		chbWirtschaftlichIdentisch.setToolTipText("<html><b>§ 45 Abs. 1 Satz 3 GKG</b>"
				+ "<br>Betreffen die Ansprüche im Fall des Satzes 1 oder 2 (Klage und Widerklage, hilfsweise geltend gemachte Ansprüche)"
				+ "<br> denselben Gegenstand, ist nur der Wert des höheren Anspruchs maßgebend.</html>");
		contentPanel.add(chbWirtschaftlichIdentisch,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
	}

	@Override
	public boolean parseEingabe() {
		// Keine Plausibilitäten zu beachten diesmal
		return true;
	}

}