/**
 * ForderungHinzufuegen.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.forderungen;

import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;

import javax.swing.JComboBox;
import javax.swing.JFormattedTextField;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.model.forderungen.AufrechnungsForderung;
import eu.gronos.kostenrechner.model.forderungen.Forderung;
import eu.gronos.kostenrechner.model.forderungen.KlageForderung;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Ein HinzufügenDialog für Forderungen
 *
 * @author Peter Schuster (setrok)
 * @date 03.08.2014
 *
 */
public class ForderungHinzufuegen extends HinzufuegenDialog<Forderung> {

	private static final long serialVersionUID = 7212266450545525463L;

	private static final String TITLE = "Neues Angriffs-/Verteidigungsmittel";
	private JComboBox<Forderung> cmbForderungsart;
	private JFormattedTextField ftfStreitwert;
	private JFormattedTextField ftfUnterliegen;

	/**
	 * Konstruktor erzeugt den Dialog
	 * 
	 * @param owner
	 *            der aufrufende JFrame
	 */
	public ForderungHinzufuegen(JFrame owner) {
		super(owner, TITLE);
	}

	/**
	 * Die Methode dient dazu, aus den Eingabefeldern des Dialogs einen
	 * Rückgabewert zusammenzubauen, der dann von showDialog() an die aufrufende
	 * Maske zurückgegeben werden kann.
	 * 
	 * @return eine Forderung in der gewählten Höhe
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#baueRueckgabewert()
	 */
	@Override
	protected Forderung baueRueckgabewert() {
		Forderung forderung = (Forderung) cmbForderungsart.getSelectedItem();
		Double wert = Double.parseDouble(ftfStreitwert.getText());
		Double erfolg = Double.parseDouble(ftfUnterliegen.getText());
		forderung.setWert(wert);
		forderung.setErfolg(erfolg);
		return forderung;
	}

	/**
	 * Die Methode füllt das contentPanel mit einer JComboBox, je einem
	 * JFormattedTextField für Streitwert und Erfolg sowie dazugehörigen
	 * JLabels.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		JLabel lblForderungsart = new JLabel("Angriffs-/Verteidigungsmittel:",
				SwingConstants.RIGHT);
		contentPanel.add(lblForderungsart, GitterBeutelBeschraenkungen
				.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		cmbForderungsart = new JComboBox<Forderung>(sammleForderungen());
		cmbForderungsart.setRenderer(new ForderungenRenderer());
		setMnemonicLabelFor(lblForderungsart, cmbForderungsart, KeyEvent.VK_V);
		contentPanel.add(cmbForderungsart, GitterBeutelBeschraenkungen
				.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblWert = new JLabel("Höhe der Forderung:", SwingConstants.RIGHT);
		contentPanel.add(lblWert, GitterBeutelBeschraenkungen.getInstance(0, 1,
				1, 1, GridBagConstraints.HORIZONTAL, false));
		ftfStreitwert = new JFormattedTextField(formatter);
		ftfStreitwert.setText(Double.toString(0));
		setEnterAction(ftfStreitwert, getOkAction());
		setMnemonicLabelFor(lblWert, ftfStreitwert, KeyEvent.VK_H);
		contentPanel.add(ftfStreitwert, GitterBeutelBeschraenkungen
				.getInstance(1, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		JLabel lblErfolg = new JLabel("Davon begründet:", SwingConstants.RIGHT);
		contentPanel.add(lblErfolg, GitterBeutelBeschraenkungen.getInstance(0,
				2, 1, 1, GridBagConstraints.HORIZONTAL, false));
		ftfUnterliegen = new JFormattedTextField(formatter);
		ftfUnterliegen.setText(Double.toString(0));
		setEnterAction(ftfUnterliegen, getOkAction());
		setMnemonicLabelFor(lblErfolg, ftfUnterliegen, KeyEvent.VK_B);
		contentPanel.add(ftfUnterliegen, GitterBeutelBeschraenkungen
				.getInstance(1, 2, 1, 1, GridBagConstraints.HORIZONTAL, false));
	}

	/**
	 * Die Methode gibt ein Array von Forderungen für JComboBox
	 * 
	 * @return ein Array Forderung[] mit einer KlageForderung und einer
	 *         AufrechnungsForderun
	 */
	public static Forderung[] sammleForderungen() {
		return new Forderung[] { new KlageForderung(0, 0),
				new AufrechnungsForderung(0, 0) };
	}

	/**
	 * Die Methode wird von okAction aufgerufen, um zu prüfen, ob alle
	 * JComponents zur Eingabe mit gültigen Werten gefüllt sind und diese
	 * zueinander nicht im Widerspruch stehen.
	 * 
	 * @return true, wenn sich die Werte in den Feldern Höhe und Begründetheit
	 *         parsen lassen und der Erfolg nicht höher als der Wert ist.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#parseEingabe()
	 */
	@Override
	public boolean parseEingabe() {
		boolean zwischen = false;
		try {
			Double wert = Double.parseDouble(ftfStreitwert.getText());
			Double erfolg = Double.parseDouble(ftfUnterliegen.getText());
			if (erfolg > wert) {
				zwischen = false;
				FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
						HinzufuegenDialog.FEHLER_BEGRUENDETHEIT_GT_WERT,
						new IllegalArgumentException(
								HinzufuegenDialog.FEHLER_BEGRUENDETHEIT_GT_WERT));
			} else
				zwischen = true;
		} catch (NumberFormatException | NullPointerException e) {
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO,
					HinzufuegenDialog.FEHLER_ZAHLENFORMAT_WERT, e);
//			e.printStackTrace();
			zwischen = false;
		}
		return zwischen;
	}
}
