/**
 * GebuehrenPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.gebuehren;

import java.awt.GridBagConstraints;
import java.awt.event.FocusEvent;
import java.awt.event.KeyEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.KeyStroke;
import javax.swing.ListSelectionModel;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.AllzweckHorcher;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrNeuAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenAufstellungAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenSetzeRechnerTabelleAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenStreitwertHorcher;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenTabelleDruckenAction;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrenTabelleKopierenAction;
import eu.gronos.kostenrechner.interfaces.GebuehrenTableTextfieldHandler;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.gebuehren.AnwaltsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.gebuehren.GerichtsGebuehrenTabelle;
import eu.gronos.kostenrechner.model.gebuehren.StreitwertTableModel;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.GebuehrenBerechnung;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.KostenJPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Diese von {@link KostenJPanel} abgeleitete Klasse nimmt die
 * Oberflächenelemente für die Gebührenberechnung auf.
 *
 * @author Peter Schuster (setrok)
 * @date 13.06.2018
 *
 */
public class GebuehrenBerechnungPanel extends KostenJPanel
		implements WerteLesbar<VerfahrensDatenContainer.GebuehrenBerechnung>, GebuehrenTableTextfieldHandler
		/*PanelTableModelHandler<GebuehrenTatbestandTableModel>, PanelTextFieldHandler*/ {
	private static final String LBL_STREITWERT_TEXT = "Bitte geben Sie den Streitwert ein:";
	private static final String SUMME_EUR = "Summe: %,.2f EUR";
	private static final String STREITWERTE_POPUPLABEL = "Streitwerte...";
	private static final long serialVersionUID = -8251865293845494552L;
	private static final GebuehrenBerechnungPanel instance = new GebuehrenBerechnungPanel();
	private Kostenrechner kostenrechner;
	private JTable tabStreitwertTabelle;
	private JFormattedTextField ftfStreitwert;
	public JTable tabGebuehrenTatbestaende;
	private JTextField tfGebuehrenSumme;
	private JPopupMenu pmStreitwert;
	private JPopupMenu pmGttmRechner;
	public GebuehrenTatbestandTableModel gttmRechner;
	// new GebuehrenRvgVerguetungAction
	private final AbstractAction neueRvgVerguetungAction = new /* GebuehrenNeueGebuehrAction */GebuehrNeuAction(this,
			"Neue Vergütung nach RVG",
			"<html><h1>Neue Gebühr</h1><p>Fügt einen neuen Vergütungstatbestand nach RVG in die Tabelle für die Gebühren zur freien Berechnung ein.</p></html>",
			KeyEvent.VK_R, "gebuehr-rvg", TABLE_GEBUEHREN, AnwaltsGebuehrenTabelle.getAllGebuehrenTatbestaende());
	private final AbstractAction neueGkgGebuehrAction = new GebuehrNeuAction(this, "Neue Gebühr nach GKG",
			"<html><h1>Neue Gebühr</h1><p>Fügt einen neuen Gebührentatbestand nach GKG in die Tabelle für die Gebühren zur freien Berechnung ein.</p></html>",
			KeyEvent.VK_N, "gebuehr-gkg", TABLE_GEBUEHREN, GerichtsGebuehrenTabelle.getAllGebuehrenTatbestaende());
	private final AbstractAction gebuehrenAufstellungAction = new GebuehrenAufstellungAction(this,
			"Ausführliche Gebührenaufstellung erstellen",
			"<html><h1>Ausführliche Gebührenaufstellung erstellen</h1><p>Erstellt eine Aufstellung der Gebühren und ihrer Summe zu dem ausgewählten Streitwert.</p></html>",
			KeyEvent.VK_E, "gebuehren-aufstellung");
	public final AbstractAction normaleGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			"Gebühren im Normalfall", "<html><h1>Gebühren im Normalfall</h1><p>Leert die Tabelle mit den Gebührentatbeständen des Rechners und <br>füllt sie mit der Konstellation eines normalen erstinstanzlichen Verfahrens</p></html>",
			KeyEvent.VK_H,
			"normal-gebuehren", GebuehrenSetzeRechnerTabelleAction.GEBUEHREN_STANDARD);
	public final AbstractAction saeumnisGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			"Gebühren bei Säumnis", "<html><h1>Gebühren bei Säumnis</h1><p>Leert die Tabelle mit den Gebührentatbeständen des Rechners und <br>füllt sie mit der Konstellation Gebühren bei Säumnis</p></html>",
			KeyEvent.VK_S,
			"saeumnis-gebuehren", GebuehrenSetzeRechnerTabelleAction.GEBUEHREN_SAEUMNIS);
	public final AbstractAction anerkenntnisGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			"Gebühren bei Anerkenntnis", "<html><h1>Gebühren bei Anerkenntnis</h1><p>Leert die Tabelle mit den Gebührentatbeständen des Rechners und <br>füllt sie mit der Konstellation Gebühren bei Anerkenntnis</p></html>",
			KeyEvent.VK_G,
			"anerkenntnis-gebuehren", GebuehrenSetzeRechnerTabelleAction.GEBUEHREN_ANERKENNTNIS);
	private final AbstractAction mitInstanzGebuehrenAction = new GebuehrenSetzeRechnerTabelleAction(this,
			"Gebühren einschließlich Berufung", "<html><h1>Gebühren einschließlich Berufung</h1><p>Leert die Tabelle mit den Gebührentatbeständen des Rechners und <br>füllt sie mit der Konstellation eines normalen erst- und zweitinstanzlichen Verfahrens</p></html>",
			KeyEvent.VK_B,
			"mit-instanz-gebuehren", GebuehrenSetzeRechnerTabelleAction.GEBUEHREN_2INSTANZ);

	private final AllzweckHorcher aktualisierungsHorcher = new GebuehrenStreitwertHorcher(this, LBL_STREITWERT_TEXT);
	private final AbstractAction druckeSwTabelleAction = new GebuehrenTabelleDruckenAction(this, "Tabelle drucken",
			"<html><h1>Tabelle drucken</h1><p>Druckt den Inhalt der Streitwerttabelle</p></html>", KeyEvent.VK_D,
			"print-table", KeyStroke.getKeyStroke(KeyEvent.VK_P, Kostenrechner.ctrlMask));
	private final AbstractAction kopiereSwTabelleAction = new GebuehrenTabelleKopierenAction(this,
			"Streitwerttabelle kopieren",
			"<html><h1>Streitwerttabelle kopieren</h1><p>Kopiert den Inhalt der Streitwerttabelle in die Zwischenablage</p></html>",
			KeyEvent.VK_K, "copy-table", KeyStroke.getKeyStroke(KeyEvent.VK_C, Kostenrechner.ctrlMask));
	public static final String RECHNER_POPUPLABEL = "Gebührentatbestände...";
	public static final String FEHLER_DRUCKEN = "Fehler beim Drucken";
	public static final int FTF_STREITWERT = 5;
	public static final int TABLE_GEBUEHREN = 8;
	private static final int TAB_GEBUEHRENRECHNER = 3;
	private JLabel lblStreitwert;

	/**
	 * Der Konstruktor ruft nur die Oberklasse auf.
	 * 
	 */
	private GebuehrenBerechnungPanel() {
		super();
	}

	/**
	 * Das Panel ist als Singelton ausgestatltet. Diese Methode gibt die alleinige
	 * Instanz zurück.
	 * 
	 * @return gibt {@link #instance} als GebuehrenBerechnungPanel zurück.
	 */
	public static GebuehrenBerechnungPanel getInstance() {
		return instance;
	}

	/**
	 * Die Methode fuellt den GebuehrenRechnerPanel.
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		lblStreitwert = new JLabel(LBL_STREITWERT_TEXT, SwingConstants.RIGHT);
		setFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT, new JFormattedTextField(KostenJPanel.FORMATTER));
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT).setColumns(10);
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT).setText("0");
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT)
				.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT).setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwert.setDisplayedMnemonic(KeyEvent.VK_S);
		lblStreitwert.setDisplayedMnemonicIndex(20);
		lblStreitwert.setLabelFor(getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT));

		// JScrollpane mit Gebührentabelle, die sich mit dem ftf synchronisiert
		JLabel lblStreitwertTabelle = new JLabel("Oder wählen Sie einen Streitwert aus:");
		StreitwertTableModel swtm = new StreitwertTableModel();
		setTabStreitwertTabelle(new JTable(swtm));
		getTabStreitwertTabelle().setDefaultRenderer(Long.class, KostenJPanel.EURO_RENDERER);
		getTabStreitwertTabelle().setDefaultRenderer(Double.class, KostenJPanel.EURO_RENDERER);
		getTabStreitwertTabelle().getTableHeader().setDefaultRenderer(KostenJPanel.HEADER_RENDERER);
		getTabStreitwertTabelle().setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		entferneTastaturFalle(getTabStreitwertTabelle());
		JScrollPane scpStreitwertTabelle = new JScrollPane(getTabStreitwertTabelle());
		lblStreitwertTabelle.setDisplayedMnemonic(KeyEvent.VK_W);
		lblStreitwertTabelle.setLabelFor(scpStreitwertTabelle);

		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT).addFocusListener(aktualisierungsHorcher);
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT).addKeyListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addFocusListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addMouseListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().addKeyListener(aktualisierungsHorcher);
		getTabStreitwertTabelle().getSelectionModel().addListSelectionListener(aktualisierungsHorcher);

		// JScrollpane mit JTable mit gttm + JLabel
		JLabel lblGebuehrenTatbestaende = new JLabel("Bitte geben Sie die angefallenen Gebühren an:");
		tabGebuehrenTatbestaende = new JTable();
//		setzeGebuehrenRechnerTabelle(0);
		leereGebuehrenTabellen();
		entferneTastaturFalle(tabGebuehrenTatbestaende);
		setzeGebuehrenTabelleAussehen(tabGebuehrenTatbestaende);
		JScrollPane scpGebuehrenTatbestaende = new JScrollPane(tabGebuehrenTatbestaende);
		lblGebuehrenTatbestaende.setDisplayedMnemonic(KeyEvent.VK_G);
		lblGebuehrenTatbestaende.setDisplayedMnemonicIndex(33);
		lblGebuehrenTatbestaende.setLabelFor(scpGebuehrenTatbestaende);

		// JButton Gebühr hinzufügen
		JButton btnNeueVerguetung = new JButton(neueRvgVerguetungAction);
		JButton btnNeueGebuehr = new JButton(neueGkgGebuehrAction);

		// JTextField Summe berechnen
		tfGebuehrenSumme = new JTextField();
		tfGebuehrenSumme.setEditable(false);
		tfGebuehrenSumme.setHorizontalAlignment(SwingConstants.RIGHT);
		JButton btnSummeBerechnen = new JButton(gebuehrenAufstellungAction);

		errechneGebuehrenSumme();

		// Der JPanel hinzufügen
		add(lblStreitwert, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT),
				GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(lblStreitwertTabelle,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(scpStreitwertTabelle, GitterBeutelBeschraenkungen.getInstance(0, 2, 2, 1, GridBagConstraints.BOTH, true));
		add(lblGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(tfGebuehrenSumme,
				GitterBeutelBeschraenkungen.getInstance(1, 3, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(scpGebuehrenTatbestaende,
				GitterBeutelBeschraenkungen.getInstance(0, 4, 2, 1, GridBagConstraints.BOTH, true));
		add(btnNeueGebuehr, GitterBeutelBeschraenkungen.getInstance(0, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(btnNeueVerguetung,
				GitterBeutelBeschraenkungen.getInstance(1, 5, 1, 1, GridBagConstraints.HORIZONTAL, false));
		add(btnSummeBerechnen,
				GitterBeutelBeschraenkungen.getInstance(0, 6, 2, 1, GridBagConstraints.HORIZONTAL, false));

		// Kontextmenü für Streitwerttabelle
		pmStreitwert = new JPopupMenu(STREITWERTE_POPUPLABEL);
		pmStreitwert.add(new JMenuItem(STREITWERTE_POPUPLABEL));
		pmStreitwert.addSeparator();
		pmStreitwert.add(new JMenuItem(druckeSwTabelleAction));
		pmStreitwert.add(new JMenuItem(kopiereSwTabelleAction));
		getTabStreitwertTabelle().addMouseListener(new KontextMouseListener(getTabStreitwertTabelle(), pmStreitwert));

		// Kontextmenü für gttm
		pmGttmRechner = new JPopupMenu(RECHNER_POPUPLABEL);
		pmGttmRechner.add(new JMenuItem(RECHNER_POPUPLABEL));
		pmGttmRechner.addSeparator();
		pmGttmRechner.add(new JMenuItem(neueGkgGebuehrAction));
		pmGttmRechner.add(new JMenuItem(neueRvgVerguetungAction));
		pmGttmRechner.add(new JMenuItem(saeumnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(anerkenntnisGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(normaleGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(mitInstanzGebuehrenAction));
		pmGttmRechner.add(new JMenuItem(eintragEntfernenAktion));
		tabGebuehrenTatbestaende.addMouseListener(new KontextMouseListener(tabGebuehrenTatbestaende, pmGttmRechner));
	}

	/**
	 * Die Methode leert alle Tabellen und sonstige Oberflächenelemente. Damit kann
	 * man dann die Registerkarte initialisieren oder auf den Anfangszustand
	 * zurücksetzen.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leereWerte()
	 */
	@Override
	public void leereWerte() {
//		setzeGebuehrenRechnerTabelle(0);
		leereGebuehrenTabellen();
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle, füllt sie mit den Gebühren aus
	 * {@link eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.GebuehrenBerechnung}
	 * und setzt auch den Streitwert entsprechend.
	 * 
	 * @param container ein
	 *                  {@link eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.GebuehrenBerechnung}
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setzeWerte(java.lang.Object)
	 */
	public void setzeWerte(final VerfahrensDatenContainer.GebuehrenBerechnung container) {
		/* Erst den Streitwert eintragen und die Tabelle springen lassen */
		getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT)
				.setText(Long.toString((long) container.streitwert));
		aktualisierungsHorcher.focusLost(
				new FocusEvent(getFormattedTextField(GebuehrenBerechnungPanel.FTF_STREITWERT), FocusEvent.FOCUS_LOST));
		/* Dann die Tatbestände in die Gebührentabelle eintragen */
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(container.gebuehren);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(container.gebuehren);
		}
	}

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * Die Methode addiert die Gebühren der Tabelle.
	 * 
	 */
	public void errechneGebuehrenSumme() {
		// tfGebuehrenSumme.requestFocus();
		String streitwert = "";
		double gebuehrenSumme = 0.0;
		if (getFormattedTextField(FTF_STREITWERT) != null && pruefeTabellen()) {// gttmRechner != null
			try {
				streitwert = getFormattedTextField(FTF_STREITWERT).getText();
				gebuehrenSumme = gttmRechner.errechneGebuehrenSumme(
						longAusFormatter(getFormattedTextField(FTF_STREITWERT), getLabelText(FTF_STREITWERT))); // lblStreitwert.getText()Long.parseLong(streitwert)
			} catch (NumberFormatException e) {
				// e.printStackTrace();
				new FehlerDialog(FehlerDialog.FEHLER_BLANKO,
						"Falsches Eingabeformat für den Streitwert: \"" + streitwert + "\"!\n ", e).showDialog();
			}
		}
		String text = String.format(SUMME_EUR, gebuehrenSumme);
		if (tfGebuehrenSumme != null)
			tfGebuehrenSumme.setText(text);
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return den Untercontainer von
	 *         {@link VerfahrensDatenContainer#gebuehrenBerechnung}
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#leseWerte()
	 */
	@Override
	public GebuehrenBerechnung leseWerte() {
		GebuehrenBerechnung container = new GebuehrenBerechnung();
		container.streitwert = 0L;
		container.gebuehren = new ArrayList<GebuehrenTatbestand>();
		try {
			container.streitwert = longAusFormatter(getFormattedTextField(FTF_STREITWERT),
					getLabelText(FTF_STREITWERT));// lblStreitwert.getText());Long.parseLong(getFtfStreitwert().getText());
		} catch (NumberFormatException e) {
			new FehlerDialog(FehlerDialog.FEHLER_BLANKO, e.getLocalizedMessage(), e).showDialog();
		}
		container.gebuehren = gttmRechner.getAllValues();// toArray();

		return container;
	}

	/**
	 * @return gibt {@link #tabStreitwertTabelle} als JTable zurück.
	 */
	public JTable getTabStreitwertTabelle() {
		return tabStreitwertTabelle;
	}

	/**
	 * @param tabStreitwertTabelle d. {@link #tabStreitwertTabelle}, d. gesetzt
	 *                             werden soll als JTable
	 */
	public void setTabStreitwertTabelle(JTable tabStreitwertTabelle) {
		this.tabStreitwertTabelle = tabStreitwertTabelle;
	}

	/**
	 * Die Methode dient dazu, das passende {@link JFormattedTextField}
	 * zurückzuliefern
	 * 
	 * @param index ein Bezeichner für das {@link JFormattedTextField}, hier nur
	 *              {@link #TABLE_FTF_GEBUEHREN}
	 * @return gibt das gewählte {@link JFormattedTextField} zurück, hier nur
	 *         {@link #ftfStreitwert}.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTextFieldHandler#getFormattedTextField(int)
	 */
	@Override
	public JFormattedTextField getFormattedTextField(int index) {
		return ftfStreitwert;
	}

	/**
	 * Die Methode soll den Text des zum {@link JFormattedTextField} gehörenden
	 * {@link JLabel}s zurückgeben.
	 * 
	 * @param index ein Bezeichner für das {@link JFormattedTextField}, hier nur
	 *              {@link GebuehrenBerechnungPanel#FTF_STREITWERT}.
	 * @return {@link JLabel#getText()} als String
	 * @see eu.gronos.kostenrechner.interfaces.PanelTextFieldHandler#getLabelText(int)
	 */
	@Override
	public String getLabelText(int index) {
		if (lblStreitwert != null)
			return lblStreitwert.getText();
		else
			return "";
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	/**
	 * Die Methode dient dazu, das passende {@link GebuehrenTatbestandTableModel}
	 * zurückzuliefern
	 * 
	 * @param index ein Bezeichner für das {@link GebuehrenTatbestandTableModel},
	 *              hier nur {@link #TABLE_FTF_GEBUEHREN}
	 * @return das gewählte {@link GebuehrenTatbestandTableModel}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTableModelHandler#getTableModel(int)
	 */
	@Override
	public GebuehrenTatbestandTableModel getTableModel(int index) {
		if (index != TABLE_GEBUEHREN && Kostenrechner.debug) {
			IllegalArgumentException ie = new IllegalArgumentException(
					String.format("Fehler: index war nicht TABLE_GEBUEHREN, sondern: '%d'!", index));
			FehlerDialog.zeigeFehler(FehlerDialog.FEHLER_BLANKO, ie.getLocalizedMessage(), ie);
		}
		return gttmRechner;
	}

	/**
	 * Die Methode prüft, ob die Gebührentabelle Werte enthält.
	 * 
	 * @return true, wenn Gebührentatbestände in der Tabelle sind.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTableModelHandler#pruefeTabellen()
	 */
	@Override
	public boolean pruefeTabellen() {
		return getTableModel(TABLE_GEBUEHREN) != null && getTableModel(TABLE_GEBUEHREN).getRowCount() > 0;
	}

	/**
	 * Die Methode leert die GebuehrenRechnerTabelle oder füllt sie mit
	 * üblicherweise anfallenden Gebühren, lässt aber die Listener in Ruhe.
	 */
	private void leereGebuehrenTabellen() {
		List<GebuehrenTatbestand> values = new ArrayList<GebuehrenTatbestand>();
		if (gttmRechner == null) {
			gttmRechner = new GebuehrenTatbestandTableModel(values);
			gttmRechner.addTableModelListener(aktualisierungsHorcher);
			tabGebuehrenTatbestaende.setModel(gttmRechner);
		} else {
			gttmRechner.clear();
			gttmRechner.addAll(values);
		}
	
	}

	/**
	 * Die Methode dient dazu, dass eine {@link BeschriebeneAktion} den Focus auf
	 * die {@link JComponent} legen kann, die sie aufruft.
	 * 
	 * @return <code>true</code>, wenn der Aufruf wahrscheinlich Erfolg hat;
	 *         <code>false</code>, wenn er zum Scheitern verurteilt ist.
	 * 
	 * @see javax.swing.JComponent#requestFocusInWindow()
	 */
	@Override
	public boolean requestFocusInWindow() {
		getKostenrechner().getTabbedPane().setSelectedIndex(TAB_GEBUEHRENRECHNER);
		return super.requestFocusInWindow();
	}
	/**
	 * @param index         ein Bezeichner für das {@link JFormattedTextField}, hier
	 *                      nur {@link #TABLE_FTF_GEBUEHREN}
	 * @param ftfStreitwert d. {@link #ftfStreitwert}, d. gesetzt werden soll als
	 *                      JFormattedTextField
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTextFieldHandler#setFormattedTextField(int,
	 *      javax.swing.JFormattedTextField)
	 */
	private void setFormattedTextField(int index, JFormattedTextField ftfStreitwert) {
		this.ftfStreitwert = ftfStreitwert;
	}

}
