/**
 * StreitwertReduktionPanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.gebuehren;

import java.awt.GridBagConstraints;
import java.awt.event.ActionEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.JToggleButton;
import javax.swing.SwingConstants;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.BeschriebeneAktion;
import eu.gronos.kostenrechner.controller.KontextMouseListener;
import eu.gronos.kostenrechner.controller.gebuehren.GebuehrNeuAction;
import eu.gronos.kostenrechner.controller.gebuehren.StreitwertReduktionHorcher;
import eu.gronos.kostenrechner.controller.gebuehren.TeilklageMethodeAction;
import eu.gronos.kostenrechner.controller.gebuehren.TeilklageTenorAction;
import eu.gronos.kostenrechner.controller.gebuehren.TeilklageTerminAction;
import eu.gronos.kostenrechner.controller.gebuehren.TeilklageVerschiebenAction;
import eu.gronos.kostenrechner.controller.system.BooleanConfigOptionHorcher;
import eu.gronos.kostenrechner.interfaces.GebuehrenTableTextfieldHandler;
import eu.gronos.kostenrechner.interfaces.WerteLesbar;
import eu.gronos.kostenrechner.model.baumbach.BaumbachBeteiligter;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestand;
import eu.gronos.kostenrechner.model.gebuehren.GebuehrenTatbestandTableModel;
import eu.gronos.kostenrechner.model.tenordaten.Beteiligter;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer;
import eu.gronos.kostenrechner.model.tenordaten.VerfahrensDatenContainer.Teilklageruecknahme;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.KostenJPanel;
import eu.gronos.kostenrechner.view.baumbach.StreitgenossenPanel;
import eu.gronos.kostenrechner.view.helpanderror.FehlerDialog;

/**
 * Diese von {@link KostenJPanel} abgeleitete Klasse nimmt die
 * Oberflächenelemente für die Streitwertreduktion auf.
 *
 * @author Peter Schuster (setrok)
 * @date 13.06.2018
 *
 */
public class TeilklageruecknahmePanel extends KostenJPanel
		implements WerteLesbar<VerfahrensDatenContainer.Teilklageruecknahme>, GebuehrenTableTextfieldHandler {
	private static final long serialVersionUID = 1618328009895591401L;
	private static final TeilklageruecknahmePanel instance = new TeilklageruecknahmePanel();
	private Kostenrechner kostenrechner;
	JComboBox<Beteiligter> cbBeklagte;
	JComboBox<Beteiligter> cbKlaeger;
	private JFormattedTextField ftfStreitwertVorher;
	public JTable tabVorher;
	private GebuehrenTatbestandTableModel gttmVorher;
	private final AbstractAction neueGebuehrVorherAction = new GebuehrNeuAction(this,
			"Neue Gebühr (vor Teilklagerücknahme entstanden)",
			"<html><h1>Neue Gebühr (vor Teilklagerücknahme entstanden)</h1><p>Fügt einen vor der Teilklagerücknahme entstandenen Gebührentatbestand <br>in die (obere) Tabelle ein.</p></html>",
			KeyEvent.VK_V, "gebuehr-vorher", TABLE_FTF_VORHER);
	private final AbstractAction neueGebuehrNachherAction = new GebuehrNeuAction(this,
			"Neue Gebühr (nach Teilklagerücknahme entstanden)",
			"<html><h1>Neue Gebühr (nach Teilklagerücknahme entstanden)</h1><p>Fügt einen nach der Teilklagerücknahme entstandenen Gebührentatbestand <br>in die (untere) Tabelle ein.</p></html>",
			KeyEvent.VK_N, "gebuehr-nachher", TABLE_FTF_NACHHER);
	private final AbstractAction berechnungsMethodeAction = new TeilklageMethodeAction(
			"Berechnungsmethode: Quotenmethode", "Berechnungsmethode: Mehrkostenmethode",
			"<html><h1>Berechnungsmethode</h1><p>Auswahl, ob &bdquo;Tenor ermitteln&ldquo; nach der Quotenmethode (Standard) oder Mehrkostenmethode rechnet.</p></html>",
			KeyEvent.VK_M, "teilklage-berechnungsmethode");
	private final AbstractAction tenorErmittelnAction = new TeilklageTenorAction(this,
			"Tenor ermitteln (Quoten-/Mehrkostenmethode)",
			"<html><h1>Tenor ermitteln</h1><p>Erzeugt den Kostentenor nach der ausgew&auml;hlten Berechnungsmethode.</p></html>",
			KeyEvent.VK_Q, "teilklage-tenor");

	private JFormattedTextField ftfStreitwertNachher;
	private JFormattedTextField ftfObsiegen;

	private final FocusListener streitwertListener = new StreitwertReduktionHorcher(this);
	public JTable tabNachher;
	private GebuehrenTatbestandTableModel gttmNachher;
	private JPopupMenu pmGttmVorher;
	private JPopupMenu pmGttmNachher;
	public AbstractAction klageruecknahmeTerminAction = new TeilklageTerminAction(this,
			"Fall Teilklagerücknahme vor/im Termin",
			"Leert die Tabelle mit den Gebührentatbeständen und \nfüllt sie mit der Konstellation Teilklagerücknahme vor Anfall der Terminsgebühr",
			KeyEvent.VK_F, "standard-gebuehren");
	private final AbstractAction gebuehrenTbVerschiebenAction = new TeilklageVerschiebenAction(this,
			"Gebühr in die andere Tabelle verschieben",
			"Entfernt den markierten Eintrag aus dieser Tabelle und fügt ihn der jeweils anderen hinzu.", KeyEvent.VK_T,
			"verschiebe-gebuehr");
	public static final String FEHLER_KEINE_GEBUEHREN = "Fehler: Keine Gebühren angefallen!";
	public static final String FEHLER_KEINE_GEBUEHREN_LANG = "Bitte geben Sie zunächst ein welche Gebühren vor und/oder nach der Streitwertreduktion angefallen waren.";
	public static final String VORHER_POPUPLABEL = "Gebühren vorher...";
	public static final String NACHHER_POPUPLABEL = "Gebühren nachher...";
	public static final int TABLE_FTF_VORHER = 0;
	public static final int TABLE_FTF_NACHHER = 1;
	public static final int FTF_OBSIEGEN = 2;
	private static final int TAB_TEILKLAGE = 0;
	private JToggleButton btnBerechnungsMethode;
	private JLabel lblStreitwertVorReduktion;
	private JLabel lblStreitwertNachReduktion;
	private JLabel lblLetztlichObsiegt;

	/**
	 * Der Konstruktor ruft die Oberklasse auf.
	 * 
	 */
	private TeilklageruecknahmePanel() {
		super();
	}

	/**
	 * Das Panel ist als Singelton ausgestatltet. Diese Methode gibt die alleinige
	 * Instanz zurück.
	 * 
	 * @return gibt {@link #instance} als TeilklageruecknahmePanel zurück.
	 */
	public static TeilklageruecknahmePanel getInstance() {
		return instance;
	}

	/**
	 * Die Methode setzt die Oberflächenelemente für die Registerkarte 1
	 * Teilklagerücknahme
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		JLabel lblParteien = new JLabel("Parteien:", SwingConstants.RIGHT);
		lblParteien.setDisplayedMnemonic(KeyEvent.VK_P);
		add(lblParteien, GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] klBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.KLAEGER, true);
		cbKlaeger = new JComboBox<Beteiligter>(klBezeichnerListe);
		cbKlaeger.setRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER_OHNE_LFDNR);
		lblParteien.setLabelFor(cbKlaeger);
		add(cbKlaeger, GitterBeutelBeschraenkungen.getInstance(1, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		Beteiligter[] bkBezeichnerListe = BaumbachBeteiligter.getBeteiligtenAuswahlListe(Beteiligter.BEKLAGTE, true);
		cbBeklagte = new JComboBox<Beteiligter>(bkBezeichnerListe);
		cbBeklagte.setRenderer(StreitgenossenPanel.BETEILIGTEN_RENDERER_OHNE_LFDNR);
		add(cbBeklagte, GitterBeutelBeschraenkungen.getInstance(2, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));

		lblStreitwertVorReduktion = new JLabel("Streitwert vor Teilklagerücknahme", SwingConstants.RIGHT);
		add(lblStreitwertVorReduktion,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(0,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.HORIZONTAL, false));

		ftfStreitwertVorher = new JFormattedTextField(KostenJPanel.FORMATTER_INT);
		ftfStreitwertVorher.setColumns(10);
		ftfStreitwertVorher.addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		ftfStreitwertVorher.setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwertVorReduktion.setLabelFor(ftfStreitwertVorher);
		add(ftfStreitwertVorher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(2,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(1, 1, 2, 1, GridBagConstraints.HORIZONTAL, false));

		// leereWerte();//geht hier noch nicht, weil der auch die Textfelder braucht
		leereGebuehrenTabellen();

		tabVorher = new JTable(getGttmVorher());
		entferneTastaturFalle(tabVorher);
		setzeGebuehrenTabelleAussehen(tabVorher);
		JScrollPane scpVorher = new JScrollPane(tabVorher);
		add(scpVorher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(0,
				// 3, 3,
				// 1, true, false));
				GitterBeutelBeschraenkungen.getInstance(0, 2, 3, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueGebhrvorher = new JButton(neueGebuehrVorherAction);
		add(btnNeueGebhrvorher,
				GitterBeutelBeschraenkungen.getInstance(0, 3, 3, 1, GridBagConstraints.HORIZONTAL, false));

		lblStreitwertNachReduktion = new JLabel("Streitwert nach Teilklagerücknahme", SwingConstants.RIGHT);
		add(lblStreitwertNachReduktion,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(3,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(0, 4, 1, 1, GridBagConstraints.HORIZONTAL, false));

		setFtfStreitwertNachher(new JFormattedTextField(KostenJPanel.FORMATTER_INT));
		getFtfStreitwertNachher().setColumns(10);
		getFtfStreitwertNachher().addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		getFtfStreitwertNachher().addFocusListener(streitwertListener);
		getFtfStreitwertNachher().setHorizontalAlignment(JTextField.RIGHT);
		lblStreitwertNachReduktion.setLabelFor(getFtfStreitwertNachher());
		add(getFtfStreitwertNachher(),
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(5,
				// 2, 1,
				// 1, false, false));
				GitterBeutelBeschraenkungen.getInstance(1, 4, 2, 1, GridBagConstraints.HORIZONTAL, false));

		tabNachher = new JTable(getGttmNachher());
		entferneTastaturFalle(tabNachher);
		setzeGebuehrenTabelleAussehen(tabNachher);
		JScrollPane scpNachher = new JScrollPane(tabNachher);
		add(scpNachher,
				// GitterBeutelBeschraenkungen.getGitterBeutelBeschraenkungen(3,
				// 3, 3,
				// 1, true, false));
				GitterBeutelBeschraenkungen.getInstance(0, 5, 3, 1, GridBagConstraints.BOTH, false));

		JButton btnNeueGebhrnachher = new JButton(neueGebuehrNachherAction);
		add(btnNeueGebhrnachher,
				GitterBeutelBeschraenkungen.getInstance(0, 6, 3, 1, GridBagConstraints.HORIZONTAL, false));

		lblLetztlichObsiegt = new JLabel("Letztlich obsiegt:", SwingConstants.RIGHT);
		add(lblLetztlichObsiegt,
				GitterBeutelBeschraenkungen.getInstance(0, 8, 1, 1, GridBagConstraints.HORIZONTAL, false));

		setFtfObsiegen(new JFormattedTextField(KostenJPanel.FORMATTER_INT));
		getFtfObsiegen().addFocusListener(HinzufuegenDialog.BEI_FOCUS_MARKIEREN);
		getFtfObsiegen().setHorizontalAlignment(JTextField.RIGHT);
		lblLetztlichObsiegt.setLabelFor(getFtfObsiegen());
		add(getFtfObsiegen(),
				GitterBeutelBeschraenkungen.getInstance(1, 8, 2, 1, GridBagConstraints.HORIZONTAL, false));

		leereTextfelder();

		btnBerechnungsMethode = new JToggleButton(berechnungsMethodeAction);
		btnBerechnungsMethode
				.addItemListener(new BooleanConfigOptionHorcher("quotenMethodeVorziehen", true, btnBerechnungsMethode));
		// Einmal actionPerformed() auslösen, um den Text der Auswahl anzupassen
		berechnungsMethodeAction.actionPerformed(new ActionEvent(btnBerechnungsMethode, ActionEvent.ACTION_FIRST,
				btnBerechnungsMethode.getActionCommand()));
		add(btnBerechnungsMethode,
				GitterBeutelBeschraenkungen.getInstance(0, 9, 3, 1, GridBagConstraints.HORIZONTAL, false));

		JButton btnBerechneTeilklageruecknahme = new JButton(tenorErmittelnAction);// (quotenMethodeAction);
		add(btnBerechneTeilklageruecknahme,
				GitterBeutelBeschraenkungen.getInstance(0, 10, 3, 1, GridBagConstraints.HORIZONTAL, false));

		pmGttmVorher = new JPopupMenu(VORHER_POPUPLABEL);
		pmGttmVorher.add(new JMenuItem(VORHER_POPUPLABEL));
		pmGttmVorher.addSeparator();
		pmGttmVorher.add(new JMenuItem(neueGebuehrVorherAction));
		pmGttmVorher.add(new JMenuItem(klageruecknahmeTerminAction));
		pmGttmVorher.add(new JMenuItem(eintragEntfernenAktion));
		pmGttmVorher.add(new JMenuItem(gebuehrenTbVerschiebenAction));
		tabVorher.addMouseListener(new KontextMouseListener(tabVorher, pmGttmVorher));

		pmGttmNachher = new JPopupMenu(NACHHER_POPUPLABEL);
		pmGttmNachher.add(new JMenuItem(NACHHER_POPUPLABEL));
		pmGttmNachher.addSeparator();
		pmGttmNachher.add(new JMenuItem(neueGebuehrNachherAction));
		pmGttmNachher.add(new JMenuItem(klageruecknahmeTerminAction));
		pmGttmNachher.add(new JMenuItem(eintragEntfernenAktion));
		pmGttmNachher.add(new JMenuItem(gebuehrenTbVerschiebenAction));
		tabNachher.addMouseListener(new KontextMouseListener(tabNachher, pmGttmNachher));
	}

	/**
	 * Die Methode leert alle Tabellen und Textfelder der Streitwertreduktion
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.WerteLesbar#leereWerte()
	 */
	@Override
	public void leereWerte() {
		leereGebuehrenTabellen();// false
		leereTextfelder();
	}

	/**
	 * Die Methode leert alle Tabellen der Streitwertreduktion oder füllt sie mit
	 * den Tatbeständen zum aus dem
	 * {@link VerfahrensDatenContainer.Teilklageruecknahme}-Behälter. Zudem füllt
	 * sie die {@link JFormattedTextField} mit den Streitwertangaben.
	 * 
	 * @param container ein {@link VerfahrensDatenContainer.Teilklageruecknahme}
	 *                  -Behälter
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setzeWerte(java.lang.Object)
	 */
	@Override
	public void setzeWerte(final Teilklageruecknahme container) {
		cbKlaeger.setSelectedItem(container.parteien.get(0));
		cbBeklagte.setSelectedItem(container.parteien.get(1));
		ftfStreitwertVorher.setText(container.streitwerteUndObsiegen.get(0).toString());
		getFtfStreitwertNachher().setText(container.streitwerteUndObsiegen.get(1).toString());
		getFtfObsiegen().setText(container.streitwerteUndObsiegen.get(2).toString());
		setGttmVorher(new GebuehrenTatbestandTableModel(container.vorher));
		tabVorher.setModel(getGttmVorher());
		setGttmNachher(new GebuehrenTatbestandTableModel(container.nachher));
		tabNachher.setModel(getGttmNachher());
	}

	/**
	 * Die Methode dient dazu, alle Programmzustände in einen
	 * {@link VerfahrensDatenContainer} zu packen
	 * 
	 * @return den Untercontainer von
	 *         {@link VerfahrensDatenContainer#teilklageruecknahme}
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#leseWerte()
	 */
	@Override
	public Teilklageruecknahme leseWerte() {
		Teilklageruecknahme container = new Teilklageruecknahme();
		container.parteien.add((Beteiligter) cbKlaeger.getSelectedItem());
		container.parteien.add((Beteiligter) cbBeklagte.getSelectedItem());
		long[] vorherNachherLetztendlich = parseVorherNachherLetztendlich();
		container.streitwerteUndObsiegen.add(vorherNachherLetztendlich[0]);
		container.streitwerteUndObsiegen.add(vorherNachherLetztendlich[1]);
		container.streitwerteUndObsiegen.add(vorherNachherLetztendlich[2]);
		container.vorher = new ArrayList<GebuehrenTatbestand>();
		container.vorher = getGttmVorher().getAllValues();
		container.nachher = new ArrayList<GebuehrenTatbestand>();
		container.nachher = getGttmNachher().getAllValues();
		return container;
	}

	/**
	 * Die Methode dient dazu, eine Referenz auf das Hauptfenster zu übermitteln
	 * 
	 * @param kostenrechner
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * Die Methode prüft, ob wenigstens eine der beiden
	 * GebuehrenTatbestandTableModels auf dem Tab für die Teilerledigung/-rücknahme
	 * Daten enhtält.
	 * 
	 * @return true, wenn eine der beiden GebuehrenTatbestandTableModels Daten
	 *         enthält; false, wenn beide leer sind.
	 */
	public boolean pruefeTabellen() {
		return getGttmVorher().getRowCount() > 0 || getGttmNachher().getRowCount() > 0;
	}

	/**
	 * Die Methode gibt ein {@link JFormattedTextField} der Wahl zurück.
	 * 
	 * @param index entweder {@link #TABLE_FTF_VORHER}, {@link #TABLE_FTF_NACHHER}
	 *              oder {@link #FTF_OBSIEGEN}
	 * @return gibt das gewählte {@link JFormattedTextField} zurück.
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTextFieldHandler#getFormattedTextField(int)
	 */
	@Override
	public JFormattedTextField getFormattedTextField(int index) {
		switch (index) {
		case TABLE_FTF_VORHER:
			return this.ftfStreitwertVorher;
		case TABLE_FTF_NACHHER:
			return this.ftfStreitwertNachher;
		case FTF_OBSIEGEN:
			return this.ftfObsiegen;
		default:
			break;
		}
		return null;
	}

	/**
	 * Die Methode soll den Text des zum {@link JFormattedTextField} gehörenden
	 * {@link JLabel}s zurückgeben.
	 * 
	 * @param index entweder {@link #TABLE_FTF_VORHER}, {@link #TABLE_FTF_NACHHER}
	 *              oder {@link #FTF_OBSIEGEN}
	 * @return das {@link JLabel#getText()} als String
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTextFieldHandler#getLabelText(int)
	 */
	@Override
	public String getLabelText(int index) {
		switch (index) {
		case TABLE_FTF_VORHER:
			return this.lblStreitwertVorReduktion.getText();
		case TABLE_FTF_NACHHER:
			return this.lblStreitwertNachReduktion.getText();
		case FTF_OBSIEGEN:
			return this.lblLetztlichObsiegt.getText();
		default:
			break;
		}
		return null;
	}

	/**
	 * @param ftfStreitwertNachher d. {@link #ftfStreitwertNachher}, d. gesetzt
	 *                             werden soll als JFormattedTextField
	 */
	public void setFtfStreitwertNachher(JFormattedTextField ftfStreitwertNachher) {
		this.ftfStreitwertNachher = ftfStreitwertNachher;
	}

	/**
	 * @param ftfObsiegen d. {@link #ftfObsiegen}, d. gesetzt werden soll als
	 *                    JFormattedTextField
	 */
	public void setFtfObsiegen(JFormattedTextField ftfObsiegen) {
		this.ftfObsiegen = ftfObsiegen;
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	/**
	 * Die Methode gibt ein {@link GebuehrenTatbestandTableModel} der Wahl zurück.
	 * 
	 * @param index entweder {@link #TABLE_FTF_VORHER} oder
	 *              {@link #TABLE_FTF_NACHHER}
	 * @return {@link #getGttmVorher()} oder {@link #getGttmNachher()}
	 * 
	 * @see eu.gronos.kostenrechner.interfaces.PanelTableModelHandler#getTableModel(int)
	 */
	@Override
	public GebuehrenTatbestandTableModel getTableModel(int index) {
		if (index == TABLE_FTF_VORHER) {
			return getGttmVorher();
		} else if (index == TABLE_FTF_NACHHER) {
			return getGttmNachher();
		}
		throw new IndexOutOfBoundsException();
	}

	/**
	 * @return gibt {@link #gttmVorher} als GebuehrenTatbestandTableModel zurück.
	 */
	public GebuehrenTatbestandTableModel getGttmVorher() {
		return gttmVorher;
	}

	/**
	 * @return gibt {@link #gttmNachher} als GebuehrenTatbestandTableModel zurück.
	 */
	public GebuehrenTatbestandTableModel getGttmNachher() {
		return gttmNachher;
	}

	/**
	 * Die Methode gibt einen Fehler aus
	 * 
	 */
	public void fehlerKeineGebuehren() {
		Throwable e = new IllegalArgumentException(TeilklageruecknahmePanel.FEHLER_KEINE_GEBUEHREN_LANG);
		new FehlerDialog(TeilklageruecknahmePanel.FEHLER_KEINE_GEBUEHREN, e.getLocalizedMessage(), e).showDialog();
	}

	/**
	 * Die Methode übergibt die gewählte Berechnungsmethode
	 * 
	 * @return
	 */
	public boolean isQuotenMethodeVorziehen() {
		if (btnBerechnungsMethode == null)
			return true;
		return btnBerechnungsMethode.isSelected();
	}

	/**
	 * Die Methode dient dazu, dass eine {@link BeschriebeneAktion} den Focus auf
	 * die {@link JComponent} legen kann, die sie aufruft.
	 * 
	 * @return <code>true</code>, wenn der Aufruf wahrscheinlich Erfolg hat;
	 *         <code>false</code>, wenn er zum Scheitern verurteilt ist.
	 * 
	 * @see javax.swing.JComponent#requestFocusInWindow()
	 */
	@Override
	public boolean requestFocusInWindow() {
		getKostenrechner().getTabbedPane().setSelectedIndex(TAB_TEILKLAGE);
		return super.requestFocusInWindow();
	}

	/**
	 * @return gibt {@link #ftfObsiegen} als JFormattedTextField zurück.
	 */
	private JFormattedTextField getFtfObsiegen() {
		return ftfObsiegen;
	}

	/**
	 * @return gibt {@link #ftfStreitwertNachher} als JFormattedTextField zurück.
	 */
	private JFormattedTextField getFtfStreitwertNachher() {
		return ftfStreitwertNachher;
	}

	/**
	 * Die Methode leert alle Textfelder der Streitwertreduktion
	 * 
	 */
	private void leereTextfelder() {
		ftfStreitwertVorher.setText("0");// "15000"
		getFtfStreitwertNachher().setText("0");// "10000"
		getFtfObsiegen().setText("0");// "10000"
	}

	/**
	 * Die Methode leert alle Tabellen der Streitwertreduktion
	 * 
	 */
	private void leereGebuehrenTabellen() {
		setGttmVorher(new GebuehrenTatbestandTableModel(new ArrayList<GebuehrenTatbestand>()));
		setGttmNachher(new GebuehrenTatbestandTableModel(new ArrayList<GebuehrenTatbestand>()));
		if (tabVorher != null)
			tabVorher.setModel(getGttmVorher());
		if (tabNachher != null)
			tabNachher.setModel(getGttmNachher());
	}

	/**
	 * Die Methode parst die JFormattedTextFields für die Streitwertreduktion.
	 * 
	 * @return ein long[] mit den Werten in der Reihenfolge Streitwert vorher,
	 *         Streitwert nachher, letztlich obsiegt.
	 */
	private long[] parseVorherNachherLetztendlich() {
		long[] zwischen = new long[3];
		long vorher;
		long nachher;
		long letztendlich;
		try {
			vorher = longAusFormatter(getFormattedTextField(TABLE_FTF_VORHER), getLabelText(TABLE_FTF_VORHER));// ftfStreitwertVorher,lblStreitwertVorReduktion.getText()
			nachher = longAusFormatter(getFormattedTextField(TABLE_FTF_NACHHER), getLabelText(TABLE_FTF_NACHHER));// getFtfStreitwertNachher(),lblStreitwertNachReduktion.getText()
			letztendlich = longAusFormatter(getFormattedTextField(FTF_OBSIEGEN), getLabelText(FTF_OBSIEGEN));// getFtfObsiegen(),lblLetztlichObsiegt.getText()
		} catch (NumberFormatException nfe) {
			new FehlerDialog("Bitte Ganzzahlen eingeben!",
					"Konnte Eingabe nicht als Ganzzahl lesen: " + nfe.getLocalizedMessage(), nfe).showDialog();
			return new long[] { 0L, 0L, 0L };
		}
		zwischen = new long[] { vorher, nachher, letztendlich };
		return zwischen;
	}

	/**
	 * @param gttmNachher d. {@link #gttmNachher}, d. gesetzt werden soll als
	 *                    GebuehrenTatbestandTableModel
	 */
	private void setGttmNachher(GebuehrenTatbestandTableModel gttmNachher) {
		this.gttmNachher = gttmNachher;
	}

	/**
	 * @param gttmVorher d. {@link #gttmVorher}, d. gesetzt werden soll als
	 *                   GebuehrenTatbestandTableModel
	 */
	private void setGttmVorher(GebuehrenTatbestandTableModel gttmVorher) {
		this.gttmVorher = gttmVorher;
	}

}
