/**
 * FehlerDialog.java
 * eu.gronos.kostenrechner (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.Color;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.event.KeyEvent;
import java.util.logging.Level;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.system.FehlerHelper;
import eu.gronos.kostenrechner.controller.system.SchreibeEmailFehler;
import eu.gronos.kostenrechner.view.GitterBeutelBeschraenkungen;
import eu.gronos.kostenrechner.view.HinzufuegenDialog;
import eu.gronos.kostenrechner.view.RechnerhammerIcons;

/**
 * Eine Klasse, die Fehlermeldungen ausführlicher darstellt.
 *
 * @author Peter Schuster (setrok)
 * @date 26.08.2014
 *
 */
public class FehlerDialog extends HinzufuegenDialog<Boolean> {

	private static final long serialVersionUID = 306283812188663774L;
	private String meldung;
	private Throwable throwable;
	private final StackTraceElement stack;
	public static final String FEHLER_BLANKO = "Fehler";

	private final AbstractAction emailSchreibenFehler;

	/**
	 * Der Konstruktor initialisiert den Dialog und gibt eine StackTrace aus.
	 * 
	 * @param title     den Titel des Dialogs
	 * @param meldung   die Fehlermeldung
	 * @param throwable das {@link Throwable} (Oberklasse von {@link Exception}),
	 *                  das geworfen wurde.
	 */
	public FehlerDialog(String title, String meldung, Throwable throwable) {
		super(Kostenrechner.getInstance(), title);
		this.meldung = meldung;
		this.throwable = throwable;
		super.setOkButtonToBeFocussed(true);
		this.stack = throwable.getStackTrace()[0];
		String klasse = stack.getClassName();
		String methode = stack.getMethodName();
		Kostenrechner.getLogger().logp(Level.WARNING, klasse, methode, meldung, throwable);
		emailSchreibenFehler = new SchreibeEmailFehler("Beim Autor beschweren",
				"<html>Öffnet den Standard-E-Mail-Programm,<br>um eine E-Mail an den Programmierer zu verfassen</html>",
				KeyEvent.VK_E, "e-mail", this.throwable);
	}

	/**
	 * Die Methode füllt das contentPane mit einem Icon-Label, einer JTextArea für
	 * die eigentliche Meldung und einer JTextArea im JScrollPane für den
	 * StackTrace.
	 * 
	 * @see eu.gronos.kostenrechner.view.HinzufuegenDialog#fuelleContentPane()
	 */
	@Override
	protected void fuelleContentPane() {
		JPanel meldungPane = new JPanel();
		meldungPane.setLayout(new FlowLayout(FlowLayout.LEFT));

		JLabel lblIcon = new JLabel(RechnerhammerIcons.getRechnerhammerRot());
		JTextArea jtaMeldung = new JTextArea(meldung);
		jtaMeldung.setRows(2);
		jtaMeldung.setEditable(false);
		jtaMeldung.setOpaque(false);

		StringBuffer sb = new FehlerHelper(throwable).getFehlerText();
		JTextArea jtaStackTrace = new JTextArea();
		jtaStackTrace.setText(sb.toString());
		jtaStackTrace.setCaretPosition(0);
		jtaStackTrace.setForeground(Color.RED);
		jtaStackTrace.setEditable(false);
		jtaStackTrace.setRows(7);
		JScrollPane scpStackTrace = new JScrollPane(jtaStackTrace);
		scpStackTrace.setVisible(Kostenrechner.debug);

		meldungPane.add(lblIcon);
		meldungPane.add(jtaMeldung);
		contentPanel.add(meldungPane,
				GitterBeutelBeschraenkungen.getInstance(0, 0, 1, 1, GridBagConstraints.HORIZONTAL, false));
		contentPanel.add(scpStackTrace,
				GitterBeutelBeschraenkungen.getInstance(0, 1, 1, 1, GridBagConstraints.BOTH, true));

		getButtonPane().add(new JButton(emailSchreibenFehler));
		addToRootInputMap(emailSchreibenFehler);
	}

	@Override
	public boolean parseEingabe() {
		return true;
	}

	@Override
	protected Boolean baueRueckgabewert() {
		return Boolean.TRUE;
	}

	/**
	 * Die Methode zeigt über {@link FehlerDialog} einen Fehlertext an.
	 * 
	 * @param title     der Titel des Dialogs, standardmäßig: {@link FEHLER_BLANKO}
	 * @param meldung   den anzuzeigenden Fehlertext
	 * @param throwable das {@link Throwable} (Oberklasse von {@link Exception}),
	 *                  das geworfen wurde.
	 */
	public static void zeigeFehler(String title, String meldung, Throwable throwable) {
		if (Kostenrechner.getInstance() == null || !Kostenrechner.getInstance().isVisible()) {
			if (Kostenrechner.getLogger() == null) {
				System.err.println(meldung);
				throwable.printStackTrace();
			} else {
				StackTraceElement stack = throwable.getStackTrace()[0];
				String klasse = stack.getClassName();
				String methode = stack.getMethodName();
				Kostenrechner.getLogger().logp(Level.SEVERE, klasse, methode, meldung, throwable);
			}
		} else
			new FehlerDialog(title, meldung, throwable).showDialog();
	}
}
