/**
 * HilfePanel.java
 * eu.gronos.kostenrechner.view (Kostenrechner)
 */
package eu.gronos.kostenrechner.view.helpanderror;

import java.awt.BorderLayout;

import javax.swing.JScrollPane;
import javax.swing.JTextPane;
import javax.swing.text.StyledDocument;
import javax.swing.text.html.HTMLEditorKit;

import eu.gronos.kostenrechner.Kostenrechner;
import eu.gronos.kostenrechner.controller.SprungmarkenHorcher;
import eu.gronos.kostenrechner.controller.files.HtmlHelper;
import eu.gronos.kostenrechner.view.KostenJPanel;

/**
 * Registerkarte 4 Hilfe
 *
 * @author Peter Schuster (setrok)
 * @version
 * @date 22.06.2018
 *
 */
public class HilfePanel extends KostenJPanel {

	private static final long serialVersionUID = 6247368145017378263L;
	private static HilfePanel instance = new HilfePanel();
	private Kostenrechner kostenrechner;

	/**
	 * Konstruktor ist private, weil Singleton.
	 * 
	 * Besonderheit: BorderLayout statt GridBagLayout
	 * 
	 */
	private HilfePanel() {
		super();
		setLayout(new BorderLayout());
	}

	/**
	 * Die Methode setzt die Registerkarte 4 Hilfe mit einem einzigen JScrollPane im
	 * JPanel und darin einen JTextPane mit Hilfetext.
	 * 
	 * 
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#buildPanel()
	 */
	@Override
	public void buildPanel() {
		JTextPane textPane = new JTextPane();
		HTMLEditorKit kit = new HTMLEditorKit();
		textPane.setEditorKit(kit);
		textPane.setEditable(false);
		StyledDocument doc = (StyledDocument) kit.createDefaultDocument();
		textPane.setStyledDocument(doc);
		HtmlHelper helper = new HtmlHelper();
		helper.leseHtml("resources/hilfetext.html", doc, kit);
		helper.leseCss("resources/kostentenor.css", kit);
		// Caret ganz nach oben setzen
		textPane.setCaretPosition(0);
		textPane.addHyperlinkListener(new SprungmarkenHorcher());
		add(new JScrollPane(textPane), BorderLayout.CENTER);
	}

	/**
	 * @return gibt {@link #kostenrechner} als Kostenrechner zurück.
	 */
	public Kostenrechner getKostenrechner() {
		return kostenrechner;
	}

	/**
	 * @see eu.gronos.kostenrechner.view.KostenJPanel#setKostenrechner(eu.gronos.kostenrechner.Kostenrechner)
	 * 
	 */
	@Override
	public void setKostenrechner(Kostenrechner kostenrechner) {
		this.kostenrechner = kostenrechner;
	}

	/**
	 * @return gibt {@link #instance} als HilfePanel zurück.
	 */
	public static HilfePanel getInstance() {
		return instance;
	}
}
