<?xml version="1.0" encoding="UTF-8"?>
<!--
	Stylesheet zur Anzeige von elektronischen Empfangsbekenntnissen auf Basis des XJustiz-Standards 2.1
	erstellt von der XJustiz-Pflegestelle
	Version 1.3 (Februar 2019): Quickfix: "k.A." aus Dokumententabelle entfernt
-->

<xsl:stylesheet version="1.0" 
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	xmlns:xjus="http://www.xjustiz.de"
	xmlns:xdo="http://www.xdomea.de/V2.2.0" >
	
	<xsl:variable name="codelistenAllgemein" select="document('xsd/xjustiz_0010_cl_allgemein_2_1.xsd')"/>
	<xsl:variable name="codelistGerichte" select="document('xsd/xjustiz_0020_cl_gerichte_2_1.xsd')"/>
	<xsl:variable name="codelistStoerungsIDs" select="document('xsd/xjustiz_2210_cl_eeb_2_0.xsd')"/>
	<xsl:variable name="empfangsbestaetigung" select="document('xjustiz_nachricht.xml')"/>
		
	<xsl:output method="html" version="1.0" encoding="UTF-8" indent="yes"/>

<!-- ________________________________________________________________________________________________ 
		Startvorlage zum Aufbau des Grundgerüsts der HTML-Seite -->
	
	<xsl:template match="/">
		<html>
			<head>
				<title>Elektronisches Empfangsbekenntnis</title>
				<style>
					h1 {	
						font-family:	Arial;
						color: 			black;
						text-align:		center;
					   }
					h2 {
						font-family:	Arial;
					   }
					p  {
						font-family:	Arial;
						color:			black;
						text-align:		left;
					   }
					   
					.hinweis {
						font-family:	Arial;
						color:			red;
						font-weight:	bold;
						text-align:		left;
					   }
					   
					.rahmen  {
						font-family:	Arial;
						color:			black;
						text-align:		left;
						border:			2px solid;
						padding:		10px;
					   }
					
					.fett {
						font-family:	Arial;
						color:			black;
						font-weight:	bold;
						text-align:		left;
					   }
					.rot {
					    color:			red;
					}
					table {
						font-family:	Arial;
						color:			black;
						width:			100%;
						border:			2px solid black;
						border-collapse: collapse;
					   }
					th {
					    background-color: #DDD;
					    padding: 		5px;
					    border:			1px solid black;
					}
					td {
						font-family:	Arial;
						color:			black;
						text-align:		left;
						border:			1px solid black;
						border-spacing:	10px;
						vertical-align:	top;
						padding: 		5px;
					   }
				</style>
			</head>
			<body>
				<xsl:choose>
					
					<!-- Prüfung 1: Passen die XML-Dokumente zueinander? -->
					<xsl:when test="//xjus:eigene_Nachrichten_ID != $empfangsbestaetigung//xjus:fremde_Nachrichten_ID">
						<h2>FEHLER: Das rücklaufende elektronische Empfangsbekenntnis bezieht sich nicht auf das an den Zustellungsempfänger übersandte elektronische Empfangsbekenntnis.</h2>
						<p class="fett">Das Empfangsbekenntnis kann nicht verifiziert werden.</p>
						<p>ID des übersandten eEB:</p>
						<p class="rahmen"><xsl:value-of select="//xjus:eigene_Nachrichten_ID"/></p>
						<p>ID des rücklaufenden eEB:</p>
						<p class="rahmen"><xsl:value-of select="$empfangsbestaetigung//xjus:fremde_Nachrichten_ID"/></p>
					</xsl:when>
					<xsl:when test="not($empfangsbestaetigung)">
						<p class="hinweis">Hinweis: Nachfolgend wird nur die Aufforderung zur Abgabe eines Empfangsbekenntnisses dargestellt.</p>
						<h1>Empfangsbekenntnis</h1>
						<xsl:apply-templates select="xjus:nachricht.gds.uebermittlung_schriftgutobjekte.0005005" />
					</xsl:when>
					<xsl:otherwise>
						<!-- Prüfung 2: Wird die richtige Schema-Version verwendet? -->
						<xsl:call-template name="versionspruefung"/>
						<h1>Empfangsbekenntnis</h1>
						<xsl:apply-templates select="xjus:nachricht.gds.uebermittlung_schriftgutobjekte.0005005" />
					</xsl:otherwise>
				</xsl:choose>
			</body>
		</html>
	</xsl:template>

<!-- ________________________________________________________________________________________________ 
		Vorlagen zur Verarbeitung des XJustiz-Istanzdokuments -->
	
	<xsl:template match="xjus:nachricht.gds.uebermittlung_schriftgutobjekte.0005005">
		<p class="fett">Geschäftszeichen</p>
		<p><xsl:value-of select="//xjus:nachrichtenkopf/xjus:aktenzeichen.absender"/></p>
		<p><xsl:apply-templates select="//xjus:absender_Gericht|//xjus:absender_Sonstige|//xjus:absender_RVTraeger"/></p><br/>
		
		<p>In Sachen </p>
		<p><xsl:value-of select="//xjus:kurzrubrum[1]"/></p>
		<xsl:choose>
			<xsl:when test="$empfangsbestaetigung != false() and not($empfangsbestaetigung//xjus:empfangsbestaetigung)">
				<p>ist mir eine Aufforderung zur Abgabe des Empfangsbekenntnisses für die Entgegennahme des/der elektronischen Dokumente(s)</p>				
			</xsl:when>
			<xsl:otherwise>
				<p>bin ich zur Entgegennahme legitimiert und habe heute als elektronische(s) Dokument(e) erhalten:</p>				
			</xsl:otherwise>
		</xsl:choose>
		
		<table>
			<tr>
				<th>Typ</th>
				<th>Dokumentendatum</th>
				<th>Anzeigename</th>
			</tr>
			<xsl:for-each select="./xjus:akte">
				<xsl:call-template name="aktendaten"/>
			</xsl:for-each>
			<xsl:for-each select="./xjus:teilakte">
				<xsl:call-template name="teilaktendaten"/>
			</xsl:for-each>
			<xsl:for-each select="./xjus:dokument">
				<xsl:call-template name="dokumentdaten"/>
			</xsl:for-each>
		</table>	

		<xsl:if test="$empfangsbestaetigung != false() and not($empfangsbestaetigung//xjus:empfangsbestaetigung)">
			<p>übermittelt worden</p>				
		</xsl:if>

		<!-- Prüfung: Wurde der Empfang bestätigt? -->
		<xsl:choose>
			<xsl:when test="not($empfangsbestaetigung)">
				
			</xsl:when>
			<xsl:when test="not($empfangsbestaetigung//xjus:empfangsbestaetigung)">
				<p class="fett rot">Das Empfangsbekenntnis wird nicht abgegeben, da</p>
				<p class="rahmen">					
					<xsl:call-template name="sucheStoerungsgrund">
						<xsl:with-param name="StoerungsID" select="$empfangsbestaetigung//xjus:stoerungs_ID/code"/>
					</xsl:call-template>
					<xsl:if test="$empfangsbestaetigung//xjus:stoerungsgrund">
						<br/>
						<br/>
						<span class="fett">Erläuterung:</span><br/>
						<xsl:value-of select="$empfangsbestaetigung//xjus:stoerungsgrund"/>
					</xsl:if>
				</p>
			</xsl:when>
			<xsl:when test="$empfangsbestaetigung != false() and $empfangsbestaetigung//xjus:empfangsbestaetigung">
				<p class="fett">Datum</p>
				<p>
					<xsl:call-template name="formatDate">
						<xsl:with-param name="date"><xsl:value-of select="$empfangsbestaetigung//xjus:empfangsbestaetigung"/></xsl:with-param>
					</xsl:call-template>	
				</p>
			</xsl:when>	
		</xsl:choose>
		<br/>

		<p class="fett">Zustellungsempfänger oder Zustellungsempfängerin</p>
		<xsl:if test="$empfangsbestaetigung//xjus:zustellungsempfaenger_abweichend = 'true'">
			<p>
			<xsl:apply-templates select="$empfangsbestaetigung//xjus:absender_Gericht|$empfangsbestaetigung//xjus:absender_Sonstige|$empfangsbestaetigung//xjus:absender_RVTraeger"/></p>
			<p>als Vertreter oder Vertreterin von</p>
		</xsl:if>
		<p>
			<xsl:apply-templates select="//xjus:empfaenger_Gericht|//xjus:empfaenger_Sonstige|//xjus:empfaenger_RVTraeger"/>
		</p>

	</xsl:template>
	
	
	<!-- Auslesen des Absenders 
	<xsl:template match="xjus:absender_Gericht">
		<xsl:call-template name="sucheBehoerde">
			<xsl:with-param name="xjustizID" select="./code" />
		</xsl:call-template>
	</xsl:template>
	
	<xsl:template match="xjus:absender_Sonstige">
		<xsl:value-of select="."/>
	</xsl:template>
	-->
	<!-- CR 18/003: als möglichen Absender das Element 'absender_RVTraeger' ergänzen -->
	<xsl:template match="xjus:absender_Gericht|xjus:absender_Sonstige|xjus:absender_RVTraeger">
		<xsl:choose>
			<xsl:when test="local-name(.) = 'absender_Gericht'">
				<xsl:call-template name="sucheBehoerde">
					<xsl:with-param name="xjustizID" select="./code" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="local-name(.) = 'absender_RVTraeger'">
				<xsl:call-template name="sucheKlarname">
					<xsl:with-param name="codeliste" select="'Code.GDS.RVTraeger'"/>
					<xsl:with-param name="code" select="./code"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<!-- Auslesen des Empfängers -->
	<xsl:template match="xjus:empfaenger_Gericht|xjus:empfaenger_Sonstige|xjus:empfaenger_RVTraeger">
		<xsl:choose>
			<xsl:when test="local-name(.) = 'empfaenger_Gericht'">
				<xsl:call-template name="sucheBehoerde">
					<xsl:with-param name="xjustizID" select="./code" />
				</xsl:call-template>
			</xsl:when>
			<xsl:when test="local-name(.) = 'empfaenger_RVTraeger'">
				<xsl:call-template name="sucheKlarname">
					<xsl:with-param name="codeliste" select="'Code.GDS.RVTraeger'"/>
					<xsl:with-param name="code" select="./code"/>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>
	</xsl:template>
	
	
	<xsl:template name="aktendaten">
		<!-- CR 18/008: Bugfix Stylesheet eEB: Prüfung der eEB-Anforderung  -->
		<!-- <xsl:if test="xjus:ruecksendung_EEB_erforderlich = 'true'">  -->
		<xsl:if test="(xjus:ruecksendung_EEB_erforderlich = 'true') or (xjus:ruecksendung_EEB_erforderlich = 1)">
			<tr>
				<td>
					<xsl:call-template name="sucheKlarname">
						<xsl:with-param name="codeliste" select="'Code.GDS.Aktentyp'"/>
						<xsl:with-param name="code" select="xjus:aktentyp/code"/>
					</xsl:call-template>
				</td>
				<td>--</td>
				<xsl:choose>
					<xsl:when test="xjus:anzeigename">
						<td><xsl:value-of select="xjus:anzeigename" /></td>
					</xsl:when>
					<xsl:otherwise>
						<td> </td>
					</xsl:otherwise>
				</xsl:choose>			
			</tr>
		</xsl:if>
	</xsl:template>
	
	
	<xsl:template name="teilaktendaten">
		<!-- CR 18/008: Bugfix Stylesheet eEB: Prüfung der eEB-Anforderung  -->		
		<xsl:if test="(xjus:ruecksendung_EEB_erforderlich = 'true') or (xjus:ruecksendung_EEB_erforderlich = 1)">
			<tr>
				<td>
					<xsl:call-template name="sucheKlarname">
						<xsl:with-param name="codeliste" select="'Code.GDS.Teilaktentyp'"/>
						<xsl:with-param name="code" select="xjus:teilaktentyp/code"/>
					</xsl:call-template>
				</td>
				<td>--</td>
				<xsl:choose>
					<xsl:when test="xjus:anzeigename">
						<td><xsl:value-of select="xjus:anzeigename" /></td>
					</xsl:when>
					<xsl:otherwise>
						<td> </td>
					</xsl:otherwise>
				</xsl:choose>
			</tr>
		</xsl:if>
	</xsl:template>
	
	
	<xsl:template name="dokumentdaten">
		<!-- CR 18/008: Bugfix Stylesheet eEB: Prüfung der eEB-Anforderung  -->		
		<xsl:if test="(xjus:ruecksendung_EEB_erforderlich = 'true') or (xjus:ruecksendung_EEB_erforderlich = 1)">
			<tr>
				<td>
					<xsl:call-template name="sucheKlarname">
						<xsl:with-param name="codeliste" select="'Code.GDS.Dokumenttyp'"/>
						<xsl:with-param name="code" select="xjus:dokumententyp/code"/>
					</xsl:call-template>
				</td>
				<xsl:choose>
					<xsl:when test="xjus:dokumentendatum">
						<td>
							<xsl:call-template name="formatDate">
								<xsl:with-param name="date"><xsl:value-of select="xjus:dokumentendatum" /></xsl:with-param>
							</xsl:call-template>
						</td>
					</xsl:when>
					<xsl:otherwise>
						<td> </td>
					</xsl:otherwise>
				</xsl:choose>
				<xsl:choose>
					<xsl:when test="xjus:anzeigename">
						<td><xsl:value-of select="xjus:anzeigename" /></td>
					</xsl:when>
					<xsl:otherwise>
						<td> </td>
					</xsl:otherwise>
				</xsl:choose>
			</tr>
		</xsl:if>
	</xsl:template>

	
	
	<xsl:template name="formatDate">
		<xsl:param name="date" />
		<xsl:variable name="year" select="substring-before($date, '-')" />
		<xsl:variable name="month" select="substring-before(substring-after($date, '-'), '-')" />
		<xsl:variable name="day" select="substring(substring-after(substring-after($date, '-'), '-'), 1, 2)" />
		<xsl:value-of select="concat($day, '.', $month, '.', $year)" />
	</xsl:template>
	

<!-- ________________________________________________________________________________________________
		Prüfung, ob richtige XJustiz-Versionsnummern vorliegen -->
	
	<xsl:template name="versionspruefung">
		<xsl:if test="not(number(substring(//xjus:grunddaten/@xjustizVersion, 1, 1)) >= 2 and number(substring(//xjus:grunddaten/@xjustizVersion, 3, 1)) >= 1)">
			<h2 class="hinweis">Wichtiger Hinweis: </h2>
			<p class="hinweis">Die XJustiz-Versionsnummer eines Instanzdokuments wird nicht unterstützt. <br/>Möglicherweise wird der Inhalt nicht korrekt dagestellt!</p>
		</xsl:if>
	</xsl:template>
	
<!-- ________________________________________________________________________________________________
		Vorlagen für die Suche in XJustiz-Codelisten -->
	
	<xsl:template name="sucheBehoerde">
		<xsl:param name="xjustizID" />
		<xsl:value-of select="$codelistGerichte//xs:enumeration[@value=$xjustizID]/xs:annotation/xs:appinfo/codeName"/>
<!--
		<xsl:for-each select="$codelistGerichte//xs:enumeration">
			<xsl:if test="./@value = $xjustizID">
				<xsl:value-of select="./xs:annotation/xs:appinfo/codeName"/>
			</xsl:if>
		</xsl:for-each>
-->
	</xsl:template>
	
	
	<!-- Suchen des Klartextes für die Störungs-ID -->
	<xsl:template name="sucheStoerungsgrund">
		<xsl:param name="StoerungsID" />
		<xsl:value-of select="$codelistStoerungsIDs//xs:enumeration[@value=$StoerungsID]/xs:annotation/xs:appinfo/codeName"/>
	</xsl:template>	
	
	
	<!-- Suchen des Klarnamens für die Codes der Codelisten Aktentyp, Dokumenttyp, RVTraeger und Sachgebiet -->
	<xsl:template name="sucheKlarname">
		<xsl:param name="codeliste"/>
		<xsl:param name="code" />
		<xsl:variable name="aktuelleListe" select="$codelistenAllgemein//xs:complexType[@name=$codeliste]" />
		<xsl:value-of select="$aktuelleListe//xs:enumeration[@value=$code]"/>
	</xsl:template>
	
</xsl:stylesheet>
